/************************************************************************/
/* astrogtk.c                                                           */
/*                                                                      */
/* Main routine for EXAMPLE CLOCK GTK client.                           */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : May   2006   		                                */
/************************************************************************/

#include "astrogtk.h"
#include "../../cliserv/include/chara_messages.h"
#include "../charalib/chara.h"

/* Globals */

int server = -1;
gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer data );
gint background_code(gpointer data);
gint use_library_callback(GtkButton *button, gpointer data);
int message_star_data(int server, struct smessage *mess);
gint star_data_callback(GtkButton *button, gpointer data);
int message_uv_data(int server, struct smessage *mess);
gint uv_data_callback(GtkButton *button, gpointer data);

int test_config_callback(void)
{
	create_message_window("I've been called.");
	return TRUE;
}

int main(int  argc, char *argv[] )
{
	GtkWidget *window;
	GtkWidget *button;
	GtkWidget *vbox;
	struct astro_control_window_data cont_info;
	struct astro_init_window_data init_info;

	/* Let GTK parse command line */

	gtk_init (&argc, &argv);

	/* Initialise mesage jobs */

	init_jobs();

	/* Attempt to open the socket */

	if (argc == 1)
	{
		if ((server = open_server_by_name("astroui")) == -1)
		{
			fprintf(stderr,"Failed to open process astroui\n");
			exit(-2);
		}
	}
	else
	{
		if ((server = open_server_by_name(argv[1])) == -1)
		{
			fprintf(stderr,"Failed to open process %s\n",argv[1]);
			exit(-2);
		}
	}

	/* Here are the standard GTK messages */

        if (!add_message_job(server, NOOP, message_noop)) return FALSE;
        if (!add_message_job(server, MESSAGE, message_message)) return FALSE;
        if (!add_message_job(server, PING, message_ping)) return FALSE;
        if (!add_message_job(server, SERVER_STOPPING, message_server_stopping))
                return FALSE;
        if (!add_message_job(server, ASTROMOD_STAR_DATA, message_star_data))
		return FALSE;
        if (!add_message_job(server, ASTROMOD_UV_DATA, message_uv_data))
		return FALSE;

        /* Setup up astro message servers */

        if (!setup_astro_messages(server)) 
	{
		fprintf(stderr,"Could not setup astro messages.");
		exit(-7);
	}

	/* Setup the callback routine */

	set_config_callback(test_config_callback);

	/* Tell system what our name is */

	set_client_name("astroui");

	/* Create a new window */

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	/* Set name of window */

	gtk_window_set_title (GTK_WINDOW (window), "ASTRO");

	/* Set a handler for delete_event that immediately exits GTK. */

	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			GTK_SIGNAL_FUNC (delete_event), NULL);

	/* We create a box to pack widgets into. */

	vbox = gtk_vbox_new(FALSE, 0);

	/* Put the box into the main window. */

	gtk_container_add (GTK_CONTAINER (window), vbox);

	/* Creates a ping button */

	button = gtk_button_new_with_label ("PING");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (ping_callback), (gpointer) &server);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Creates a astro control button */

	cont_info.name = "ASTROGTK";
	cont_info.server = server;
	button = gtk_button_new_with_label ("SET STAR");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (create_astrogtk_control_window), 
		(gpointer) &cont_info);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Creates a astro init button */

	init_info.name = "ASTROGTK";
	init_info.server = server;
	init_info.get_config = TRUE;
	button = gtk_button_new_with_label ("CONFIGURATION");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (create_astrogtk_init_window), 
		(gpointer) &init_info);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Add a USE LIBRARY buttont */

	button = gtk_button_new_with_label ("USE LIBRARY");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (use_library_callback), (gpointer)P_TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Add a USE PROGRAM buttont */

	button = gtk_button_new_with_label ("USE PROGRAM");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (use_library_callback), (gpointer)P_FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Add a STAR DATA buttont */

	button = gtk_button_new_with_label ("STAR DATA");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (star_data_callback), (gpointer)0);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Add a UV DATA buttont */

	button = gtk_button_new_with_label ("UV DATA");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (uv_data_callback), (gpointer)0);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* Add a quit buttont */

	button = gtk_button_new_with_label ("QUIT");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (delete_event), NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* OK, show the box */

	gtk_widget_show(vbox);

	/* and the window */

	gtk_widget_show (window);

	/* Now start up the background process. */

        gtk_timeout_add(50, background_code, 0);

	/* Rest in gtk_main and wait for the fun to begin! */

	gtk_main ();

	return(0);
}

/************************************************************************/
/* delete_event()							*/
/*									*/
/* Callback function for program exit.					*/
/************************************************************************/

gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
	gtk_main_quit();
	close_server_socket(server);
	return(FALSE);

} /* delete_event() */

/************************************************************************/
/* background_code()							*/
/*									*/
/* Background code, deals with incomming messages.			*/
/************************************************************************/

gint background_code(gpointer data)
{
	process_server_socket(server);
	return TRUE;
}

/************************************************************************/
/* use_library_callback()  	                                        */
/*                                                                      */
/* Set global in the server.                                            */
/************************************************************************/

gint use_library_callback(GtkButton *button, gpointer data)
{
	struct  smessage mess;

	mess.type = ASTROMOD_SET_USE_ASTROMOD_LIB;
	mess.data = (unsigned char *)data;
	mess.length = sizeof(int);

	if (!send_message(server, &mess))
	{
	    create_message_window(
		    "Failed to send ASTROMOD_SET_USE_ASTROMOD_LIB message.");
        }

	return 0;

} /* use_library_callback() */

/************************************************************************/
/* message_star_data()							*/
/************************************************************************/

int message_star_data(int server, struct smessage *mess)
{
        struct s_star  *data;
	char	s[8000];

        /* Make sure data is right length */

        if (mess->length != sizeof(*data))
        {
            create_message_window(
            "Recieved ASTROMOD_INIT message with wrong amount of data.");
        }

	data = (struct s_star *)mess->data;

	sprintf(s,
	   "Star data received\nName = %s\nNumber = %d\nRa = %lf\nType = %s",
		data->name, data->chara_number, data->ra, data->spectype);

 	create_message_window(s);
	
	return NOERROR;

} /* message_star_data() */

/************************************************************************/
/* star_data_callback()  	                                        */
/*                                                                      */
/* Set global in the server.                                            */
/************************************************************************/

gint star_data_callback(GtkButton *button, gpointer data)
{
	struct  smessage mess;

	mess.type = ASTROMOD_STAR_DATA;
	mess.data = NULL;
	mess.length = 0;

	if (!send_message(server, &mess))
	{
	    create_message_window(
		    "Failed to send ASTROMOD_STAR_DATA message.");
        }

	return 0;

} /* star_data_callback() */

/************************************************************************/
/* message_uv_data()							*/
/************************************************************************/

int message_uv_data(int server, struct smessage *mess)
{
        struct s_uv  *data;
	char	s1[8000],s2[200],s3[80];
	int	i;

        /* Make sure data is right length */

        if (mess->length != sizeof(*data))
        {
            create_message_window(
            "Recieved ASTROMOD_INIT message with wrong amount of data.");
        }

	data = (struct s_uv *)mess->data;

	sprintf(s1,"UV data received\n");
	sprintf(s2,"Az = %.2f\n", data->az/D2R);
	strcat(s1,s2);
	sprintf(s2,"El = %.2f\n", data->el/D2R);
	strcat(s1,s2);
	sprintf(s2,"HA = %s\n", time_to_string(data->ha,s3));
	strcat(s1,s2);
	for(i=0; i<15; i++)
	{
		sprintf(s2,"%s U = %.1f V = %.1f\n",
			data->uv_names[i], data->uv[i][U], data->uv[i][V]);
		strcat(s1,s2);
	}
	printf(s1);

	return NOERROR;

} /* message_uv_data() */

/************************************************************************/
/* uv_data_callback()  		                                        */
/*                                                                      */
/* Set global in the server.                                            */
/************************************************************************/

gint uv_data_callback(GtkButton *button, gpointer data)
{
	struct  smessage mess;
	struct  s_time_now now;

	now.year = 0;
	now.lambda = 2.134;

	mess.type = ASTROMOD_UV_DATA;
	mess.data = (unsigned char *)&now;
	mess.length = sizeof(now);

	if (!send_message(server, &mess))
	{
	    create_message_window(
		    "Failed to send ASTROMOD_UV_DATA message.");
        }

	return 0;

} /* uv_data_callback() */
