/************************************************************************/
/* error.c								*/
/*                                                                      */
/* Error message handling for the entire system. One function only.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/* 
 * KLUDGE!!!
 */

int socket_manager_remove_process(char *name);

/*
 * Global here... do we send messages?
 */

static bool send_error_messages = TRUE;
extern void (*user_close_function)(void);
static void (*user_error_function)(void) = NULL;
extern char    sockman_name[256];
bool inside_error_function = FALSE;

/************************************************************************/
/* error()								*/
/*									*/
/* This function handles the error messages for the entire system.	*/
/* It receives the error level, the error message and up to five 	*/
/* other arguments. If the error level is NOERROR nothing happens.	*/
/* If the error level is WARNING or MESSAGE the message is placed in the*/
/* error window. If the error is FATAL					*/
/* curses is closed ,the message is placed in stderr and the programme	*/
/* stops.								*/
/* Newlines in the formatted text are interpreted correctly.            */
/* Any lines of formatted text which are too wide for the error         */
/* window are broken up.                                                */ 
/* If the active_socket global has been sent the error message will be  */
/* sent to the socket, not to the screen.				*/
/************************************************************************/

int error(int level, char *fmt, ...)
{
	/* Internal Variables */

	char	sprintf_err_string[4097];/* Contains sprinf formatted message */
	char	err_string[4097];/* formatted message with line breaks*/
	char	s[4097];/* Contains formatted message with line breaks*/
	int	number_lines;		/* Number of lines in the message */
	char	*c;			/* Character pointer */
	WINDOW	*berror_window;		/* box around error window */
	WINDOW	*error_window;		/* ...to display message in */
	WINDOW	*error_head_window;	/* ...to display title in */
	int	sprintf_len;	/* number of characters out of sprintf */
	int	break_len;   /* number of characters out of break_into_lines */
	va_list	args;
#ifdef __NCURSES_H
        MEVENT  mouse;          	/* A mouse event. */
#endif
	int	key_stroke;

	if (!send_error_messages) return level;

	/* return if NOERROR */

	if (level == NOERROR) return level;

	/* We are inside the error function now */

	inside_error_function = TRUE;

	/*
	 * If the error came from the background we shouldn't process the 
	 * it for a while 
	 */

	if (active_socket == -1 && background_function) background_off(0,NULL);

	/* Create the whole error string */

        va_start(args, fmt);
	sprintf_len = vsprintf(sprintf_err_string, fmt, args);

	/* Break the string up into separate lines if necessary. */

	break_len = break_into_lines
	(
		err_string, sizeof(err_string), 
		sprintf_err_string, SUB_MAIN_COLS - 2 - 1
	);

	/* If error generated by active socket send message there */

	if (active_socket != -1)
	{
		switch(level)
		{
			case MESSAGE: socket_print(active_socket,
					"!!!: %s\n",err_string);
				      break;
			case ERROR: socket_print(active_socket,
					"ERROR: %s\n",err_string);
				      break;
			case WARNING: socket_print(active_socket,
					"WARNING: %s\n",err_string);
				      break;
			case FATAL: socket_print(active_socket,
			    "A FATAL ERROR HAS OCCURED ... SHUT DOWN\n");
			            socket_print(active_socket,
					"\nFATAL: %s\n",err_string);
				    socket_print(active_socket,
			    "\nYOU NEED TO RESTART THE PROGRAMME\n\n");
				      break;
			default: socket_print(active_socket,
					"UNKNOWN: %s\n",err_string);
				      break;
		}

		if (level != FATAL) 
		{
			inside_error_function = FALSE;
			return level;
		}
	}
	
	/* If this came from a background job, send it to all clients */

	if (background_function)
	{
	    switch(level)
	    {
		case MESSAGE: ui_print_message_all(
					"!!!: %s\n",err_string);
			      break;

		case ERROR: ui_print_message_all(
					"ERROR: %s\n",err_string);
			      break;

		case WARNING: ui_print_message_all(
					"WARNING: %s\n",err_string);
			      break;

		case FATAL: ui_print_message_all(
		    "A FATAL ERROR HAS OCCURED ... SHUT DOWN\n"
				"\nFATAL: %s\n"
		    "\nYOU NEED TO RESTART THE SERVER\n\n",
				err_string);
			      break;

		default: ui_print_message_all( 
				"UNKNOWN: %s\n",err_string);
			      break;
	    }
	} 

	if (client_socket != -1)
	{

	    /* If error generated by client send message there */

	    if (!background_function) switch(level)
	    {
		case MESSAGE: ui_print_message(client_socket,
					"!!!: %s\n",err_string);
			      break;
		case ERROR: ui_print_message(client_socket,
					"ERROR: %s\n",err_string);
			      break;
		case WARNING: ui_print_message(client_socket,
					"WARNING: %s\n",err_string);
			      break;
		case FATAL: ui_print_message(client_socket,
		    "A FATAL ERROR HAS OCCURED ... SHUT DOWN\n"
				"\nFATAL: %s\n"
		    "\nYOU NEED TO RESTART THE SERVER\n\n",
				err_string);
			      break;
		default: ui_print_message(client_socket, 
				"UNKNOWN: %s\n",err_string);
			      break;
	    }
	    if (level != FATAL)
	    {
		inside_error_function = FALSE;
		return level;
	    }
	}
	
	/* Now deal with server user */

	if (level == FATAL)
	{
		clear();
		refresh();
		system("stty sane");
		fprintf(stderr,"A FATAL ERROR HAS OCCURED ... SHUT DOWN\n");
		fprintf(stderr,"\n");
		fprintf(stderr, err_string);
		fprintf(stderr,"\n\nYOU NEED TO RESTART THE PROGRAMME\n\n");
		fprintf(stderr,"Type RETURN to continue.\n");
		while(!kbhit());
		get_command();
		if (user_close_function != NULL) (*user_close_function)();
		close_curses();
		send_server_stopping();
		close_command_socket();
		close_data_socket();
		close_mirror_socket(0,NULL);
		if (*sockman_name != 0)
			socket_manager_remove_process(sockman_name);
		//delete_scripts();
		exit(1);
	}

	/* If we get to here it is something generated by the server user */

	/* Count the number of lines in message */

	for ( c = err_string, number_lines = 1; *c != (char)NULL; c++)
		if (*c == '\n') number_lines++;

	/* Make sure there are not too many lines */

	if (number_lines > SUB_MAIN_LINES - 3)
	{
		sprintf(s,"\nToo many lines %d in error message.\n",
			number_lines);
		strcat(s,"Consult software supplier. Message:\n");
		strcat(s,"err_string");
		error(FATAL,err_string);
	}

	/* create the title */

	if(level == WARNING)
		c = "WARNING";
	if(level == ERROR)
	{
		if ( user_error_function != NULL) (*user_error_function)();
		c = "ERROR";
	}
	if(level == MESSAGE)
		c = "! ! !";

	/* Create the windows */

	if ((berror_window = newwin(number_lines + 3, SUB_MAIN_COLS, 
		(SUB_MAIN_BEGIN_Y + SUB_MAIN_LINES) - number_lines - 3, 
							SUB_MAIN_BEGIN_X)) 
			== (WINDOW *)ERR)
	{
		error(FATAL,FATAL_1,"error system");
	}

	if ((error_head_window = subwin(berror_window, 1, SUB_MAIN_COLS-2,
		(SUB_MAIN_BEGIN_Y + SUB_MAIN_LINES) - number_lines - 2, 
							SUB_MAIN_BEGIN_X + 1)) 
			== (WINDOW *)ERR)
	{
		error(FATAL,FATAL_1,"error system");
	}

	if ((error_window = subwin(berror_window,number_lines, SUB_MAIN_COLS-2, 
		(SUB_MAIN_BEGIN_Y + SUB_MAIN_LINES) - number_lines - 1, 
							SUB_MAIN_BEGIN_X + 1)) 
			== (WINDOW *)ERR)
	{
		error(FATAL,FATAL_1,"error system");
	}

	wstandout(error_head_window);

	box(berror_window,0,0);

	/* put title into error_head_window */

	mvwaddstr(error_head_window,0,((SUB_MAIN_COLS-2)-strlen(c))/2,c);

	/* Put in the message */

	nl(); 			/* To ensure new lines get new lines */
	mvwaddstr(error_window,0,0,err_string);
	nonl();

	/* make the windows show */

	wrefresh(berror_window);
	wrefresh(error_head_window);
	wrefresh(error_window);

	/* Put up continuation message */

	message(system_ol_window,CONT_TEXT);
	while(1)
	{
		key_stroke = get_command();

#ifdef __NCURSES_H
                if (key_stroke == KEY_MOUSE)
                {
                        getmouse(&mouse);

                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
                                /* OK, what key did the mouse represent. */

                                if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x <  strlen(CONT_TEXT)+2)
                                {
                                        key_stroke = KEY_ESC;
                                }
			}
		}
#endif
		if (key_stroke  == KEY_ESC) break;
	}
	werase(system_ol_window);
	touchwin(system_ol_window);
	wrefresh(system_ol_window);
	
	/* We can start the background again */

	if (active_socket == -1 && background_function) background_on(0,NULL);

	/* Get rid of windows */

	delwin(error_window);
	delwin(error_head_window);
	delwin(berror_window);

	/* refresh the currently active window */

	touchwin(active_window);
	wrefresh(active_window);
	touchwin(system_window);
	wrefresh(system_window);
	inside_error_function = FALSE;
	return level;
	
} /* error() */

/************************************************************************/
/* break_into_lines()                                                   */
/*                                                                      */
/* This function breaks a string into lines of maximum length.          */
/* The string may already contain new-lines, these are preserved.       */
/* It cannot work in place, the length of the output buffer is          */
/* supplied and will not be overfilled.                                 */
/* Return value:                                                        */
/* Number of characters written to output buffer, excluding the null    */
/* or                                                                   */
/* size of output buffer if insufficient space.                         */
/* If there is insufficient space, the output buffer is filled up       */
/* and the last character of the buffer is set to null.                 */
/* Spaces at the beginning of lines are removed.                        */
/************************************************************************/

int break_into_lines( 
	char	*output,
	int	sizeof_output, /* Size of output buffer including null */
	char	*input,
	int	max_line_width)
{
	bool	input_processed;
	bool	output_full;
	bool	newline;
	bool	line_too_long;
	bool	first_line;
	bool	space;
	bool	beginning_of_line;
	int		num_chars;
	int		output_remaining;
	int		output_chars;
	char		*input_next;
	char		*output_next;
	char		*input_line;
	
	input_next = input;
	output_next = output;
	output_remaining = sizeof_output;
	input_processed = FALSE;
	output_full = FALSE;
	first_line = TRUE;

	while (!input_processed && !output_full)
	{
		/* Scan over spaces */
		
		space = FALSE;
		while(space)
		{
			space = isspace(*input_next);
			if(space)
			{
				input_next++;
			}
		}

		/* Start a line */

		input_line = input_next;
		newline = FALSE;
		line_too_long = FALSE;
		num_chars = 1;

		while (!input_processed && !newline && !line_too_long)
		{
			input_processed = *input_next == (char)0;
			if(!input_processed)
			{
				newline = *input_next == '\n';
				if(!newline)
				{
					num_chars++;
					input_next++;
					line_too_long = num_chars > max_line_width;
				}
			}
		}
		if(line_too_long)
		{
			space = FALSE;
			beginning_of_line = FALSE;
			while(!space && !beginning_of_line)
			{
				space = isspace(*input_next);
				beginning_of_line = num_chars == 0;
				if(!space)
				{
					if(!beginning_of_line)
					{
						input_next--;
						num_chars--;
					}
				}
			}
			if(beginning_of_line)
			{
				/* No spaces in line so truncate. */

				num_chars = max_line_width;
			}
			else
			{
				/* Found a space so leave it out */
				num_chars--;

				/* Skip over space */
				input_next++;
			}
		}
		else
		{
			/* Pointing at \n or 0 */
			if(newline)
			{
				/* Don't need to copy \n from input. */
				num_chars--;

				/* Need to skip over \n */
				input_next++;
			}
			/* 
				When input is completely processed 
				the null from input
				is copied to output 
			*/
		}
		if(!first_line && (num_chars > 0 || newline))
		{
			/* Must add newline */
		
			output_full = output_remaining < 1;
			if(!output_full)
			{
				*output_next++ = '\n';
				output_remaining--;
			}
		}
		first_line = FALSE;
		output_full = num_chars > output_remaining;
		if(output_full)
		{
			num_chars = output_remaining;
		}
		if(num_chars > 0)
		{
			strncpy(output_next, input_line, num_chars);
			output_next += num_chars;
			output_remaining -= num_chars;
		}
	}
	
	if(output_full)
	{
		output_chars = sizeof_output;
		output[sizeof_output - 1] = (char)0;
	}
	else
	{
		output_chars 
		= 
		sizeof_output - output_remaining - sizeof((char)0);
	}

	return output_chars;

} /* break_into_lines() */

/************************************************************************/
/* error_messages_on() 							*/
/* 									*/
/* Ensure that error messages are sent to the outside world.		*/
/************************************************************************/

int error_messages_on(int argc, char **argv)
{
	send_error_messages = TRUE;
	return NOERROR;

} /* error_messages_on() */

/************************************************************************/
/* error_messages_off() 						*/
/* 									*/
/* Ensure that error messages are not sent to the outside world.	*/
/************************************************************************/

int error_messages_off(int argc, char **argv)
{
	send_error_messages = FALSE;
	return NOERROR;

} /* error_messages_off() */

/************************************************************************/
/* set_user_error_function()						*/
/*									*/
/* Setup a function for the system to call when an error occurs.	*/
/* Set function to NULL to turn it off. 				*/
/************************************************************************/

void set_user_error_function(void (*function)(void))
{
	user_error_function = function;

} /* set_user_error_function() */
