/************************************************************************/
/* gtk_stand_mess.c							*/
/*                                                                      */
/* Standard message handling routines.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#define	USE_GTK

#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"
#include <stdio.h>
#include <errno.h>

/* Global for a name */

static char gtkclient_name[256];

/* Global that keeps track of the number of messages up */

int     number_of_messages_displayed = 0;

/* GLobals for use in pointing at things */

int true_pointer = TRUE;
int false_pointer = FALSE;

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A message to do nothing.						*/
/************************************************************************/

int message_noop(int server, struct smessage *mess)
{
	return TRUE;
}

/************************************************************************/
/* message_message()							*/
/*									*/
/* A message to display a message.					*/
/************************************************************************/

int message_message(int server, struct smessage *mess)
{
        char *s,*p1,*p2;
        int     i;

        if (mess->length == 0)
        {
		return TRUE;
        }
        else
        {
                if ((s = malloc(mess->length+1)) == NULL)
                {
			create_message_window(
				"Ran out of memory geting message string.");
			exit(-7);
                }

                for(i=0, p1=s, p2=(char *)mess->data;
                    i < mess->length; i++, p1++, p2++) *p1 = *p2;
                *p1 = 0;

                create_message_window("MESSAGE FROM SERVER: %s\n\n%s",
			servers[server].name,s);

                free(s);
        }

	return TRUE;

} /* message_message() */

/************************************************************************/
/* set_client_name()							*/
/************************************************************************/

void set_client_name(char *name)
{
	strcpy(gtkclient_name, name);

} /* set_client_name() */

/************************************************************************/
/* message_ping()							*/
/*									*/
/* If we get pinged by server, although why I do not know.		*/
/************************************************************************/

int message_ping(int server, struct smessage *mess)
{
        print_message(server, "%s is alive.",gtkclient_name);
	return TRUE;

} /* message_ping() */

/************************************************************************/
/* message_server_stopping()						*/
/*									*/
/* What do we do if we get told the server has died?			*/
/************************************************************************/

int message_server_stopping(int server, struct smessage *mess)
{

	close_server_socket(server);
	server = -1;
	create_message_window(
"Server is stopping.\nYou will have to REOPEN the connection\nonce it is running again.");
	return TRUE;

} /* message_server_stopping() */

/************************************************************************/
/* create_message_window()                                              */
/*                                                                      */
/* Displays a text message in a window for the user to see.             */
/* Code mostly generated by GLADE.                                      */
/************************************************************************/

void create_message_window (char *fmt, ...)
{
  va_list args;
  char    mess[2048];
  GtkWidget *message_window;
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *ok_button;

  /* Assemble the message */

  va_start(args, fmt);
  vsprintf(mess, fmt, args);

  /* Display the message */

  message_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (message_window), "message_window",
                message_window);
#warning KLUDGE!!!!!
/* This cause trouble I do not understand 
  gtk_window_set_title (GTK_WINDOW (message_window), 
		(const gchar *)gtkclient_name);
*/

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (message_window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (message_window), vbox1);

  label1 = gtk_label_new (mess);
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (message_window), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, TRUE, TRUE, 0);

  ok_button = gtk_button_new_with_label ("OK");
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (message_window), "ok_button", ok_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ok_button);
  gtk_box_pack_start (GTK_BOX (vbox1), ok_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (message_ok_button_clicked),
                      NULL);

  number_of_messages_displayed++;

  gtk_widget_show(message_window);

}

/************************************************************************/
/* message_ok_button_clicked()                                          */
/*                                                                      */
/* Callback function for OK button in create_message_window().          */
/************************************************************************/

void message_ok_button_clicked(GtkButton *button, gpointer user_data)
{
        gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET (button)));
	if (--number_of_messages_displayed < 0)
		number_of_messages_displayed = 0;

} /* message_ok_button_clicked() */

/************************************************************************/
/* noop_callback()							*/
/*									*/
/* Callback for NOOP, data is the server to use.			*/
/************************************************************************/

void noop_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = NOOP;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
                create_message_window("Failed to send NOOP message.");

} /* noop_callback() */

/************************************************************************/
/* ping_callback()							*/
/*									*/
/* Callback for PING, data is the server to use.			*/
/************************************************************************/

void ping_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = PING;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
                create_message_window("Failed to send PING.");

} /* ping_callback() */

/************************************************************************/
/* all_noop_callback()							*/
/*									*/
/* Callback for ALL_NOOP, data is the server to use.			*/
/************************************************************************/

void all_noop_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = ALL_NOOP;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
                create_message_window("Failed to send ALL_NOOP message.");

} /* all_noop_callback() */

/************************************************************************/
/* all_ping_callback()							*/
/*									*/
/* Callback for ALL_PING, data is the server to use.			*/
/************************************************************************/

void all_ping_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = ALL_PING;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
                create_message_window("Failed to send ALL_PING message.");

} /* all_ping_callback() */

/************************************************************************/
/* send_ui_signal()							*/
/*									*/
/* Sends a signal to the server.					*/
/************************************************************************/

void send_ui_signal(int server, unsigned char signal)
{
	struct smessage mess;

        /* Make sure everything is OK */

        if (!server_socket_open(server)) return;

        /* Construct command */

        mess.type = UI_SIGNAL;
        mess.length = sizeof(signal);
        mess.data = &signal; 

        if (!send_message(server, &mess))
                create_message_window("Failed to send UI_SIGNAL message.");

} /* send_ui_signal() */
