/************************************************************************/
/* matrox_general.c							*/
/*									*/
/* Routines to work with the meteor matrox frame grabber.		*/
/* General purpose routines.						*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy			*/
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.		*/
/*									*/
/*									*/
/* Telephone: 1-404-651-1882						*/
/* Fax      : 1-404-651-1389						*/
/* email    : theo@chara.gsu.edu					*/
/* WWW	    : http://www.chara.gsu.edu/~theo/theo.html			*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1999   						*/
/************************************************************************/

#include "matrox_meteor.h"
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <ioctl_meteor.h>
#include <stdlib.h>
#include <signal.h>

int last_brightness = -1;
int last_contrast = -1;

/************************************************************************/
/* set_meteor_chan()							*/
/*									*/
/* Set int input channel used by the meteor frame grabber (1-4).	*/
/* Returns 								*/
/* 0 if it worked							*/
/* 1 if the device is not open						*/
/* 2 if the chanel number is out of range				*/
/* 3 if the ioctl call failed.						*/
/************************************************************************/

int set_meteor_chan(int chan)
{
	int c;

	if (meteor_fd < 0) return 1;

        switch(chan)
        {
                case 1: c = METEOR_INPUT_DEV0;
                        break;
                case 2: c = METEOR_INPUT_DEV1;
                        break;
                case 3: c = METEOR_INPUT_DEV2;
                        break;
                case 4: c = METEOR_INPUT_DEV3;
                        break;
		default: return 2;
        }

        if (ioctl(meteor_fd, METEORSINPUT, &c) < 0) return 3;

	return 0;

} /* set_meteor_chan() */

/************************************************************************/
/* set_meteor_contrast()						*/
/*									*/
/* Set's the contrast to the given value.				*/
/* Returns 0 if it works, 1 if the device is not open and 2 if the 	*/
/* value is out of range and 3 if the ioctl call fails.			*/
/************************************************************************/

int set_meteor_contrast(int contrast)
{
	
	if (meteor_fd < 0) return 1;

	if (contrast < MIN_CONTRAST || contrast > MAX_CONTRAST) return 2;

	if (ioctl(meteor_fd,METEORSCONT,&contrast) < 0) return 3;

	last_contrast = contrast;

	return 0;

} /* set_meteor_contrast() */

/************************************************************************/
/* set_meteor_brightness()						*/
/*									*/
/* Set's the brightness to the given value.				*/
/* Returns 0 if it works, 1 if the device is not open and 2 if the 	*/
/* value is out of range and 3 if the ioctl call fails.			*/
/************************************************************************/

int set_meteor_brightness(int brightness)
{
	
	if (meteor_fd < 0) return 1;

	if (brightness<MIN_BRIGHTNESS || brightness>MAX_BRIGHTNESS) return 2;

	if (ioctl(meteor_fd,METEORSBRIG,&brightness) < 0) return 3;

	last_brightness = brightness;

	return 0;

} /* set_meteor_brightness() */
