/************************************************************************/
/* exampleX.c 								*/
/*									*/
/* Example program to make an X window.  				*/
/* Puts a 256x256 pixel image on the screen.				*/
/* To put new image up, destroy theImage, create a new one and display.	*/
/* This version works for 8, 16 or 32 bit depths.			*/
/* To compile: 								*/
/* gcc -o exampleX exampleX.c -lnsimpleX -lnrc -lc -lm -lX11 		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : October 1998   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
/* These should be

#include <nrc.h>
#include <nsimpleX.h>

 */

#include "../nrc/nrc.h"
#include "nsimpleX.h"

#include <stdlib.h>

int main(int argc, char **argv)
{
	Window	theWindow;
	XImage	*theImage;
	int	i,j;
	char *picture;
	char *p8;
	unsigned short int *p16;
	unsigned int *p32;
	size_t  size;
	int	color;

	/* Initialize the X windows connection */

	if (argc > 1)
		i = initX(argv[1]);
	else
		i = initX(NULL);

        if (!i)
        {
                fprintf(stderr,"Failed to open display.\n");
		exit(-1);
        }

	/* Allocate memory for the picture */

	size = 256 * 256;
        if ((picture = (char *)malloc(size*theBitmapBytes)) == NULL)
        {
                fprintf(stderr,"Ran out of memory.\n");
                exit(-1);
        }

	/* Print out information about the server */

	infoX();

	/* Put something into the image */

	p8 = picture;
	p16 = (unsigned short int *)picture;
	p32 = (unsigned int *)picture;
	color = 0;
	for(i=0; i<256;i++)
	for(j=0; j<256;j++)
	{
		switch(theDepthBytes)
		{
			case 1: *p8++ = GreyScale[color];
				break;

			case 2: *p16++ = GreyScale[color];
				break;

			default: *p32++ = GreyScale[color];
				break;
		}

		/* Color must be between 0 and NUM_COLORS-1 (127) */

		if (++color > NUM_COLORS-1 || color < 0) color = 0;
	}

	/* Create the window */

	theWindow = openWindow("EXAMPLE",100,100,256,256);

	/* Create an Image */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture,256,256,theBitmapPad, 0);

	/* Did that work ? */

	if (!theImage) 
	{
		fprintf(stderr,"Could not create image.\n");
	}

	/* Put the image into the display */

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,256,256);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	/* Wait for a few seconds */

	sleep(2);

	/* Cleanup */

	XDestroyImage(theImage);
	XDestroyWindow(theDisplay, theWindow);
	quitX();

	exit(0);
}
