/************************************************************************/
/* nsimpleX.h                                                           */
/*                                                                      */
/* Globals and defines for entire simple X system			*/
/* Part of the nsimpleX library.					*/
/* This version works for 8, 16 or 32 bit depths.			*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : August 1994                                                 */
/************************************************************************/

#ifdef _SIMPLEX_H_
#warning You seem to have used both simpleX and nsimpleX, this is not wise.
#endif

#ifndef _NSIMPLEX_H_ /* Ensure we haven't been here before */
#define _NSIMPLEX_H_

#include <stdlib.h>
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* Defines */


#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define BORDER_SIZE	2
#define NUM_COLORS	128 
#define LIN     0
#define LOG     1
#define EXP     2
#define STD     3

#define MAX_SIGNALS	10

#define RADIANS_PER_ARCSEC	4.84814e-6

/* Note that these are also defined in zernike.h and MUST BE THE SAME */

#ifndef MEAN
#define MEAN    3
#define MAX     2
#define MIN     1
#define ZERO    0

/* Now we define some colors */

#define BLACK (GreyScale[0])
#define WHITE (GreyScale[NUM_COLORS-1])

/* These colors are for the set_plot_color function */

#define PLOT_BLACK	0,0,0
#define PLOT_RED	65535,0,0
#define PLOT_GREEN	0,65535,0
#define PLOT_DARK_BLUE	0,0,65535
#define PLOT_YELLOW	65535,65535,0
#define PLOT_PURPLE	65535,0,65535
#define PLOT_LIGHT_BLUE	0,65535,65535
#define PLOT_WHITE	65535,65535,65535

#endif

/*
 * Globals
 */

extern Display	*theDisplay;
extern int	theScreen;
extern int	theDepth;
extern int	theDepthBytes;
extern int	theBitmapPad;
extern int	theBitmapBytes;
extern int	theWidth;
extern int	theHeight;
extern Visual  *theVisual;
extern GC	theGC;
extern Window	theRootWindow;
extern unsigned int   fcol,bcol;
extern Colormap        theCmap;
extern unsigned int GreyScale[NUM_COLORS];
extern int display_is_open;
extern unsigned int plot_color;

/*
 * Structures for scope functions
 */

typedef struct SIGNAL {

	int	color;	/* Signal color */
	float   yscale;	/* Maximum range for signal */
	float   offset;	/* Signal offset */
	int	zero_pos; /* Framed position of zero point */

		      } scope_signal;

typedef struct SCOPE {
	char	name[81];	/* The name of the scope */
	Window	scope_window;	/* The window to put it in */
	XImage	*scope_image;	/* Image structure for scope */
	int	xpix;		/* Number of pixels in x axis */
	int	ypix;		/* Number of pixels in y axis */
	int	total_pix;	/* Total number of pixels */
	int	nsignals;	/* Number of signals to watch */
	scope_signal  signals[MAX_SIGNALS];	/* The signals to track */
	int *last_position;     /* Where they where last */
	char *picture;          /* The actual picture we see */
		     } scope;

/* Function Declarations */

unsigned char initX(char *display_name);

void quitX(void);

Window openWindow(char *name, 
		  int x, 
		  int y, 
		  int width, 
		  int height);
void nameWindow(Window theWindow, char *name);

void infoX(void);

char    *make_scaled_picture(int  nx, 
		   int ny, 
		   int pixmult, 
		   float **values, 
		   char disp_type,
		   float *min,
		   float *max);

char    *make_picture(int  nx, 
		   int ny, 
		   int pixmult, 
		   float **values, 
		   char disp_type);

void    make_ppm_P3(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture); 

void    make_ascii_pict(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture); 

void    make_ppm_P6(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture);

void make_2D_gnuplot_file(float **data,
                          int nx,
                          float *x,
                          int ny,
                          float *y,
                          char *filename);

scope open_scope(char *name, 
		 int x, 
		 int y, 
		 int nx, 
		 int ny);

void close_scope(scope *TheScope);

int     add_signal(scope *TheScope,
                   int red, 
		   int green, 
		   int blue,
                   float ymin,
                   float ymax);

void	update_scope(scope TheScope, float *values);

float	**subtract_images(float **image1,
			  float **image2,
			  int npix_x,
			  int npix_y);

void 	normalize_image(float **image,
			int npix_x,
			int npix_y,
			int *max_x,
			int *max_y);

void	smooth_image(float 	**image,
		     int	npix_x,
		     int	npix_y,
		     float	**weights,
		     int	spix);	

void	shift_matrix(float **data, 
		     int npix_x, 
		     int npix_y, 
		     int x_shift, 
		     int y_shift);

void	shift_vector(float *data, 
		     int npix, 
		     int shift);

void	sum_images( float **result, 
		    float **image, 
		    int npix_x,
		    int npix_y );

float	max_image_intensity( float **image, 
			     int npix_x, 
			     int npix_y, 
			     int *pos_x, 
			     int *pos_y);

float	min_image_intensity( float **image, 
			     int npix_x, 
			     int npix_y, 
			     int *pos_x, 
			     int *pos_y);

float   total_image( float **image,
                             int npix_x,
                             int npix_y);

void    scale_image( float **image,
                     float scale,
                     int npix_x,
                     int npix_y);

float **copy_image( float **image,
                             int npix_x,
                             int npix_y);

float centroid_image(float **data, int nx, int ny, /* The data arrays size */
                    int gx, int gy, /* The first guess of the position */
                    int size,       /* Size of area to search in */
                    float *px, float *py); /* The resulting position */

float local_max_image_intensity(float **data, int nx, int ny,
                    int gx, int gy, /* The first guess of the position */
                    int size,       /* Size of area to search in */
                        int *xpos, int *ypos);

Bool get_mouse(
        Window  theWindow,
        int     pixmult,
        int     *x,
        int     *y);

void display_picture(Window theWindow, float **values,
        int npix, int pixmult, char disp_type);

/* overlay_line.c */

void overlay_line(
	int nx,
	int ny,
	char *picture,
	int x,
	int y,
	int length,
	int width,
	int horiz,
	int red, int green, int blue);

void overlay_rectangle(
	int nx,
	int ny,
	char *picture,
	int ulx,
	int uly,
	int lrx,
	int lry,
	int width,
	int red, int green, int blue);

/* build_airy.c */

float **build_airy(int xpix, int ypix, float scale, 
		char *filter, float tel_diam, float tel_obsc);

float airy_intensity(float v);

float airyobsc_intensity(float v, float e);

/* calculate_fwhm.c */

float calculate_fwhm(float **psf, int xpix, int ypix, float scale);

/* calculate_ps.c */

float **calculate_ps(float **data, int nx, int ny, int imbedp2, int fill);

/* caluclate_fwhp.c */

float calculate_fwhp(float **psf, int xpix, int ypix, float scale);

/* zero_edge.c */

float zero_edge(float **image, int nx, int ny);

#ifdef JUNK

/* This has been removed.... it needs more work. */

/* calculate_fft.c */

fcomplex **calculate_fft(float **data, int nx, int ny, int imbedp2, int fill);

float **calculate_ifft(fcomplex **fft, int nx, int ny, int imbedp2);
#endif

/* convolve.c */

float **convolve(float **data1, float **data2, int nx, int ny, int fill);

/* plot.c */

char *plot_funct(int width, int height, float *data, int num_data,
		float *min, float *max);

char *merge_plots(char *plot1, char *plot2, int size_x, int size_y);

int set_plot_color(int red, int green, int blue);

/* get_filter_value.c */

double get_filter_value(
        char    *filter,
        double  *fwidth);

/* fitgaussian.c */


float fit_2D_gaussian(float **pict, int nx, int ny, float *amp, 
			float *x0, float *y0, float *c, float *fwhm, 
			float per_rms_err, int maxit, int show);
 
void    gassian_2D_func(float *ftn, float *var, float *param);

float fit_1D_gaussian(int ndata, /* Number of data points */
		  float *x, /* x[1..ndat] */
		  float *y, /* y[1..ndat] */
		  float *amp, float *mean, float *stddev, /* Results */
		  float per_rms_err, int maxit, int show);

void    gauss_1D_func(float *ftn, float *var, float *param);

float fit_fringe_envelope(int ndata, /* Number of data points */
                  float *x, /* x[1..ndat] */
                  float *y, /* y[1..ndat] */
                  float *v, float *x0, float *dwn, /* Results */
                  float per_rms_err, int maxit, int show);

void fit_fringe_envelope_func(float *ftn, float *var, float *param);
#endif
