/************************************************************************/
/* testconvolve.c							*/
/*									*/
/* Test programme for convolution routine.  				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : July  1998   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "nsimpleX.h"

int main(int argc, char **argv)
{
	Window	theWindow;
	int	i,j;
	float	**value1;
	float	**value2;
	float	**value3;
	int	npix,pixmult;
	int	mid,quart;

	if (argc < 3)
	{
		fprintf(stderr,"usage: %s npix pixmult <display>\n",argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&npix);
	sscanf(argv[2],"%d",&pixmult);

	if (argc > 3)
		i = initX(argv[3]);
	else
		i = initX(NULL);

	if (!i)
	{
		fprintf(stderr,"Failed to open display.\n");
		exit(-1);
	}

	infoX();

	theWindow = openWindow("THIS IS IT",100,100,npix*pixmult,npix*pixmult);

	/* Build a test image */

	value1 = matrix(1,npix,1,npix);
	value2 = matrix(1,npix,1,npix);

	mid = npix/2;
	quart = mid/2;
	for(i=1;i<=npix;i++)
	{
		for(j=1;j<=npix;j++)
		{
			if ((j == quart && i == quart) ||
				(j == npix-quart && i == npix-quart))
				value1[i][j] = 1.0;
			else
				value1[i][j] = 0.0;

			if (j>quart && j<npix-quart && i>quart && i<npix-quart)
				value2[i][j] = 1.0;
			else
				value2[i][j] = 0.0;
		}
	}

	value3 = convolve(value1, value2, npix, npix, ZERO);
	printf("value3[1][1] = %f\n",value3[1][1]);

	printf("First image max = %f.\n",
		max_image_intensity(value1,npix,npix,&i,&j));
	fflush(stdout);
	display_picture(theWindow, value1, npix, pixmult, LIN);
	sleep(2);
	printf("Second image max = %f.\n",
		max_image_intensity(value1,npix,npix,&i,&j));
	fflush(stdout);
	display_picture(theWindow, value2, npix, pixmult, LIN);
	sleep(2);
	printf("Convolution max = %f.\n",
		max_image_intensity(value1,npix,npix,&i,&j));
	fflush(stdout);
	display_picture(theWindow, value3, npix, pixmult, LIN);
	sleep(2);

	free_matrix(value1,1,npix,1,npix);
	free_matrix(value2,1,npix,1,npix);
	free_matrix(value3,1,npix,1,npix);
        XDestroyWindow(theDisplay, theWindow);
	quitX();
	exit(0);
}
