/************************************************************************/
/* editfits.c								*/
/*									*/
/* Programme to edit s fits file header.				*/
/*									*/
/* Usage: edit file							*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : July 1996 							*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"

int main(int argc, char **argv)
{
	/* FITs file data */

	fits_data data;
	char	field_type;
	int	loop;
	int	int_value;
	float	float_value;
	char	text_value[81];
	char	comment[81];
	char	keyword[81];
	int	change_made;

	/* Ensure command line is OK */

	if (argc < 2)
	{
		fprintf(stderr,"usage: %s file.fits\n",argv[0]);
		exit(-1);
	}

	printf("EDITFITS : %s\n\n",argv[1]);

	/* Read the FITS file */

	data = read_fits(argv[1]);

	if (data.nheader == 0) exit(-2);

	/* Loop forever allowing edits */

	loop = TRUE;
	change_made = FALSE;
	while(loop)
	{
		/* Display the header info */

		putchar('\n');
		print_fits_header(data);

		/* See what the user wants to edit. */

		printf("\nEnter key to edit or quit to finish : ");
		scanf("%s",keyword);

		/* See if this is it. */

		if (strcmp(keyword,"QUIT") == 0 ||
			strcmp(keyword,"quit") == 0) 
		{
			loop = FALSE;
			continue;
		}

		/* Make sure it exists */

		if (find_key(data, keyword) == 0)
		{
			printf("Key %s does not exist.\n",keyword);
			printf("Floating point - f\n");
			printf("Integer        - i\n");
			printf("Text           - t\n");
			printf("Which do I use : ");
			getchar();
			gets(comment);
			if (*comment == 'i') field_type = INT;
			else if (*comment == 'f') field_type = FLOAT;
			else if (*comment == 't') field_type = TEXT;
			else
			{
				printf("Eh?\n\n");
				continue;
			}
		}
		else if ((field_type = fits_header_type(data, keyword)) 
			== UNKNOWN)
		{
			printf("Type for field %s is unknown.\n",keyword);
			continue;
		}

		/* OK, let's decide what to do about this */

		switch(field_type)
		{

			case INT: int_value = read_fits_header_int(data,
						keyword, comment);
				  printf("Current Value   : %d\n",int_value);
				  printf("Enter new value : ");
				  scanf("%d",&int_value);
				  write_fits_header_int(&data, keyword,
						int_value, comment);
				  change_made = TRUE;
				  break;

			case FLOAT: float_value = read_fits_header_float(data,
						keyword, comment);
				  printf("Current Value   : %g\n",float_value);
				  printf("Enter new value : ");
				  scanf("%f",&float_value);
				  write_fits_header_float(&data, keyword,
						float_value, comment);
				  change_made = TRUE;
				  break;

			case TEXT: if (read_fits_header_text(data, keyword, 
					text_value, comment) != 0)
				  {
				    printf("Current Value   : %s\n",text_value);
				    getchar();
				  }
				  printf("Enter new value : ");
				  gets(text_value);
					printf("%s\n",text_value);
				  write_fits_header_text(&data, keyword,
						text_value, comment);
				  change_made = TRUE;
				  break;

			default: printf("Unknown field type %d\n",field_type);
				 break;
		}
	}

	if (change_made)
	{
		printf("\nFINAL VERSION OF HEADER: \n\n");
		print_fits_header(data);

		printf("\nDo you want to save it (y/n) ? ");
		scanf("%s",keyword);
		if (*keyword == 'y' || *keyword == 'Y')
		{
			printf("Saving file %s.\n",argv[1]);
			write_fits(data, argv[1]);
		}
	}

	/* Clean up and go */

	free_fits_data(data);
	exit(0);
}
