/************************************************************************/
/* extractfits.c							*/
/*									*/
/* Programme to extract a fits file header and image.			*/
/*									*/
/* Usage: extractfits infile outfile					*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : Feb 2001 							*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

int main(int argc, char **argv)
{
	/* FITs file data */

	fits_data fits_data;
	FILE	*output;
	int	i,j;

	/* Ensure command line is OK */

	if (argc != 3)
	{
		fprintf(stderr,"usage: %s infile.fits outfile.bin\n",argv[0]);
		exit(-1);
	}

	/* Read the FITS file */

	fits_data = read_fits(argv[1]);

	if (fits_data.nheader == 0) exit(-2);

	/* Display the header info */

	print_fits_header(fits_data);

	/* Write binary data */

	if ((output = fopen(argv[2],"wb")) == NULL)
	{
		fprintf(stderr,"Failed to open %s for writing.\n",argv[2]);
		exit(-1);
	}

	for(i=1; i<=fits_data.nx; i++)
	for(j=1; j<=fits_data.ny; j++)
	if (fwrite(&(fits_data.data[i][j]), sizeof(fits_data.data[i][j]), 
		1, output) != 1)
	{
		fprintf(stderr,"Failed to write to %s.\n",argv[2]);
		exit(-2);
	}

	fclose(output);
		
	/* Clean up and go */

	free_fits_data(fits_data);

	exit(0);
}
