/************************************************************************/
/* showfits.c								*/
/*									*/
/* Programme to display a fits file header and image.			*/
/*									*/
/* Usage: showfits file							*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : July 1996 							*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

int main(int argc, char **argv)
{
	/* FITs file data */

	fits_data data;

#ifdef DISPLAY
	/* X windows variables */

	Window	theWindow;
	XImage	*theImage;
	char *picture;
#endif
	
	/* General variables */

	int	xpos,ypos;
	int	xmin,xmax,ymin,ymax;
#ifdef TEST
	char	c;
	char 	*s;
	char	str[81];
	char	comment[81];
	fits_data copy;
	int	bitpix;
	float	bscale;
	float	bzero;
#endif

	/* Ensure command line is OK */

	if (argc < 2)
	{
		fprintf(stderr,"usage: %s file.fits <display>\n",argv[0]);
		exit(-1);
	}

	printf("FILE : %s\n\n",argv[1]);

	/* Read the FITS file */

	data = read_fits(argv[1]);

	if (data.nheader == 0) exit(-2);

	/* Display the header info */

	print_fits_header(data);

	/* Try out the text reading...... */

#ifdef TEST
	s = read_fits_header_text(data, "DatE", str, comment);
	printf("DatE = %s # %s\n",s,comment);
	s = read_fits_header_text(data, "DATE", str, comment);
	printf("\nDATE = %s # %s\n",s,comment);
	s = read_fits_header_text(data, "TEXTBUM", str, comment);
	printf("TEXTBUM = %s # %s\n",s,comment);

	bitpix = read_fits_header_int(data, "BITPIX", comment);
	printf("\nBITPIX = %d # %s\n",bitpix,comment);
	bitpix = read_fits_header_int(data, "INTBUM", comment);
	printf("INTBUM = %d # %s\n",bitpix,comment);

	bscale = read_fits_header_int(data, "BSCALE", comment);
	printf("\nBSCALE = %f # %s\n",bscale,comment);
	bscale = read_fits_header_int(data, "FLOATBUM", comment);
	printf("FLOATBUM = %f # %s\n",bscale,comment);

	printf("\nMAKING COPY OF FILE......\n\n");
	fflush(stdout);
	copy = copy_fits(data);

	write_fits_header_int(&copy, "FRAME", 23, "THIS IS A NEW FRAME");
	write_fits_header_int(&copy, "NEwfrAME12345", 32, "THIS IS A NEW ENTRY WHICH IS MUCH TOO LONG TO FIT AND SO WE CAN BUT HOPE THAT IT WILL BE TRUNCATED AT THE RIGHT PLACE");
	write_fits_header_float(&copy, "BSCALE", 3.142, "THIS IS A NEW BSCALE");
	write_fits_header_float(&copy, "FNEwfrAME12345", 32, "THIS IS A NEW ENTRY WHICH IS MUCH TOO LONG TO FIT AND SO WE CAN BUT HOPE THAT IT WILL BE TRUNCATED AT THE RIGHT PLACE");
	write_fits_header_text(&copy, "OBJECT", "I\'m an object", "THIS IS A NEW OBJECT");
	write_fits_header_text(&copy, "TNewframe12345", "sadkjfh sdfkjh sdfkjh sdkfjh sdkjfh sdkjfh sdkfjh sdkjfh sdkfjhdsfkjhFUmmmmmblely", "THIS IS A NEW ENTRY WHICH IS MUCH TOO LONG TO FIT AND SO WE CAN BUT HOPE THAT IT WILL BE TRUNCATED AT THE RIGHT PLACE");
	print_fits_header(copy);

	bitpix = read_fits_header_int(data, "BITPIX", comment);
	printf("Writing FITs file test.fits.\n");
	fflush(stdout);

	bscale = read_fits_header_int(data, "BSCALE", comment);
	bzero = read_fits_header_int(data, "BZERO", comment);
	write_fits(data, "test.fits");
	free_fits_data(copy);
#endif

#ifdef DISPLAY

	/* Set up the X windows stuff */

	if (argc > 2)
		initX(argv[2]);
	else
		initX(NULL);

	theWindow = openWindow(argv[1],theWidth-data.nx-16,
				16,data.nx,data.ny);
	picture = make_picture(data.nx,data.ny,1,data.data,LIN);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture,data.nx,data.ny,theBitmapPad, 0);

	/* Display the image */

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,data.nx,data.ny);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
#endif

	printf("Image maximum = %f ", max_image_intensity(data.data, 
		data.nx, data.ny, &xpos, &ypos));
	printf("(%d, %d)\n", xpos, ypos);
	printf("Image minimum = %f ", min_image_intensity(data.data, 
		data.nx, data.ny, &xpos, &ypos));
	printf("(%d, %d)\n", xpos, ypos);
	printf("Total = %f\n",total_image(data.data, data.nx, data.ny));

	if (!read_fits_datasec(data,&xmin,&xmax,&ymin,&ymax))
	{
		printf("Could not read DATASEC.\n");
	}
	else
	{
		printf("DATASEC: xmin = %d xmax = %d ymin = %d ymax = %d\n",
			xmin,xmax,ymin,ymax);
	}

	if (!read_fits_ccdsec(data,&xmin,&xmax,&ymin,&ymax))
	{
		printf("Could not read CCDSEC.\n");
	}
	else
	{
		printf("CCDSEC: xmin = %d xmax = %d ymin = %d ymax = %d\n",
			xmin,xmax,ymin,ymax);
	}

	if (!read_fits_biassec(data,&xmin,&xmax,&ymin,&ymax))
	{
		printf("Could not read BIASSEC.\n");
	}
	else
	{
		printf("BIASSEC: xmin = %d xmax = %d ymin = %d ymax = %d\n",
			xmin,xmax,ymin,ymax);
	}
#ifdef DISPLAY
	printf("Type a key then return to exit.\n");

	getchar();


	/* Clean up the X windows stuff */

	XDestroyImage(theImage);
	XDestroyWindow(theDisplay, theWindow);
	quitX();

	/* Clean up memory and leave */

#endif

	free_fits_data(data);
	exit(0);
}
