#!/usr/bin/python

from matplotlib.mlab import bivariate_normal
from matplotlib.mlab import meshgrid
from pylab import *
import mpfit2 as mpfit
from numarray import *
from numarray import linear_algebra
from random import randrange

def residuals(p,fjac=None,x=None,y=None,err=None):
	
	base,amp,sigmax,sigmay,mux,muy = p

	model = base + amp * exp(-0.5*(((x[:,0]-mux)**2)/(sigmax**2) + ((x[:,1]-muy)**2)/(sigmay**2))) 
	status = 0
	return([status, (y-model)/err])
	
def gauss_fit(intensity=0,x_pos=0,y_pos=0):

	#intensity is a 2d array and x_pos and y_pos are 1d
	#make all the same dimension and length

	#insert test function to be replaced

	#x_pos = arange(50000.,55000.,100.)
	#y_pos = arange(30000.,34000.,100.)

	#x_mesh,y_mesh = meshgrid(x_pos,y_pos)

	#base0 = 20.
	#amp0 = 50.
	#sigma0 = 1000
	#sigmax0 = 1000.
	#sigmay0 = 1000.
	#mux0 = 52500.
	#muy0 = 32000.

	#intensity = base0 + amp0 * exp(-0.5*(((x_mesh-mux0)**2)/(sigma0**2) + ((y_mesh-muy0)**2)/(sigma0**2)))
	
	intensity = array(intensity)
	x_pos = array(x_pos)
	y_pos = array(y_pos)

	x = zeros(len(intensity.flat))
	y = zeros(len(intensity.flat))
	z = intensity.flat
	
	for i in range(len(y_pos)):
		for j in range(len(x_pos)):
			x[i*len(x_pos)+j] = x_pos[j]
			y[i*len(x_pos)+j] = y_pos[i]
			#z[i*len(x_pos)+j] = intensity[i][j]	

	#p contains the guesses
    	
    	# gets the index of the maximum of z (1d array)
    	iMax=argmax(z)
    	# gets the x coordinate of the maximum
    	mux1=x[iMax]
    	# gets the y coordinate of the maximum
    	muy1=y[iMax]
    	#guess sigma is ~5 microns
	sigmax1 = 5 * 250 #steps per micron
	sigmay1 = 5 * 250
	#guess amp is highes value
	amp1 = z.max()
	#guess that the base is the lowest value
	base1 = z.min()

	#actual values used in the function
	#p1 = array([base0,amp0,sigmax0,sigmay0,mux0,muy0])

	#guess values 
	p2 = [base1,amp1-base1,sigmax1,sigmay1,mux1,muy1]
 
	err = 1.0
	
	parinfo=[]
	for i in range(6): 
		parinfo.append({'value':p2[i],'fixed':False,'limited':[1,0],'limits':[0.,0.],'mpprint':1})
	
	parinfo[2]['limited']=[1,1]
	parinfo[2]['limits']=[2000.0,5000.0]

	parinfo[3]['limited']=[1,1]
	parinfo[3]['limits']=[2000.0,5000.0]

	parinfo[4]['limited']=[1,1]
	parinfo[4]['limits']=[0.,128000.]

	parinfo[5]['limited']=[1,1]
	parinfo[5]['limits']=[0.,128000.]

	#print 'parinfo: ',parinfo
	
	#parinfo[0]['fixed']=True
	#parinfo[1]['fixed']=True
	#parinfo[2]['fixed']=True
	#parinfo[3]['fixed']=True
	
    	xy=transpose(reshape(concatenate((x,y)),(2,shape(x)[0])))
    	
	fa={'x':xy, 'y':z, 'err':err}
    	
    	#print 'actual (p1): ',p1
	print 'guesses(p2): ',p2
	m=mpfit.mpfit(residuals,p2,parinfo=parinfo,functkw=fa,quiet=0, ftol=1.e-2, xtol=1.e-2, gtol=1.e-2)
    	print 'fit mparams: ', m.params	
	
	base,amp,sigmax,sigmay,mux,muy = m.params
		
	#for the fit use more data points, using delta
	delta = (x_pos[1]-x_pos[0])/5.
	x_fit = arange(x_pos.min(),x_pos.max()+delta,delta)
	y_fit = arange(y_pos.min(),y_pos.max()+delta,delta)
	x_fit_mesh,y_fit_mesh = meshgrid(x_fit,y_fit) 
	intensity_fit = base + amp*exp(-0.5*(((x_fit_mesh-mux)**2)/(sigmax**2) + ((y_fit_mesh-muy)**2)/(sigmay**2)))
	
	corner_delta = (x_pos[1]-x_pos[0])*0.5
	#extent = x_pos.min()-corner_delta,x_pos.max()+corner_delta,y_pos.min()-corner_delta,y_pos.max()+corner_delta,

	#imshow(intensity,origin='lower',cmap=cm.gray,extent=extent)
	#contour(x_fit,y_fit,intensity_fit,origin='lower')#,extent=extent)
	#show()

	#print 'intensity_fit - intensity: ',z-intensity_fit.flat
	
	return intensity_fit,m.params
