/*luminos.c*/

/* This c file holds the functions to be converted to Python commands which will control the Luminos mounts */
/* Use c++ for compiling */

#include <iostream>  /* for c++ routines if necessary */
#include <stdio.h>   /* Standard input/output definitions */
#include <stdlib.h>  /* Standard C library routines */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions for RS-232*/
#include <string.h>  /* String function definitions */
#include <sys/uio.h>

struct termios old_termios; // Old RS-232 settings
int fd; // file descriptor for sending data to /dev/tty (RS-232)

/* Functions */
int configure();
int renumber();
int index_position(char path[20], int stages);
int save_position(char path[20], int stages);
int recall_position(char path[20], int stages);
int set_position( int stage, int axis, long int position);
int command( int device, int command, long int data);
int home(int stage, int axis);
long int absolute_position( int stage, int axis );
int move_absolute( int stage, int axis, long int position );
int move_relative( int stage, int axis, long int position );

int main()
{
	return 0;
}

/* Configure RS-232 port for communiction */
int configure()
{
	/* Get the file descriptor for ttyS0 RS-232 port */
	/*fd = open( "/dev/ttyS0" , O_RDWR | O_NOCTTY);*/
	  fd = open( "/dev/ttyS1" , O_RDWR | O_NOCTTY);

	if( fd <= 0 )
	{
		printf("open error %d %s\n", errno, strerror(errno));
		return fd;
	}
	else
	{

		int c;
		struct termios new_options;

		/* Save the current options for the port to be reinstated at close */
		tcgetattr(fd, &old_termios); 

		bzero(&new_options, sizeof(new_options));

		//new_options.c_cflag = B9600;  /*Set baud rate to 9600*/
		cfsetispeed(&new_options, B9600);
		cfsetospeed(&new_options, B9600);
		new_options.c_cflag |= (CLOCAL | CREAD);
		new_options.c_cflag &= ~CRTSCTS;/*Disable hardware flow control*/
		new_options.c_cflag &= ~PARENB; /*No parity*/
		new_options.c_cflag &= ~CSTOPB;
		new_options.c_cflag &= ~CSIZE; /*Mask the bit size*/
		new_options.c_cflag |= CS8; /* 8 bit, 1 stop bit */
		/*new_options.c_cflag = INLCR; /* Do not convert line feeds or carriage returns*/
		/*new_options.c_cflag = ICRNL; /* Map CR to NL */
		/*new_options.c_lflag = ICANON;*/
		/*new_options.c_iflag = ICRNL;*/
		new_options.c_lflag = 0; /* Non-canonical mode, no echo */
		new_options.c_iflag &= ~(IXON | IXOFF | IXANY); /* Disable software flow control*/
		new_options.c_cc[VTIME] = 50; /* If no data comes back in 1 ms, read aborted */
		new_options.c_cc[VMIN] = 0; /* Blocking until 6 chars received */

		/* Flush the modem line */
		tcflush(fd, TCIFLUSH); 

		/* Instate the new options */
		tcsetattr(fd, TCSANOW, &new_options);
		
		return 1; 
	}
}

int restart()
{
        if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6); //output set to zero
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(0);
	o_buf[1] = char(0);
	
	tcflush(fd,TCIFLUSH);
	write(fd, o_buf, 6);
	usleep(2000000);
	
	return 1;	
}

int renumber()
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6); //output set to zero
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(0);
	o_buf[1] = char(2);
	
	tcflush(fd,TCIFLUSH);
	write(fd, o_buf, 6);
	usleep(2000000);
	
	int stages = 0;
	int call;
	
	o_buf[1] = char(60);
		
	do{
		o_buf[0]=char(3*stages+1);
		memset(i_buf, char(0), 6);
		tcflush(fd, TCIFLUSH);
		write(fd, o_buf, 6);
		tcflush(fd, TCIFLUSH);
		read(fd,i_buf,6);
		if(i_buf[1] != 0) stages++;
	}while(i_buf[1] != 0);
	return stages;	
}

int index_position(char path[20], int stages)
{
	int e1, e2; //error holders
	int n=3*stages; //number of axes
	
	unsigned char o_buf[6];
	o_buf[1] = char(45);
	//unsigned char i_buf[6];
			
	if( configure() < 0 )
	{
		printf("Could not open RS-232 interface.  Reset permissions on /dev/ttyS0\n");
		close(fd);
		return -1;
	}
		
	//renumber();
	
	//char path[20];
	//printf("Please the a file name with the positions for this configuration: ");
	//cin >> path;
	
	FILE *pos = fopen(path, "r");
	
	if( pos == NULL )
	{
		printf("File name invalid\n");
		return -1;
	}
				
	//unsigned char position_char[4];
	unsigned char output_data[4];
	//char line[20];
	long int li;
	
	for(int i=1;i<=n;i++)
	{
		char line[20];
		int index = -1;
		do{
			index++;
			line[index] = getc (pos);
		}while(line[index] != '\n');
		//getline(read(pos, position_char, 4);
		li = strtol(line,NULL,10);//convert string to long int
		memcpy(output_data, &li, 4);//convert long int to array of chars
		o_buf[0] = char(i);
		//printf("%i.) the string is %s\nthe line is %li\n",i,line,li);
		for(int j=0;j<4;j++)
			o_buf[j+2] = output_data[j];
		write(fd,o_buf,6);
	}
			
	fclose(pos);  /* close the file prior to exiting the routine */
		
	printf("Luminos stages have been renumbered and programmed \nto the positions specified by %s.\n",path);
	return 1;
}

int save_position(char path[20], int stages)
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure().\n");
		return -1;
	}
	
	int e1, e2;
	
	//char path[20];
	//printf("Please enter a file name for this configuration: ");
	//cin >> path;
		
	long int position;
	//unsigned char position_char[4];
	char line[8];
	
	//create file in necessary, overwrite if it exists
	int pos = open( path , O_WRONLY | O_TRUNC | O_CREAT );
	if(pos<1)
	{
		printf("Please choose another file name.\n");
		return -1;
	}
	//set permissions
	chmod( path , S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP );
						
	for(int h=1;h<=stages;h++)
		for(int i=1;i<=3;i++)
		{	
			position = absolute_position( h , i );
			//memcpy(position_char, &position, 4);
			sprintf(line,"%li\n",position);
			write(pos,line,strlen(line));
		}
					
	//tcsetattr(fd, TCSADRAIN, &old_termios);
	
	close(pos);
	
	printf("Luminos positions have been saved to %s.\n",path);
		
	return 1;
}

int recall_position(char path[20], int stages)
{
	int e1, e2; //error holders
	int n=3*stages; //number of axes
	
	unsigned char o_buf[6];
	o_buf[1] = char(20);
	//unsigned char i_buf[6];
			
	if( configure() < 0 )
	{
		printf("Could not open RS-232 interface.  Reset permissions on /dev/ttyS0\n");
		close(fd);
		return -1;
	}
		
	//renumber();
	
	//char path[20];
	//printf("Please the a file name with the positions for this configuration: ");
	//cin >> path;
	
	FILE *pos = fopen(path, "r");
	
	if( pos == NULL )
	{
		printf("File name invalid\n");
		return -1;
	}
				
	//unsigned char position_char[4];
	unsigned char output_data[4];
	//char line[20];
	long int li;
	
	for(int i=1;i<=stages;i++)
	{
	  for(int j = 1; j<=3;j++)
	    {
	        char line[20];
		int index = -1;
		do{
			index++;
			line[index] = getc (pos);
		}while(line[index] != '\n');
		//getline(read(pos, position_char, 4);
		li = strtol(line,NULL,10);//convert string to long int
		move_absolute(i,j,li);
		//memcpy(output_data, &li, 4);//convert long int to array of chars
		//o_buf[0] = char(i);
		//printf("%i.) the string is %s\nthe line is %li\n",i,line,li);
		//for(int j=0;j<4;j++)
		//	o_buf[j+2] = output_data[j];
		//write(fd,o_buf,6);
	    }
	}
			
	fclose(pos);  /* close the file prior to exiting the routine */
		
	printf("Luminos stages have been moved to the positions specified by %s.\n",path);
	return 1;
}

int set_position( int stage, int axis, long int position)
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	int device = 3*stage-3+axis;
	int command = 45;
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6); //output buffer set to zero
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(device);
	o_buf[1] = char(command);
	
	//Split 4-byte long int into 4-char array
	unsigned char output_data[4];
	memcpy(output_data, &position, 4);
	
	for(int i=0;i<4;i++)	o_buf[i+2] = output_data[i];
	
	write(fd, o_buf, 6);
	tcflush(fd, TCIFLUSH);
	if(device == 0)
	{
		printf("Please use save_position(filename) to save all axes' postions to a file\n");
		return -1;
	}
	else
	{
		//do{
			read(fd, i_buf, 6);
		//}while(i_buf[0] != char(device) || i_buf[1] != char(command));
	}
	
	if (i_buf[0] == 0 && i_buf[1] == 0)
	{
		printf("Requested actuator did not reply\n");  
		return -1;
	}
	
	unsigned char input_data[4];
	for(int j=0;j<4;j++)	input_data[j] = i_buf[j+2];		
		
	long int returned_data;
	
	memcpy(&returned_data, input_data, 4);
	
	if( int(i_buf[1]) == 255 )
	{
		printf("Requested position setting of stage %i and axis %i is out of range.\n",stage,axis);
		return -1;
	}
	else
		return 1;
}

int command(int device, int command, long int data)
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6);
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(device);
	o_buf[1] = char(command);
		
	//Split 4-byte long int into 4-char array
	unsigned char output_data[4];
	memcpy(output_data, &data, 4);
	
	for(int i=0;i<4;i++)	o_buf[i+2] = output_data[i];
	
	write(fd, o_buf, 6);
	tcflush(fd, TCIFLUSH);
	if(device == 0)
		read(fd, i_buf, 6);
	else
	{
		//do{
			read(fd, i_buf, 6);
		//}while(i_buf[0] != char(device) || i_buf[1] != char(command));
	}
	
	if (i_buf[0] == 0 && i_buf[1] == 0)
	{
		printf("Requested actuator did not reply\n");  
		return -1;
	}
	
	unsigned char input_data[4];
	for(int j=0;j<4;j++)	input_data[j] = i_buf[j+2];		
		
	long int returned_data;
	
	memcpy(&returned_data, input_data, 4);
	
	int returned_device = int(i_buf[0]);
	int returned_command = int(i_buf[1]);
	
	printf("Returned signal is: \n %i %i %li\n",returned_device,returned_command,returned_data);
		
	return 1;	
}

int home(int stage, int axis)
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6);
	//memset(i_buf, char(0), 6); //input set to zero
	
	o_buf[1] = char(1); //command for home
	
	if (stage == 0) 
		o_buf[0] = char(0);
	else 
	{
		if (axis == 0)
		{
			for(int i=1;i<=3;i++)
			{
				o_buf[0] = char(3*(stage - 1)+i);	
				write(fd, o_buf, 6);
				read(fd, i_buf, 6);
			}
			return 1;
		}
		else
			o_buf[0] = char(3*(stage - 1)+axis);
		
	 }
				
	write(fd, o_buf, 6); 
        read(fd,i_buf,6);
	return 1;
}

long int absolute_position( int stage, int axis )
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6);
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(3*stage-3+axis);
	o_buf[1] = char(60);
		
	write(fd, o_buf, 6);
	tcflush(fd, TCIFLUSH);
	//do{
		read(fd, i_buf, 6);
	//}while(i_buf[0] != char(3*stage-3+axis) || i_buf[1] != char(60));
	
	if (i_buf[0] == 0 && i_buf[1] == 0)
	{
		printf("Requested actuator did not reply\n");  
		return -1;
	}
		
	unsigned char input_data[4];
	for(int i=0;i<4;i++)	input_data[i] = i_buf[i+2];		
		
	long int returned_data;
	
	memcpy(&returned_data, input_data, 4);
	
	return returned_data;
}

int move_absolute( int stage, int axis, long int position )
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	int device = 3*stage-3+axis;
	int command = 20;
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	memset(o_buf, char(0), 6); //output buffer set to zero
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(device);
	o_buf[1] = char(command);
	
	//Split 4-byte long int into 4-char array
	unsigned char output_data[4];
	memcpy(output_data, &position, 4);
	
	for(int i=0;i<4;i++)	o_buf[i+2] = output_data[i];
	
	
	write(fd, o_buf, 6);
	tcflush(fd, TCIFLUSH);
	if(device == 0)
		read(fd, i_buf, 6);
	else
	{
		//do{
			read(fd, i_buf, 6);
		//}while(i_buf[0] != char(device) );
	}
	
	if (i_buf[0] == 0 && i_buf[1] == 0)
	{
		printf("Requested actuator did not reply\n");  
		return -1;
	}
	
	unsigned char input_data[4];
	for(int j=0;j<4;j++)	input_data[j] = i_buf[j+2];		
		
	long int returned_data;
	
	memcpy(&returned_data, input_data, 4);
	
	if( int(i_buf[1]) == 255 )
	{
		printf("Requested move of stage %i and axis %i is out of range.\n",stage,axis);
		return -1;
	}
	else
		return 1;
}

int move_relative( int stage, int axis, long int position )
{
	if(fd < 1)
	{
		printf("Please configure serial port first with configure()\n");
		return -1;
	}
	
	unsigned char o_buf[6];
	unsigned char i_buf[6];
	
	int device = 3*stage-3+axis;
	int command = 21;
	
	memset(o_buf, char(0), 6); //output buffer set to zero
	memset(i_buf, char(0), 6); //input set to zero
	o_buf[0] = char(device);
	o_buf[1] = char(command);
	
	//Split 4-byte long int into 4-char array
	unsigned char output_data[4];
	memcpy(output_data, &position, 4);
	
	for(int i=0;i<4;i++)	o_buf[i+2] = output_data[i];
	
	write(fd, o_buf, 6);
	tcflush(fd, TCIFLUSH);
	if(device == 0)
		read(fd, i_buf, 6);
	else
	{
		//do{
			read(fd, i_buf, 6);
		//}while(i_buf[0] != char(device));
	}
	//printf("Return data is %i %i\n",int(i_buf[0]),int(i_buf[1]));
	if (i_buf[0] == 0 && i_buf[1] == 0)
	{
		printf("Requested actuator did not reply\n");  
		return -1;
	}
	
	unsigned char input_data[4];
	for(int j=0;j<4;j++)	input_data[j] = i_buf[j+2];		
		
	long int returned_data;
	
	memcpy(&returned_data, input_data, 4);
	
	if( int(i_buf[1]) == 255 )
	{
		printf("Requested move of stage %i and axis %i, is out of range.\n",stage,axis);
		return -1;
	}
	else
		return 1;
}
