#!/usr/bin/env python
#
# $Id: setup.py,v 1.79 2005/08/23 23:41:41 epedrett Exp $
# Based on apiTest.c from Scott Streit 

import time, sys, os, Gnuplot, numarray.numeric
import threading
#from fcntl import *
from numarray import *
from numarray import strings
## from libastropciAPI import *
from numarray.fft import *    
from numarray.nd_image import *

class API:
########################################################################
#      CONSTRUCTOR, OBTAIN FILE DESCRIPTOR, ALLOC BUFFER 
########################################################################
    def __init__(self, **kwds):
	self.__dict__.update(kwds)
	self.graph = Graphics()
	self.pci_fd = openDriver(PCI_DEVICE_NAME)

#	if ((self.do_cds and self.cds) or not(self.do_cds)):
#	    self.col_length = (self.cols - self.colOffset * 2) * self.nreads
#	    self.row_length = (self.rows - self.rowOffset * 2) * 2
#	else:
#	    self.col_length = (self.cols - self.colOffset * 2) * self.nreads
#	    self.row_length = (self.rows - self.rowOffset * 2) 

# New allocation:
	if ((self.do_cds and self.cds) or not(self.do_cds)):
	    self.col_length = self.cols * self.nreads
	    self.row_length = self.rows * 2
	else:
	    self.col_length = self.cols * self.nreads
	    self.row_length = self.rows  


	    
#	self.bufferSize = 2 * (self.rows - 2* self.rowOffset) * (self.cols - 2 * self.colOffset) * 2
	self.bufferSize = 2200*2200*2
#	self.bufferSize = power(2,27)
#	self.bufferSize = self.rows * self.cols * 4
	self.lodDir = os.getenv("ASTROCAM")
	self.allocPCI()
	

########################################################################
#      DESTRUCTOR, FREE BUFFER 
########################################################################
    def __del__(self):
	print "Closing driver connection \n"
	print "Memory freed! \n"
	self.done()
	del(self.graph)

########################################################################
#	DO CONTROLLER SETUP
########################################################################
    def do_controller_setup(self):
	data = 1
	i = 0

	# -----------------------------------
	#   LOAD PCI FILE
	# ----------------------------------- 
	if self.do_pci_file:
	    print "Loading PCI file "+ self.lodDir + self.pci_file +"..."
	    if loadPciFile(self.pci_fd, self.lodDir + self.pci_file) == ERROR:
		print "failed. -> Could not load pci file.\n"
		raise IOError(repr(hex(getError())))
	    else:
		print "done.\n"

        #-----------------------------------
	# SET (HARDWARE) BYTE SWAPPING
        #----------------------------------- 
		self.byteSwapping = setHardwareByteSwapping(self.pci_fd)
		
	#-----------------------------------
        # RESET CONTROLLER
	#----------------------------------- 
	if self.do_reset:
	    print "Resetting controller..."
	    if hcvr(self.pci_fd, RESET_CONTROLLER, UNDEFINED, SYR) == ERROR:
		print "failed. -> Could not reset controller.\n"
		raise IOError(repr(hex(getError())))
	    else:
		print "done.\n"
	
	#-----------------------------------
	#  HARDWARE TESTS
	#-----------------------------------
	if self.do_hardware_test:
	    # PCI TESTS 
	    if self.do_pci_hardware_test:
		print "Doing " + repr(self.num_pci_tests) + " PCI hardware tests...\n"
		for i in range(self.num_pci_tests):
		    if doCommand1(self.pci_fd, PCI_ID, TDL, data, data) == ERROR: 
			print "\tERROR: " + hex(getError()) + "\n"	
		    data += HARDWARE_DATA_MAX/self.num_pci_tests

	     # TIMING TESTS
	    if self.do_tim_hardware_test:
		print "Doing " + repr(self.num_tim_tests) + " timing hardware tests...\n"
		for i in range(self.num_tim_tests):
		    if doCommand1(self.pci_fd, TIM_ID, TDL, data, data) == ERROR:
			print "\tERROR: " + hex(getError()) + "\n"	
		    data += HARDWARE_DATA_MAX/self.num_tim_tests
			
	     # UTILITY TESTS 
	    if (self.do_util_hardware_test):
		print "Doing " + repr(self.num_util_tests) + " utility hardware tests...\n" 
		for i in range(self.num_util_tests):
		    if doCommand1(self.pci_fd, UTIL_ID, TDL, data, data == ERROR):
			print "\tERROR: " + hex(getError()) + "\n"	
		    data += HARDWARE_DATA_MAX/self.num_util_tests
				
	#-----------------------------------
	# LOAD TIMING FILE/APPLICATION
	#----------------------------------- 
	    if (self.do_tim_file):
		print "Loading timing file " + self.lodDir + self.tim_file + "..." 
		loadFile(self.pci_fd, self.lodDir + self.tim_file, self.validate)
		print "done.\n"
	    elif self.do_tim_app:
		print "Loading timing application " + self.tim_app + "..." 
		if doCommand1(self.pci_fd, TIM_ID, LDA, self.tim_app, DON) == ERROR:
			print "failed -> " + hex(getError()) + "\n"
			raise IOError(repr(hex(getError())))
		else:
		    print "done.\n"

	#-----------------------------------
        # LOAD UTILITY FILE/APPLICATION
	#----------------------------------- 
	if self.do_util_file:
	    print "Loading utility file " + self.lodDir + self.util_file + "..."
	    loadFile(self.pci_fd, self.lodDir + self.util_file, self.validate)

	elif self.do_util_app:
		print "Loading utility application " + repr(self.util_app) + "...\n"
		if doCommand1(self.pci_fd, UTIL_ID, LDA, self.util_app, DON) == ERROR:
		    print "failed -> " + hex(getError()) + "\n"
		    raise IOError(repr(hex(getError())))
		else:
		    print "done.\n"

	#-----------------------------------
	#  POWER ON
	#----------------------------------- 
	if self.do_power_on:
	    print "Doing power on..."
	    if doCommand(self.pci_fd, self.controllerMaster, PON, DON) == ERROR:
		print "failed -> " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))
	    else:
		print "done.\n"

        #-----------------------------------
        # SET DIMENSIONS
        #----------------------------------- 
	if self.do_dimensions:
	    print "Setting image columns " + repr(self.cols) + "..."
	    if doCommand2(self.pci_fd, TIM_ID, WRM, (Y | 1), self.cols * self.nreads, DON) == ERROR:
		print "failed -> " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))
	    else:
		print "done.\n"
		print "Setting image rows " + repr(self.rows) + "..."
		if ((self.do_cds and self.cds) or not(self.do_cds)):
		    rows = self.rows * 2
		else:
		    rows = self.rows

		print ".. Corrected rows = %d\n" % rows 
		if doCommand2(self.pci_fd, TIM_ID, WRM, (Y | 2), rows, DON) == ERROR:
		    print "failed -> " + hex(getError()) + "\n"
		    raise IOError(repr(hex(getError())))
		else:
		    print "done.\n"

	#--------------------------------------
      	# READ THE CONTROLLER PARAMETERS INFO
	#-------------------------------------- 
	self.configWord = doCommand(self.pci_fd, TIM_ID, RCC, UNDEFINED)
	self.setControllerParameters()
    
########################################################################
#	SET CONTROLLER PARAMETERS
#
#	See the Voodoo Programmers Manual available at www.astr-cam.com
#	for a complete listing of available controller parameters.
#######################################################################
    def setControllerParameters(self):

	#-----------------------------
	# Set MPP mode
        #-----------------------------
	if (self.configWord & MPP_MASK) == MPP_CAPABLE:
	    self.do_mpp = TRUE
	else:
	    self.do_mpp = FALSE

	#-----------------------------
	# Set serial readout mode
        #-----------------------------
	if  ((self.configWord & READOUT_MASK) == SERIAL) or ((self.configWord & READOUT_MASK) == PARALLEL) or ((self.configWord & READOUT_MASK) == BOTH_READOUTS):
	    self.do_readout_mode = TRUE
	else:
	    self.do_readout_mode = FALSE

	#-----------------------------------
        # Set the integrator gain and speed
        #-----------------------------------
	if ((self.configWord & VIDEO_PROCESSOR_MASK) == CCD_REV3B):
		self.do_gain = TRUE
	else:
		self.do_gain = FALSE

	#-----------------------------
	# Set the array temperature
	#----------------------------- 
	if (self.configWord & TEMPERATURE_READOUT_MASK) != NO_TEMPERTURE_CONTROL:
	    self.do_temperature = TRUE

	    # Set temperature conversion algorithm. 
	    if ((self.configWord & TEMPERATURE_READOUT_MASK) == NONLINEAR_TEMP_CONV):
		contParams.temperature_algorithm = "nonlinear"

	    elif ((self.configWord & TEMPERATURE_READOUT_MASK) == LINEAR_TEMP_CONV):
		    self.temperature_algorithm = "linear"
	else:
	    self.do_temperature = FALSE

	#-----------------------------
	#  Set continuous readout
        #----------------------------- 
# This seems wrong. Should it be 0x100000 ?
# Yes it is. CONTINUOUS_READOUT contains 0x100000 so, use that and comment out old line.
#	if (self.configWord & 0x000005) == 0x000005:
	if (self.configWord & CONTINUOUS_READOUT) == CONTINUOUS_READOUT:
	    self.do_continuousReadout = TRUE
	    # Determine the FPB, where each new image starts on a 1024 address boundary 
	    self.framesPerBuffer = nFramesPerBuffer(self.mem_fd, self.bufferSize, self.row_length, self.col_length, self.bits_per_pixel)
	    print "Frames-Per-Buffer set to: " + repr(self.framesPerBuffer) + "\n"
	else:
	    self.do_continuousReadout = FALSE

	#-----------------------------------
	# Summary
        #----------------------------------- 
	print "\n"
	print "-----------------------------\n"
	print "Controller Parameters Summary\n"
	print "-----------------------------\n"
	print "Configuration Word: " + hex(self.configWord) + "\n"
	print "MPP AVAILABLE: " + repr(bool(self.do_mpp)) + "\n"
	print "READOUT MODES AVAILABLE: " + repr(bool (self.do_readout_mode)) + "\n"
	print "GAIN MODES AVAILABLE: " + repr(bool(self.do_gain)) + "\n"
	print "TEMPERATURE MODES AVAILABLE: " + repr(bool(self.do_temperature)) + "\n"


########################################################################
#	DO CONTROLLER PARAMETERS
########################################################################
    def doControllerParameters(self):

	#-----------------------------
        # Set MPP mode
        #----------------------------- 
	if (self.do_mpp):
	    print "Setting MPP mode: " + repr(self.mpp) + "...\n"
	    if (doCommand1(self.pci_fd, TIM_ID, MPP, self.mpp, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))
			
	#-----------------------------
	# Set serial readout mode
	#-----------------------------
	if (self.do_readout_mode):
	    ascii_value = to_ascii(self.amp)
	    print "Setting readout mode: %s...\n" % ascii_value
	    if (doCommand1(self.pci_fd, TIM_ID, SOS, self.amp, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

			
	#-----------------------------
	# Select Subarray mode
	#-----------------------------
	if (self.do_subarray_mode):
	# Row and column offset are swapped due to a documentation error
	    print "Setting subarray mode: row = %d col = %d...\n" % (self.rowOffset, self.colOffset)
	#   if (doCommand2(self.pci_fd, TIM_ID, SSA, self.rowOffset, self.colOffset, DON) == ERROR):
	    if (doCommand2(self.pci_fd, TIM_ID, SSA, self.colOffset, self.rowOffset, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------
	# Set number of reads per pixel
	#-----------------------------
	if (self.do_nreads):
	    print "Setting number of reads per pixel = %d...\n" % (self.nreads)
	    if (doCommand1(self.pci_fd, TIM_ID, SNR, self.nreads, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------
	# Set trigger mode
	#-----------------------------
	if (self.do_trigger):
	    print "Setting trigger mode = " + repr(bool(self.trigger)) + "\n"
	    if (doCommand1(self.pci_fd, TIM_ID, STM, self.trigger, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------
	# Set readout mode to CDS
	#-----------------------------
	if (self.do_cds):
	    print "Setting CDS mode = " + repr(bool(self.cds)) + "\n"
	    if (doCommand1(self.pci_fd, TIM_ID, CDS, self.cds, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------
	# Enable READ signal toggle
	#-----------------------------
	if (self.do_read):
	    print "Enable READ signal toggle = " + repr(bool(self.read)) + "\n"
	    if (doCommand1(self.pci_fd, TIM_ID, SRD, self.read, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------
	# Set number of frames per reset
	#-----------------------------
	if (self.do_framesPerReset):
	    print "Set frames per reset = %d \n " % (self.framesPerReset)
	    if (doCommand1(self.pci_fd, TIM_ID, SFR, self.framesPerReset, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------------
	#  Set the integrator gain 
        #-----------------------------------
	if (self.do_gain):
	    print "Setting gain: %3d, speed: %3d...\n" % (self.gain, self.speed)
	    if (doCommand2(self.pci_fd, TIM_ID, SGN, self.gain, self.speed, DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------------
	#  Set the bias voltages
	#-----------------------------------
	if (self.do_bias):
	    print "Setting PICNIC bias:"
	    print "VOFFSET: %6.3f V" % self.voffset 
	    print "VRESET: %6.3f V" % self.vreset
	    print "VD: %6.3f V" % self.vd
	    print "ICTL: %6.3f V\n" % self.ictl
	 
	    if (doCommand4(self.pci_fd, TIM_ID, SBN, 0, DAC2, VID,
			   (int) (VOLTS2WORD * self.voffset), DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))
		
	    if (doCommand4(self.pci_fd, TIM_ID, SBN, 0, DAC3, VID,
			   (int) (VOLTS2WORD * self.vreset), DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))
		
	    if (doCommand4(self.pci_fd, TIM_ID, SBN, 0, DAC4, VID,
			   (int) (VOLTS2WORD * self.vd), DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))
		
	    if (doCommand4(self.pci_fd, TIM_ID, SBN, 0, DAC5, VID,
			   (int) (VOLTS2WORD * self.ictl), DON) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	#-----------------------------
	# Set the array temperature
        #-----------------------------
	if (self.do_temperature):
	    # Set the array temperature algorithm
	    print "Setting temperature algorithm to: " + self.temperature_algorithm + "...\n"
	    if (set_algorithm(self.temperature_algorithm) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

	    # By default, the temperature algorithm is set to "nonlinear". If linear algorithm
	    # is required, set the algorithm to "linear" to set the coefficients.
	    if self.temperature_algorithm  == "linear":
		print "Setting linear temperature coeff0: %3d, coeff1: %3d...\n" %  (self.linear_temp_coeff0, self.linear_temp_coeff1)
		if (set_linear_coefficients(self.linear_temp_coeff0, self.linear_temp_coeff1) == ERROR):
		    print "\tERROR: " + hex(getError()) + "\n"
		    raise IOError(repr(hex(getError())))

	    print "Setting temperature: %2d deg C...\n" % self.temperature
	    if (doCommand2(self.pci_fd, UTIL_ID, WRM, (Y | 0x1C), calculate_adu(self.temperature), UNDEFINED) == ERROR):
		print "\tERROR: " + hex(getError()) + "\n"
		raise IOError(repr(hex(getError())))

#######################################################################
#	set exposure
#######################################################################

    def set_singleExposure(self):   
    
	# Set the shutter position 
	if self.do_open_shutter:
	    print "Setting shutter position: %d...\n" % self.do_open_shutter
	    if self.shutter_position() == ERROR:
		print "\tERROR: Setting shutter position failed: %X\n" % getError()
		raise IOError(repr(hex(getError())))
		
	

       	# Set the exposure time 
	print "Setting exposure time: %4.f sec.\n" % (self.exptime/1000.0)
	if self.controllerMaster == TIM_ID:
	    if doCommand1(self.pci_fd, TIM_ID, SET, self.exptime, DON) == ERROR:
		print "\tERROR: Setting exposure time failed -> %X\n" % getError()
		raise IOError(repr(hex(getError())))
	else:
	    if doCommand2(self.pci_fd, UTIL_ID, WRM, (Y | 0x18), self.exptime, DON) == ERROR:
		print "\tERROR: Setting exposure time failed -> %X\n" % getError()
		raise IOError(repr(hex(getError())))
	    



#######################################################################
#	DO EXPOSURE
#######################################################################
    def do_exposure(self):

	i = 0
	elapsed_time = 0
	maxPixelCount = self.row_length * self.col_length 
	currentPixelCount = intp()
	currentPixelCount.assign(0)
	lastPixelCount = 0
	exposureCounter = 0
	timeoutCounter = 0
	status = 0
	
	for i in range(self.number_of_exposures):
        
	    # Initialize all the variables. 
	    currentPixelCount.assign(0)
	    lastPixelCount = 0
	    exposureCounter = 0
	    timeoutCounter = 0
	    elapsed_time = 0
	    status = 0

	    # Start the exposure 
#	    print "Starting exposure.\n"
	    if doCommand(self.pci_fd, self.controllerMaster, SEX, DON) == ERROR:
		print "\tERROR: Starting exposure failed -> %X\n" % getError()
		raise IOError(repr(hex(getError())))
		
	    while (currentPixelCount.value() < maxPixelCount):
		status = (getHstr(self.pci_fd) & HTF_BITS) >> 3
#		print "status: %d\n" % status

	    ########################################
	    #	READ ELAPSED EXPOSURE TIME
	    ########################################
		if (status != READOUT and exposureCounter >= 20 and self.exptime > 1000):
		    if (self.controllerMaster == TIM_ID):
			elapsed_time = doCommand(self.pci_fd, TIM_ID, RET, UNDEFINED)
		    else:
			elapsed_time = doCommand1(self.pci_fd, UTIL_ID, RDM, (Y | 0x17), UNDEFINED)

#		    print "Elapsed Time: %d\n" % elapsed_time

			# Make sure the controller has started the timer. 
		    if (elapsed_time < 0):
			elapsed_time = 0
			    
		    exposureCounter = 0
		    timeoutCounter = 0
			    
		else:
		    exposureCounter = exposureCounter + 1

			########################################
			#	READOUT
			########################################


		    doIoctl(self.pci_fd,currentPixelCount)

		    # print "Current pixel count: %d\n" % currentPixelCount.value()

		    # Increment the timeout counter. 
		    if (currentPixelCount.value() == lastPixelCount):
			timeoutCounter = timeoutCounter + 1

		    lastPixelCount = currentPixelCount.value()

		    if timeoutCounter >= READ_TIMEOUT:
			print "ERROR: Readout TIMEOUT!\n"
			raise IOError("timeout")
			    
#		    time.sleep(0.00025)
		    time.sleep(0.025)

	    # Swap image data 
	    if not self.byteSwapping:
		swap_memory(self.mem_fd, maxPixelCount*(self.bits_per_pixel/8))

	    # Deinterlace image 
	    if self.do_deinterlace:
		print "Deinterlacing image: %d...\n" % self.deinterlace_mode
		if deinterlace(self.col_length, self.row_length, self.mem_fd, self.deinterlace_mode) == ERROR:
		    print "\tERROR: DEINTERLACE failed -> 0x%X\n" % getError()
		    raise IOError(repr(hex(getError())))
	
	    # Write FITS file 
	    if self.do_image_save:
		print "Writing FITS...\n"
		if writeFits(self.mem_fd, self.row_length, self.col_length, self.exptime, self.image_filename, self.image_directory) == ERROR:
		    raise IOError('fitserror')
            
	    self.arr = C2Numarray(self.mem_fd, self.row_length, self.col_length)
	    self.arr.byteswap()
	    self.img=self.arr[self.cols:,:] - self.arr[:self.cols,:]
	    self.img.transpose()


	    self.img[0,0] = 0
	    self.img[0,127] = 0
	    self.img[0,255] = 0
	    self.img[0,128] = 0
	    self.img[1,127] = 0
	    self.img[1,128] = 0
            self.img[1,255] = 0
	    self.img[1,0] = 0
            self.img[2,245] = 0
	    self.img[2,115] = 0
            self.img[8,116] = 0
	    self.img[27,88] = 0
	    self.img[31,139] = 0
            self.img[32,140] = 0
  	    self.img[32,139] = 0
            self.img[39,120] = 0
            self.img[79,106] = 0
            self.img[80,148] = 0
            self.img[80,149] = 0
            self.img[82,22] = 0
    	    self.img[84,149] = 0
            self.img[128,127] = 0
	    self.img[128,128] = 0
	    self.img[128,0] = 0
	    self.img[128,255] = 0
	    self.img[129,0] = 0
	    self.img[129,255] = 0
	    self.img[129,128] = 0
            self.img[129,127] = 0 
            self.img[130,255] = 0
            self.img[132,255] = 0
            self.img[167,246] = 0
            self.img[168,11] = 0
            self.img[176,221] = 0
            self.img[179,61] = 0
            self.img[187,78] = 0
            self.img[208,211] = 0
            self.img[208,210] = 0
            self.img[214,180] = 0
            self.img[219,173] = 0
            self.img[230,239] = 0
            self.img[249,217] = 0
            self.img[249,218] = 0
            self.img[249,219] = 0


#	    time.sleep(.3)			    		    

	    # Show image 
	    if self.do_image_show:
#		print "Showing image: %d...\n" % self.deinterlace_mode
#		tv(self.img, self.gp)
		self.graph.tvscl(self.img)




#######################################################################
#	DO CONTINUOUS READOUT (COADDITION) EXPOSURE
#######################################################################
    def doCoadditionExposure(self):

	pixelCount = self.col_length * self.row_length

	print "Setting FPB to %d\n" % self.framesPerBuffer
	if (doCommand1(self.pci_fd, TIM_ID, FPB, self.framesPerBuffer, DON) == ERROR): 
	    print "\tERROR: could not set frames-per-buffer\n"
	    raise IOError('FPB')
	
	# Set the number of frames 
	print "Setting number of frames to %d\n" % self.numberOfFrames
	if (doCommand1(self.pci_fd, TIM_ID, SNC, self.numberOfFrames, DON) == ERROR): 
	    print "\tERROR: could not set number of frames\n"
	    raise IOError('SNC')
	

	# Create and clear the accumulator image buffer 
	dest_fd = createBuffer(pixelCount)

	# Clear the image buffer mem_fd 
	zero_mem_fd(self.mem_fd, self.bufferSize)
		
	# Set the shutter position 
	if (self.do_open_shutter): 
	    print "Setting shutter position: %d...\n" % self.do_open_shutter
	    if (shutter_position() == ERROR): 
		print "\tERROR: Setting shutter position failed: 0x%X\n" % getError()
		destructBuffer(dest_fd)
		raise IOError(repr(hex(getError())))
		
	# Set the exposure time 
	print "Setting exposure time: %4.f sec.\n" % (self.exptime/1000.0)
	if (self.controllerMaster == TIM_ID): 
	    if (doCommand1(self.pci_fd, TIM_ID, SET, self.exptime, DON) == ERROR): 
		print "\tERROR: Setting exposure time failed -> 0x%X\n" % getError()
		destructBuffer(dest_fd)		
		raise IOError(repr(hex(getError())))
	else: 
	    if (doCommand2(self.pci_fd, UTIL_ID, WRM, (Y | 0x18), self.exptime, DON) == ERROR): 
		print "\tERROR: Setting exposure time failed -> 0x%X\n" % getError()
		destructBuffer(dest_fd)
		raise IOError(repr(hex(getError())))

	# Start the exposure 
	print "Starting exposure.\n"
	if (doCommand(self.pci_fd, self.controllerMaster, SEX, DON) == ERROR): 
	    print "\tERROR: Starting exposure failed -> 0x%X\n", getError()
	    destructBuffer(dest_fd)
	    raise IOError(repr(hex(getError())))
	
	# Start the coaddition 
	coaddition (self.pci_fd, self.framesPerBuffer, self.numberOfFrames, self.mem_fd, dest_fd, pixelCount, self.byteSwapping)

	# Deinterlace image 
	if self.do_deinterlace: 
	    print "Deinterlacing image: %d...\n" % self.deinterlace_mode
	    if (deinterlace32(self.col_length, self.row_length, dest_fd, self.deinterlace_mode) == ERROR): 
		print "\tERROR: DEINTERLACE failed -> 0x%X\n" % getError()
		destructBuffer(dest_fd)
		raise IOError(repr(hex(getError())))

	self.arr=C2Numarray32(dest_fd, self.row_length, self.col_length)
	self.arr.byteswap()
	self.img=self.arr[self.col_length:,:]-self.arr[:self.col_length,:]
	self.img.transpose()


	# Show image 
	if self.do_image_show:
#	    print "Showing image: %d...\n" % self.deinterlace_mode
	    self.graph.tvscl(self.img)


	# Write FITS file 
	if self.do_image_save:
	    print "Writing FITS...\n"
	    if writeFits32(dest_fd, self.row_length, self.col_length, self.exptime, self.image_filename, self.image_directory) == ERROR:
		destructBuffer(dest_fd)
		raise IOError('fitserror')

	destructBuffer(dest_fd)

########################################################################
#	DO CONTINUOUS READOUT (WRITE-TO-DISK) EXPOSURE
########################################################################
    def doWriteToDiskExposure(self):

	print "Setting FPB to %d\n" % self.framesPerBuffer
	if (doCommand1(self.pci_fd, TIM_ID, FPB, self.framesPerBuffer, DON) == ERROR): 
	#   print "\tERROR: could not set frames-per-buffer\n"
	    raise IOError('\tERROR: could not set frames-per-buffer\n')
	
	if (self.numberOfFrames <= 2):
	    raise IOError('\tERROR: number of frames <= 2')

	# Set the number of frames 
	print "Setting number of frames to %d\n" % self.numberOfFrames
	if (doCommand1(self.pci_fd, TIM_ID, SNC, self.numberOfFrames, DON) == ERROR): 
	    # print "\tERROR: could not set number of frames\n"
	    raise IOError('\tERROR: could not set number of frames\n')

	# Clear the image buffer mem_fd 
	zero_mem_fd(self.mem_fd, self.bufferSize)

	# Set the shutter position 
	if (self.do_open_shutter): 
	    print "Setting shutter position: %d...\n" % self.do_open_shutter
	    if (shutter_position() == ERROR): 
		# print "\tERROR: Setting shutter position failed: 0x%X\n", getError()
		raise IOError("Setting shutter position failed: " + hex(getError()) + "\n")
		
	# Set the exposure time 
	print "Setting exposure time: %4.f sec.\n" % (self.exptime/1000.0)
	if (self.controllerMaster == TIM_ID):
	    if (doCommand1(self.pci_fd, TIM_ID, SET, self.exptime, DON) == ERROR): 
		# print "\tERROR: Setting exposure time failed -> 0x%X\n", getError()
		raise IOError("Setting exposure time failed: " + hex(getError()) + "\n")
	
	elif (doCommand2(self.pci_fd, UTIL_ID, WRM, (Y | 0x18), self.exptime, DON) == ERROR): 
	    # print "\tERROR: Setting exposure time failed -> 0x%X\n", getError()
	    raise IOError("Setting exposure time failed: " + hex(getError()) + "\n")

	# Start the exposure 
	print "Starting exposure.\n"
	if (doCommand(self.pci_fd, self.controllerMaster, SEX, DON) == ERROR): 
	    # print "\tERROR: Starting exposure failed -> 0x%X\n", getError()
	    raise IOError("\tERROR: Starting exposure failed: " + hex(getError()) + "\n")
	

	# -----------------------------------
	#	Start Write-To-Disk
	# ----------------------------------- 
	writeToDisk(self.pci_fd, self.mem_fd, self.image_directory, 
		    self.image_filename, self.row_length, self.col_length,
		    self.numberOfFrames, self.framesPerBuffer, self.exptime, self.byteSwapping,
		    self.do_deinterlace, self.deinterlace_mode)


#######################################################################
#	DO CUSTOM (MIRC) EXPOSURE
#######################################################################
    def doCustomExposure(self):

	self.pixelCount = self.col_length * self.row_length

	print "Setting FPB to %d\n" % self.framesPerBuffer
	if (doCommand1(self.pci_fd, TIM_ID, FPB, self.framesPerBuffer, DON) == ERROR): 
	    print "\tERROR: could not set frames-per-buffer\n"
	    raise IOError('FPB')
	
	# Set the number of frames 
	print "Setting number of frames to %d\n" % self.numberOfFrames
	if (doCommand1(self.pci_fd, TIM_ID, SNC, self.numberOfFrames, DON) == ERROR): 
	    print "\tERROR: could not set number of frames\n"
	    raise IOError('SNC')

	# Clear the image buffer mem_fd 
	zero_mem_fd(self.mem_fd, self.bufferSize)
		
	# Set the shutter position 
	if (self.do_open_shutter): 
	    print "Setting shutter position: %d...\n" % self.do_open_shutter
	    if (shutter_position() == ERROR): 
		print "\tERROR: Setting shutter position failed: 0x%X\n" % getError()		
		raise IOError(repr(hex(getError())))
		
	# Set the exposure time 
	print "Setting exposure time: %.4f sec.\n" % (self.exptime/1000.0)
	if (self.controllerMaster == TIM_ID): 
	    if (doCommand1(self.pci_fd, TIM_ID, SET, self.exptime, DON) == ERROR): 
		print "\tERROR: Setting exposure time failed -> 0x%X\n" % getError()
		raise IOError(repr(hex(getError())))
	else: 
	    if (doCommand2(self.pci_fd, UTIL_ID, WRM, (Y | 0x18), self.exptime, DON) == ERROR): 
		print "\tERROR: Setting exposure time failed -> 0x%X\n" % getError()
		return ERROR

	# Start the exposure 
	print "Starting exposure.\n"
	if (doCommand(self.pci_fd, self.controllerMaster, SEX, DON) == ERROR): 
	    print "\tERROR: Starting exposure failed -> 0x%X\n", getError()


#######################################################################
#	SET SHUTTER POSITION
#######################################################################
    def shutter_position(self):

	currentStatus = 0

	# Ask for the current controller status.
	if (self.controllerMaster == TIM_ID):
	    currentStatus = doCommand1(self.pci_fd, TIM_ID, RDM, (X | 0), UNDEFINED)
	else:
	    currentStatus = doCommand1(self.pci_fd, UTIL_ID, RDM, (X | 1), UNDEFINED)

	if (self.expose.do_open_shutter):
	    if (self.controllerMaster == TIM_ID):
		if (doCommand2(self.pci_fd, TIM_ID, WRM, (X | 0), (currentStatus | OPEN_SHUTTER), DON) == ERROR):
		    return ERROR
		
		else:
		    if (doCommand2(self.pci_fd, UTIL_ID, WRM, (X | 1), (currentStatus | 0x1), DON) == ERROR):
			return ERROR
		
				
	elif ( not self.expose.do_open_shutter):
	    if (self.controllerMaster == TIM_ID):
		if (doCommand2(self.pci_fd, TIM_ID, WRM, (X | 0), (currentStatus & CLOSE_SHUTTER), DON) == ERROR):
		    return ERROR
		
	    else: 
		if (doCommand2(self.pci_fd, UTIL_ID, WRM, (X | 1), (currentStatus & 0xFFFFFFFE), DON) == ERROR):
		    return ERROR
			
	return NO_ERROR


########################################################################
#	DONE
#
#	Frees all handles/descriptors and exits the program.
########################################################################
    def done(self):
	closeDriver(self.pci_fd)
	free_memory(self.pci_fd, self.mem_fd)
	del(self.pci_fd)
	del(self.mem_fd)

########################################################################
#	TO ASCII
#
#	Converts a 24-bit integer into it's 3 character ascii equivalent
#	for a command. Used to output more descriptive DSP command
#	values. The input value is checked to be between ascii 0x20 and
#	0x7E.
########################################################################
def to_ascii(value):

    ascii_value = strings.array([chr((value & 0xff0000) >> 16),
				 chr((value & 0x00ff00) >> 8),
				 chr(value & 0x0000ff),
				 '\0'])

    if (ord(ascii_value[0]) < 0x20 or ord(ascii_value[0]) > 0x7E):
	ascii_value[0] = '?'

    if (ord(ascii_value[1]) < 0x20 or ord(ascii_value[1]) > 0x7E):
	ascii_value[1] = '?'

    if (ord(ascii_value[2]) < 0x20 or ord(ascii_value[2]) > 0x7E):
	ascii_value[2] = '?'

    return ''.join(ascii_value)


class Detector(API):

    def __init__(self):
        """
        Constructor, setting initial variables
        """

	API.__init__(self,	
		     # General Setup
		     do_image_show = TRUE
		     ,do_plot_show = FALSE
		     ,do_pspec_show = FALSE
		     ,pci_fd = 0
		     ,do_pci_file = TRUE
		     ,do_reset = TRUE
		     ,do_hardware_test =TRUE
		     ,do_pci_hardware_test = TRUE
		     ,do_tim_hardware_test = TRUE
		     ,do_util_hardware_test = FALSE
		     ,do_tim_app = FALSE
		     ,do_tim_file = TRUE
		     ,do_util_app = FALSE
		     ,do_util_file = FALSE
		     ,do_power_on = TRUE
		     ,do_dimensions = TRUE
		     ,bits_per_pixel = 16
		    # ,pci_file = "pciInt.lod"
		     ,pci_file = "pci.lod"
		     ,tim_file = "tim_readon.lod"
		     ,util_file = "util.lod"
		     ,tim_app = 0 
		     ,util_app = 0
		     ,num_pci_tests = 10
		     ,num_tim_tests = 10
		     ,num_util_tests = 10
		     ,cols = 256
		     ,rows = 256
		     ,controllerMaster = TIM_ID 
		     ,byteSwapping = FALSE
		     ,validate = TRUE
		     # Controller Parameters
		     ,configWord =  0
		     ,do_mpp = FALSE
		     ,do_readout_mode = TRUE 
		     ,do_subarray_mode = FALSE                   # Enable subarray readout
		     ,do_nreads = FALSE                          # Enable nreads
		     ,do_framesPerReset = FALSE                         # Enable loops 
		     ,do_trigger = FALSE                         # Enable external trigger
		     ,do_gain = FALSE
		     ,do_cds = TRUE                             # Enable correlated double sampling 
		     ,do_read = FALSE                            # Enable/disable READ signal toggle
		     ,do_bias = TRUE                            # Enable PICNIC bias load
		     ,do_temperature = FALSE 
		     ,do_continuousReadout = FALSE               # Enable continuous readout
		     ,mpp = FALSE                               
		     ,amp = ALL_AMP                             # Ascii ALL four amps (quad)
		     ,gain = TEN
		     ,rowOffset = 0                            # Subquadrant row offset
		     ,colOffset = 0                            # Subquadrant column offset
		     ,nreads = 1                                # Number of reads per pixel 
		     ,framesPerReset = 1                                # Number of reads per reset frame 
		     ,trigger = FALSE                           # External trigger active (or not)
		     ,cds = TRUE                                # Set CDS true/false
		     ,read = FALSE                              # Set READ toggle
		     ,speed = FAST
		     ,arrayLine = 32
		     ,temperature = -20 
		     ,temperature_algorithm = "nonlinear"
		     ,linear_temp_coeff0 = 0.0
		     ,linear_temp_coeff1 = 0.0
		     ,voffset = 4.0                            # Board 0, Pin 1 (DAC2) 
		     ,vreset = 0.6                              # Board 0, Pin 2 (DAC3) 
		     ,vd = 5.0                                  # Board 0, Pin 3 (DAC4) 
		     ,ictl = 3.3                                # Board 0, Pin 4 (DAC5) 
		     ,framesPerBuffer = 0
		     ,numberOfFrames = 1
		     # Exposure Parameters
		     ,do_open_shutter = FALSE
		     ,do_image_save = FALSE
		     ,do_deinterlace = TRUE
		     ,number_of_exposures = 1
		     ,exptime = 1000
		     ,image_directory = os.getenv("HOME")+"/frames/"
		     ,image_filename = "Image0.fit"
		     ,deinterlace_mode = CDS_QUAD_IR
		     )

    def set(self):
	self.setController()
	self.ldControllerParams()

########################################################################
# Set continuous readout mode
########################################################################
    def setCont(self,nreads=1,rows=256,cols=256,rowOffset=0,colOffset=0,numberOfFrames = 64,voffset = 3.97):
	self.tim_file = "tim.lod"
	self.exptime=0
	self.do_subarray_mode = TRUE
	self.do_nreads = TRUE
	self.do_framesPerReset = TRUE
	self.do_cds = TRUE			# enable flipping CDS state
	self.cds = FALSE			# CDS state is FALSE (no CDS)
	self.do_read = TRUE 
	self.do_continuousReadout = TRUE
	self.numberOfFrames = numberOfFrames
	self.rowOffset = rowOffset                       
	self.colOffset = colOffset        
	self.rows = rows
	self.cols = cols
	self.nreads = nreads 
	self.framesPerReset =  numberOfFrames 
	self.trigger = FALSE
	self.read = FALSE
	self.voffset = voffset
#	if ((self.do_cds and self.cds) or not(self.do_cds)):
#	    self.col_length = (self.cols - self.colOffset * 2) * self.nreads
#	    self.row_length = (self.rows - self.rowOffset * 2) * 2
#	    self.deinterlace_mode = CDS_QUAD_IR
#	else:
#	    self.col_length = (self.cols - self.colOffset * 2) * self.nreads
#	    self.row_length = (self.rows - self.rowOffset * 2) 
#	    self.deinterlace_mode = QUAD_IR
# New allocation
	if ((self.do_cds and self.cds) or not(self.do_cds)):
	    self.col_length = (self.cols) * self.nreads
	    self.row_length = (self.rows) * 2
	    self.deinterlace_mode = CDS_QUAD_IR
	else:
	    self.col_length = (self.cols) * self.nreads
	    self.row_length = (self.rows) 
	    self.deinterlace_mode = QUAD_IR



	self.setController()
	self.ldControllerParams()


    def __del__(self):
	API.__del__(self)


########################################################################
#      PCI BUFFER ALLOCATION
########################################################################
    def allocPCI(self): 
	self.mem_fd = uShortArray.frompointer(create_memory(self.pci_fd, self.row_length * 2, self.col_length, self.bufferSize))
	if self.mem_fd == NONE:
	    print "Unable to create image buffer: " + hex(self.mem_fd) + "\n"
	    print "\tERROR:" +  cvar.error_string + "\n"
	    print "Closing driver connection \n"
       	    self.done()
	    sys.exit()

########################################################################
#      GET INFO FROM SYSTEM, LOAD PCI FILE IN ASTROCAM
########################################################################
    def setController(self):
	self.byteSwapping = setHardwareByteSwapping(self.pci_fd)
	try:
	    self.do_controller_setup()
	except IOError, detail:
	    print "Retry to set controller ... \n"
	    self.do_controller_setup()
	    


#	if self.do_controller_setup() == ERROR:
#	    print "Closing driver connection\n"
#	    self.done()
#	    sys.exit()

########################################################################
#      LOAD CONTROLLER PARAMETERS IN ASTROCAM
########################################################################
    def ldControllerParams(self):
	try:
	    self.doControllerParameters()
	except IOError, detail:
	    print "Retry to load controller parameters ... \n"
	    self.doControllerParameters()

#	if self.doControllerParameters() == ERROR:
#	    print "Closing driver connection\n"
#	    self.done()
#	    sys.exit()

########################################################################
#      START SINGLE EXPOSURE
########################################################################
    def Exposure(self):
	try:
	    self.do_exposure()
	except IOError, detail:
	    print "Retry exposure ...\n"
	    self.do_exposure()

########################################################################
#      START COADDITION EXPOSURE
########################################################################
    def CoadditionExposure(self):
	try:
	    self.doCoadditionExposure() 
	except IOError, detail:
	    print "Retry exposure ...\n"
	    self.doCoadditionExposure()

########################################################################
#      START CONTINUOUS EXPOSURE
########################################################################
    def ContinuousExposure(self):
	try:
	    self.doWriteToDiskExposure() 
	except IOError, detail:
	    print "Retry exposure ...\n"
	    self.doWriteToDiskExposure()

########################################################################
#      START CUSTOM EXPOSURE
########################################################################
    def CustomExposure(self):
	try:
	    self.doCustomExposure()
	except IOError, detail:
	    print "Retry exposure ...\n" 
	    self.doCustomExposure()

########################################################################
#      ABORT EXPOSURE
########################################################################
    def AbortExposure(self):
	if doCommand(self.pci_fd, self.controllerMaster, ABR, DON) == ERROR:
	    print "failed -> " + hex(getError()) + "\n"
	    raise IOError(repr(hex(getError())))
	else:
	    print "done.\n"





class Thread(threading.Thread, Detector):
    """
    A sample thread class
    """
    def __init__(self):
        """
        Constructor, setting initial variables
        """
        self._stopevent = threading.Event()
        self._sleepperiod = .1

        threading.Thread.__init__(self, name="TheThread")
	Detector.__init__(self)
	self.pciFrameCount = intp()	
	self.totalFrameCount = intp()
	self.noOfFrames = intp()

    def __del__(self):
	Detector.__del__(self)


    def run(self):
        """
        overload of threading.thread.run()
        main control loop
        """
        print "%s starts" % (self.getName(),)

	i = 0
	m = 0
	frameCount = 1
	pixCount = 0
    	self.totalFrameCount.assign(0)
	self.pciFrameCount.assign(0)
	self.noOfFrames.assign(self.numberOfFrames)
        count = 0
	self.CustomExposure()
	byte_count = self.pixelCount * 4
        while not self._stopevent.isSet():
	    while (self.totalFrameCount.value() < self.noOfFrames.value()):
		pollAstropci(self.pci_fd, self.pciFrameCount, self.totalFrameCount, self.noOfFrames)

		# Swap the data if hardware byte swapping not available 
		if (not self.byteSwapping):
		    swap_memory(pixels + m, byte_count)


		m += self.pixelCount
		frameCount =frameCount + 1
		self.totalFrameCount.assign(self.totalFrameCount.value() + 1)

		# Determine where each new image starts on a 1024 address boundary (512 in
		# this case because we are working with pixels and not bytes). 
		if ((m % 512) != 0):
		    m = 512*(int(m/512) + 1)

		# Loop back to start of kernel image buffer (pixels[0]), and
		# re-initialize the frame count. 
		if (frameCount >= self.framesPerBuffer):
		    m = 0
		    frameCount = 1

		# Deinterlace image 
		if self.do_deinterlace:
		    if deinterlace((self.cols - 2 * self.colOffset), (self.rows - 2 * self.rowOffset) * 2, self.mem_fd, self.deinterlace_mode) == ERROR:
			print "\tERROR: DEINTERLACE failed -> 0x%X\n" % getError()
			raise IOError(repr(hex(getError())))
		    self.arr = C2Numarray(self.mem_fd, (self.rows - 2 * self.rowOffset) * 2, (self.cols - 2 * self.colOffset))
		    self.arr.byteswap()
		    self.img=self.arr[(self.cols - 2 * self.colOffset):,:] - self.arr[:(self.cols - 2 * self.colOffset),:]
		    self.img.transpose()
		    self.ft=real_fft2d(fourier_shift(self.img ,[(self.rows - 2 * self.rowOffset),(self.cols - 2 * self.colOffset)/2
]))
		    self.pspec=pow(abs(self.ft),2)

		    # Show image 
		    if self.do_image_show:
			self.graph.tvscl(self.img)
		    # Show plot 
			if self.do_plot_show:
			    self.graph.plot(self.img[:,self.arrayLine])

		    # Show PSPEC
		    if self.do_pspec_show:
			self.graph.tvscl(self.pspec)

	    
            self._stopevent.wait(self._sleepperiod)

        print "%s ends" % (self.getName(),)

    def stop(self,timeout=None):
        """
        Stop the thread
        """
        self._stopevent.set()
        threading.Thread.join(self, timeout)

class Graphics:

    def __init__(self):
	self.gp = Gnuplot.Gnuplot()
	self.pl = Gnuplot.Gnuplot()

    def __del__(self):
	del(self.gp)
	del(self.pl)

    def tvscl(self,img):
	a=img.min()
	b=img.max()
	self.strin='set cbrange ['+str(a)+':'+str(b)+']'
	self.gp('unset surface')
	self.gp('set view 0,0')
	self.gp('unset ztics')
	self.gp('set isosample 10,10')
	self.gp('set size square')
	self.gp('set pm3d')
	self.gp('set palette gray')
	self.gp(self.strin)
	self.gpdata = Gnuplot.GridData(img)
	self.gp.splot(self.gpdata)

    def tv(self,img, a, b):
	self.strin='set cbrange ['+str(a)+':'+str(b)+']'
	self.gp('unset surface')
	self.gp('set view 0,0')
	self.gp('unset ztics')
	self.gp('set isosample 10,10')
	self.gp('set size square')
	self.gp('set pm3d')
	self.gp('set palette gray')
	self.gp(self.strin)
	self.gpdata = Gnuplot.GridData(img)
	self.gp.splot(self.gpdata)

    def plot(self, y):    
	self.pl('set term x11')
	self.pl('set size 1,1')
	self.x = arange(y.size())
	self.pl('set autoscale xy')
	self.pl.plot(Gnuplot.Data(self.x, y, with = 'lines lt 1 lw 1'))


    
