/*
 * spooler.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __BOOLEAN__
#define __BOOLEAN__
enum BOOL {FALSE, TRUE};
typedef enum BOOL boolean;
#endif

#define NAXIS_LEN       3
#define GPS             "gps"
#define OPLE            "ople"
#define TIPTILT         "tiptilt"
#define SHUTTER         "shutter"
#define STATUSMSG       "status"
#define PORT            2000
#define ALTPORT         4000
#define BUFLEN          739
#define LLENGTH         19
#define SOCKNUM         4
#define NUM_THREADS	4
#define SLEEPTIME       1000000
#define N 1000
#define MEGEXTRA 1000000


/* Format of status message from ople server and format string used to parse it*/
/* Local Tm:  16:41 Lost T/S :      0/ 0OPLE Tm :       02:36 Msg/Data :   31/   0 */
/* CHARA Tm:  23:41                     Astromod: Not Used    Sts Reads:     17028 */
/* OPLE Cmd Status          Laser m       Target m      Error m   L FH FS HM BS BH */
/* S1   OFF_DONE          -0.000000298  +0.000000000  -0.000000298         X  X */
/* S2   OFF_DONE          +0.000000000  +0.000000000  +0.000000000         X  X */
/* E1   OFF_DONE          +0.000000000  +0.000000000  +0.000000000         X  X */
/* E2   OFF_DONE          +0.000000000  +0.000000000  +0.000000000         X  X */
/* W1   OFF_DONE          +0.000000000  +0.000000000  +0.000000000         X  X */
/* W2   OFF_DONE          +0.000000000  +0.000000000  +0.000000000         X  X */
/* COMMAND_COMPLETE:0 */

#define OPLEFMT   "%*[^\n] %*s%*s%*s%*s %s %*[^\n] %*[^\n]%*22c %lf %lf %lf %*[^\n] %*22c %lf %lf %lf %*[^\n] %*22c %lf %lf %lf %*[^\n] %*22c %lf %lf %lf %*[^\n] %*22c %lf %lf %lf %*[^\n] %*22c %lf %lf %lf %*[^\n] "

/* Format of status message from tiptilt server */

/* Local Tm:  16:41:33         SUM   DETrms MIRrms SERVO MOD SIN r0(cm) Seeing Snd */
/* CHARA Tm:  22:30:37    B1 0126130  0.9437 0.000  OFF       NO    0.0   0.00  ON */
/* Lost T/S:      0/XX    B2 0060242  0.258  0.000  OFF       NO    0.0   0.00  ON */
/* ExpTm mS:         1    B3 0075723  0.669  0.000  OFF       NO    0.0   0.00  ON */
/* Frms/Sec:    464.00    B4 0066127  0.847  0.000  OFF       NO    0.0   0.00  ON */
/* Frame Tp: SUB_FRAME    B5 0037738  0.2901 0.000  OFF       NO    0.0   0.00 OFF */
/* Frame Ps:    0    0    B6 0035685  0.6595 0.000  OFF       NO    0.0   0.00 OFF */
/* Frame Sz:   20   18    SIGNALS:                                              ON */
/* Npixels :       360 */
/* COMMAND_COMPLETE:0 */


/** Delay lines vector indexes*/
enum {
  S1_LASER,
  S1_TARGET,
  S1_ERROR,
  S2_LASER,
  S2_TARGET,
  S2_ERROR,
  E1_LASER,
  E1_TARGET,
  E1_ERROR,
  E2_LASER,
  E2_TARGET,
  E2_ERROR,
  W1_LASER,
  W1_TARGET,
  W1_ERROR,
  W2_LASER,
  W2_TARGET,
  W2_ERROR  
};


/* Temporary definitions for testing */
#define TIMERINT 30
#define MAX_SELECT_FD 20
#define SELECT_TIMEOUT 2*1000


static const char* mon[12] = {
  "Jan",
  "Feb",
  "Mar",
  "Apr",
  "May",
  "Jun",
  "Jul",
  "Aug",
  "Sep",
  "Oct",
  "Nov",
  "Dec"
};


/** Spooler messages*/
enum {
  HEADER,
  DATA,
  END,
  KILL
};

/* Sockets' file descriptors */
enum SOCK {
  GPSFD,
  OPLEFD,
  TIPTILTFD,
  SPOOLFD,
  SHUTTERFD
};

typedef enum SOCK tcpsock;

typedef void *(*PTF) (void *);

/* Structure to initialise threads */
struct thread_data{
  int  thread_id;
  tcpsock  socknum;
  char *message;
};
typedef struct thread_data threadData;

struct theHeader {
  unsigned short rows;
  unsigned short cols;
  unsigned short rowOffset;
  unsigned short colOffset;  
  unsigned short nreads;
  unsigned short numberOfCoadds;
  unsigned short framesPerReset;
  unsigned short exptime;
  unsigned long numberOfFrames;
  unsigned char do_image_save;
  float voffset;
  float vreset;  
  float vd;
  float ictl;
  char image_directory[30];
  char image_filename[20];
  char object[20];
  char observer[20];
  float lambda;
  float bandwidth;
  float tint;
  char comment[100];
 };

/* This structure is used to communicate data to the python interface and with the spooler */
typedef struct theHeader header_struct;

struct theHost {
  char hostname[30];
  int port;
};

typedef struct theHost host_struct;

static const host_struct gps = {
  GPS,
  PORT,
};

static const host_struct ople = {
  OPLE,
  PORT,
};

static const host_struct shutter = {
  SHUTTER,
  ALTPORT,
};

static const host_struct tiptilt = {
  TIPTILT,
  PORT,
};


