/*
 * astropci_usr.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* #include <linux/autoconf.h> */
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <getopt.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <math.h>
#include <fftw3.h>
#include <sched.h>
//#include <netdb.h>  // Added to make sockets work with chara code

#include <rtai/task.h>
#include <rtai/sem.h>
#include <rtai/pipe.h>
#include <rtai/timer.h>
#include <rtai/registry.h>

#include "DSPCommand.h"
#include "spooler.h"
#include "astropci_usr.h"
#include "loadDspFile.h"

#include "chara_messages.h"  
#include "cliserv.h" 
#include "ople_mirc.h" 
#include "dlCommands.h"
#include "misc.h"
#include "time.h"
#include <netdb.h>


int pci_fd = 0;
int pipeMinor = 0;
RT_TASK sem_task_ptr;
RT_TASK target_task_ptr;
RT_PIPE pipe_desc;

unsigned short *mem_fd, *dataBuffer;
daq_struct daq;
daqOut_struct daqOut;
exchange_struct exc = {
  0,        /* PixelCount */
  0,        /* byte_count_short */
  0,        /* byte_count_float */
  FALSE,    /* daqRunning */
  FALSE,    /* endProg */
  FALSE,    /* resetBkgnd */
  FALSE,    /* takeBkgnd */
  FALSE,    /* useBkgnd */
  C_IDLE,   /* cmd      */
  35,                          /* bkgndNoiseCol */
  {15,20,10,5,25,29},          /* cpeak */ 
  {0.0,0.0,0.0,0.0,0.0,0.0},   /* gdtWeight */
  {1 ,1 ,2.5 ,1 ,1 ,1},        /* gdtThreshold */
  {0.0,0.0,0.0,0.0,0.0,0.0},   /*  gdtOpd */
  {0.0,0.0,0.0},               /* gdtOptOpd */
  {-1,-1,+1,-1,+1,+1},         /* OPDSigns */
  {S2,E2,W1,W2},               /* BeamOrder 1234 */
  {0.0,0.0,0.0,0.0,0.0,0.0},           /* DLOffset */
  0,                            /*OffsetsReady */
  0,                            /*manOffsetsReady */
  W2,                        /* RefTel */
  1.65,                        /*  lambda */
  0.3,                          /* bandwidth */
  1024 ,                         /* framesPerBuffer */ 
  FALSE,                        /* abort */
  FALSE,                      /*  continuousReadout */
  "mav0",                 /*image_filename */
  "                "      /* string for calling spooler */
};

gdt_struct gdt = {
  {0.0,0.0,0.0,0.0,0.0,0.0},                 /* dlOffsets */
  {FALSE,FALSE,FALSE,FALSE,FALSE,FALSE},     /* activeBaselines */
  FALSE,                                      /* gdtLock */
  {0.0,0.0,0.0,0.0,0.0,0.0},                  /*manualOffset */  
  FALSE			/* continuousReadout */
};

double dloffsets[NUMDLS], last_dloffsets[NUMDLS];
double dloffsets_test[NUMDLS];
int DL_DEBUG=0;

/************************************************************************/
/* a = pointer to array, n = number of elements				*/
/*									*/
/* Function to sort an array     					*/
/************************************************************************/
void bubbleSort(float *a, unsigned int n)
{
  unsigned int i = 0, j = 0;
  float tmp;
  for (i=0; i<n-1; i++) {
    for (j=0; j<n-1-i; j++)
      if (a[j+1] < a[j]) {  /* compare the two neighbors */
	tmp = a[j];         /* swap a[j] and a[j+1]      */
	a[j] = a[j+1];
	a[j+1] = tmp;
      }
  }
}


//************************************************************************/
/* set_manual_target()							*/
/*									*/
/* Function to set a manual target.					*/
/************************************************************************/

void set_manual_target(int server, int dl, double pos)
{
  struct smessage mess;
  struct s_target target;
 
  /* Set out the command data */

  target.dl_number = (int) dl;
  target.position = (double) pos; /* meters */
  target.rate = (double) 0.0; // rate; /* meters per second */
  target.time = 0; //(unsigned long) opletime; /* Only important if rate != 0 */

  /* Send messages */
 
  mess.type = OPLE_MANUAL_TARGET;
  mess.length = sizeof(target);
  mess.data = (unsigned char *) &target;
	printf("Talking DIRECTLY to OPLE !!\n");
  if (!send_message(server, &mess))
    {
      fprintf(stderr,"Failed to set Manual target.");
    }

} /* set_manual_target() */

/************************************************************************/
/* set_manual_target()							*/
/*									*/
/* Function to set a manual target.					*/
/************************************************************************/

void vme_set_manual_target(int ople_socket, int dl_number, double position)
{
  /* this is essentially a direct copy of the routine ople_manual_target
     which can be found in control/modules/ople/ople_comms.c
  */

  double rate =0.0;
  OPLE_TIME time=0;
  ISIT_CMD command, command_send;
  int iGizmoId = 0;
  int delay_line_gizmo_id[NUM_DELAY_LINES] = {1, 2, 3, 4, 5, 6};
  char     *c1,*c2;      /* For copying things */
  int      i;            /* Loop variable */

    position = position *  LASER_SIGN/LASER_TO_METERS;

 /* Is this a valid delay line number? */
        
        if (dl_number < 0 || dl_number >= NUM_DELAY_LINES)
        {
                printf("Invalid delay line number %d.\n",dl_number);
                return ;
        }
      /* Rightio, construct the command */

        command.genericCmd.Command = INST_CMD_SET_MANUAL_TARGET;
        command.instsmtCmd.iDlGizmoId = delay_line_gizmo_id[dl_number];
        command.instsmtCmd.Target.dPosition = rint(position);
        command.instsmtCmd.Target.dRate = rint(rate);
        command.instsmtCmd.Target.tTime = time;
  
	/* next part is copy of send_ople_command() in ople_comms.c */

            /* Fill in some of the bits of Command structure */

        command.genericCmd.iGizmoId = iGizmoId;
        command.genericCmd.tTimeStamp =0; // can I get away with this? JDM

        /* copy the command structure over */

        for(c1 = (char *)&command, c2 = (char *)&command_send, i=0;
                i < (int)sizeof(command) ; i++, c1++, c2++) *c2 = *c1;

        /* First thing we do is check out what command this is */
	/* hey.. its INST_CMD_SET_MANUAL_TARGET */
	byte_swap(command_send.instsmtCmd.iDlGizmoId);
        byte_swap(command_send.instsmtCmd.Target.dPosition);
        byte_swap(command_send.instsmtCmd.Target.dRate);
        byte_swap(command_send.instsmtCmd.Target.tTime);

	/* byte swap the command itself */
        byte_swap(command_send.genericCmd.tTimeStamp);
        byte_swap(command_send.genericCmd.iGizmoId);
        byte_swap(command_send.genericCmd.iWhichRte);
        byte_swap(command_send.genericCmd.iWhichCpu);
        byte_swap(command_send.genericCmd.Command);

        /* Finally!!! we send it away */
	printf("Talking DIRECTLY to vme !!\n");
        if (write(ople_socket, &command_send, sizeof(command_send)) !=
                sizeof(command_send))
        {
                printf("Failed to send command to ople cage.");
                return;
        }



} /* vme set_manual_target() */


/************************************************************************/
/* set_manual_offsets()							*/
/*									*/
/* Function to set a manual target.					*/
/************************************************************************/

void set_manual_offsets(int server, double *pos)
{
        struct smessage mess;
 
        /* Send messages */

        mess.type = OPLE_MANUAL_OFFSETS;
        mess.length = 6 * sizeof(double);
        mess.data = (unsigned char *) pos;

        if (!send_message(server, &mess))
        {
		fprintf(stderr,"Failed to set Manual offsets.");
        }

} /* set_manual_offsets() */

/***********************************************************************
 *	Calculate frames per buffer 
 ***********************************************************************/
int nFramesPerBuffer(unsigned short *mem_fd, int bufferSize, int rows, int cols, int bits_per_pixel)
{
  int x, framesPerBuffer;
  unsigned short *imageEndAddr;

  x = 1;
  imageEndAddr = mem_fd + (rows*cols*(bits_per_pixel/8));
  while (((int)(imageEndAddr + x) % 1024) != 0)
    x++;

  framesPerBuffer = (int)floor((double)(bufferSize/(imageEndAddr - mem_fd + x)));
  /* The framesPerBuffer must be evenly divisible by 4 for this mode, which requires
     the buffer be sub-divided in quarters. */
  while (framesPerBuffer % 4 != 0)
    framesPerBuffer--;

  return framesPerBuffer;
} 

/*******************************************************************************
*
*	Function:
*	---------
*	int create_memory(int pci_fd, int rows, int cols)
*
*	Description:
*	------------
*       This function is used to allocate space to an image buffer.
*	The function returns a pointer to a block of at least 'image_size'
*	bytes suitably aligned for any use.
*
*	Parameters:     
*	-----------
*	pci_fd  	The reference to the PCI driver returned by open().
*	rows	  	The number of image rows (pixels).
*	cols	  	The number of image columns (pixels).
*	size		The size of the buffer in bytes (no affect on WIN2K).
*
*	Returns:        
*	--------
*	NULL if a not successfull, Pointer to image buffer otherwise.
*
*
*       Version:        1.00
*       Author:         Scott Streit
*       Date:           02/05/2001
*
*	Revision History:
*	-----------------
*	Date            Who   Version    Description
*	----------------------------------------------------------------------
*	02/05/2001      sds     1.00    Initial
*
*
*******************************************************************************/
unsigned short *create_memory(int pci_fd, unsigned int bufferSize)
{
  unsigned short *mem_fd;
  printf("\nCalling mmap ...\n");

  /* Map the kernel buffer */
  mem_fd = (unsigned short *)mmap(0, bufferSize, (PROT_READ | PROT_WRITE), MAP_SHARED, (int)pci_fd, 0);

  printf("\nmmap fd = 0x%X\n", (int) mem_fd);

  /* Check for an out of physical memory error. */
  if (mem_fd == NULL && errno == ENOMEM) {
    sprintf(error_string, "Error Allocating Image Buffers. The Systems \n Physical Limits Have Been Exceeded. \n");
    return NULL;
  }

  /* Check for an out of memory error. */
  else if (mem_fd == NULL && errno == EAGAIN) {
    sprintf(error_string, "Error Allocating Image Buffers. \n Not Enough Memory CURRENTLY Available. \n");
    return NULL;
  }

  /* Check for any other errors. */
  else if (mem_fd == NULL) {

    sprintf(error_string, "Error Allocating Memory (errno = %d) \n", errno);
    return NULL;
  }

  return mem_fd;

}

/***********************************************************************
 *	DEINTERLACE IMAGE
 ***********************************************************************/
int deinterlace(int cols, int rows, unsigned short *image_fd)
{
  unsigned short *old_iptr;
  unsigned short *new_iptr;
  int i = 0;
  int j = rows-1;
  int counter = 0;
  int end = 0;
  int begin = 0;

  old_iptr = image_fd;


  /* Allocate a new buffer to hold the deinterlaced image. */
  if((new_iptr = (unsigned short *) malloc(cols*rows*sizeof(unsigned short))) == NULL) {
    sprintf(error_string, "Error in allocating memory for deinterlacing. \nSaving interlaced image to disk.");
    free (new_iptr);
    return _ERROR;
  }

  if ( (float)cols/2 != (int)cols/2 || (float)rows/2 != (int)rows/2) {
    sprintf(error_string, "Number of cols AND rows must be EVEN for quad readout. \nSkipping deinterlace.");
    free (new_iptr);
    return _ERROR;
  }

  /*--------------- IR SPLIT QUAD READOUT --------------------------------------*/				
  while(i<cols*rows) {
    if (counter%(cols/2) == 0) {
      end = (j-(rows/2))*cols;
      begin = j*cols;
      j--; 			/*number of completed rows*/
      counter=0; 		/*reset for next convergece*/
    }

    *(new_iptr+begin+counter)       = *(old_iptr+i++);		/* front_row--->  */
    *(new_iptr+begin+(cols/2)+counter) = *(old_iptr+i++);	/* front_row<--   */
    *(new_iptr+end+(cols/2)+counter)   = *(old_iptr+i++);	/* end_row<----   */
    *(new_iptr+end+counter)         = *(old_iptr+i++);		/* end_row---->   */
    counter++;
  }

  memcpy(old_iptr, new_iptr, cols*rows*sizeof(unsigned short));
  free(new_iptr);
  
  return _NO_ERROR;
}

/***********************************************************************
 *	CUMULATORS FOR DFT
 *  src_fd  = mem_fd = camera data
 *  dets_fd = pointer containing 12(Sxy - 0.5(N+1)Sy)/(N(N^2-1))
 *  r       = number of rows
 *  c       = number of columns
 *  nr      = number of reads
 *  nca     = number of coadds
 *  cac     = coaddCount
 *  m       = frame number in mem_fd
 ***********************************************************************/

void psAccum(float *src_fd, float *oldBuffer, float *im_fd, float *dest_fd, unsigned short r, unsigned short c, 
	     unsigned short nr, fftwf_plan plan, unsigned int pixelCount, fftwf_complex *fft, float *bkgndBuffer)
{
  register float *ptr_src;
  register float *ptr_dest;
  register int i, j, ir;
  float C1; 
  // Add Reads and Do Difference -- filling im_fd single CDS buffer
  // Only do ONE QUAD!
  C1 = 1./(float)nr;

 
  //for(j=0,ptr_src=src_fd+m, ptr_dest = im_fd; j< r/2; j++){
  for(j=0,ptr_src=src_fd, ptr_dest = im_fd; j< r/2; j++){
    for(i=0;i<c/2; i++) {
      *(ptr_dest+j*c/2+i)=0;
      for(ir=1;ir<nr;ir++){ // KLUDGE: SKIP first ONE READ due to transient
	*(ptr_dest+j*c/2+i) +=( (float) *(ptr_src+j*c*nr+i*nr+ir) -
				(float) *(oldBuffer+j*c*nr+i*nr+ir) )*C1;
      }
      //background subtraction
      /* NOT USED NOW  *(ptr_dest+j*c/2+i) -=*(bkgndBuffer+j*c/2+i); */
    }
  }

// Currently the last couple rows have some hot pixels due to some
// data alignment problem we are still tracking down.
// as a work around, we will ignore the last 2 columns of data and replace with
// the value in a nearby pixel.

  /*      // FAKE FRINGES
	  for(j=0,ptr_dest = im_fd; j< r/2; j++){
	  for(i=0;i<c/2; i++) {
	  *(ptr_dest+j*c/2+i) = cos(-2.0*3.14* ((float)i)/10.0+2.0*3.14*((float)j)/10.0)+1;
	  }
	  }
  */
     
  // Do PS.
    
  fftwf_execute(plan);
  // accumulate in the correct buffer, but only fill half of array.
  for(j = 0, ptr_dest = dest_fd; j < (r/2); j++) {
    for(i = 0; i < c/4+1 ; i++)
      *(ptr_dest++) += pow(c_re((*(fft+j*(c/4+1)+i)) ),2) +  pow(c_im(*(fft+j*(c/4+1)+i)),2);
  }
} //end of psAccum


/***********************************************************************
 *	CUMULATORS FOR LINEAR FIT
 *  src_fd  = mem_fd = camera data
 *  dets_fd = pointer containing 12(Sxy - 0.5(N+1)Sy)/(N(N^2-1))
 *  r       = number of rows
 *  c       = number of columns
 *  nr      = number of reads
 *  nca     = number of coadds
 *  cac     = coaddCount
 *  m       = frame number in mem_fd
 ***********************************************************************/
void linearFit(float *src_fd, float *dest_fd, 
	       unsigned short r,unsigned short c, unsigned short nr, unsigned short nca, unsigned int cac, int m)
{
  register float *ptr_src;
  register float *ptr_dest;
  register int i, j;
  float c1, c2, n;
  // number of coadds = 1, no fit, add reads
  if (nca == 1){
    c1 = 1./nr;
    // for(i = 0, ptr_src = src_fd + m, ptr_dest = dest_fd; i < r*c; i++){
    for(i = 0, ptr_src = src_fd, ptr_dest = dest_fd; i < r*c; i++){
      // sum n read into one pixel
      for (j = 0; j < nr; j++)
	*(ptr_dest+i) += (float) *(ptr_src++)*c1;    
    } 
  }
  // number of coadds > 1, linear fit, add reads
  else {
    n = (float) nca;
    c1 = 0.5*(n + 1);
    c2 = 12./(n*(n*n - 1.)*nr);
    //compute c2 *(Sxy - c1Sy)
    //for(i = 0, ptr_src = src_fd + m, ptr_dest = dest_fd; i < r*c; i++){
    for(i = 0, ptr_src = src_fd, ptr_dest = dest_fd; i < r*c; i++){
      // sum n read into one pixel
      //kludge 00 skip first 1 read
      ptr_src=ptr_src+1;
      for (j = 1; j < nr; j++)
	*(ptr_dest+i) += (float) *(ptr_src++) * ((float)cac-c1) * c2;    
    }
  }
}


/*******************************************************************************
 *
 *	Function:
 *	---------
 *	void swapmemory_memory()
 *
 *	Description:
 *	------------
 *       used to byte swap the words within the data Dwords.
 *
 *	Parameters:     
 *	-----------
 *	mem_fd	The reference to the image buffer returned by create_memory().
 *	size    The number of bytes to swap.
 *
 *       Returns:        
 *	--------
 *	None.
 *
 *       Version:        1.00
 *       Author:         Scott Streit
 *       Date:           08/16/2001
 *******************************************************************************/
void swap_memory(unsigned short *mem_fd, int size)
{
  int x;
  unsigned int temp1;
  unsigned int temp2;
  unsigned int temp3;
  unsigned int temp4;
  unsigned short *fd = mem_fd;

  /* Check for a NULL image buffer. */
  if (mem_fd == NULL) {
    sprintf(error_string, "Error! Attempting To Access A NULL Image Buffer. \n");
    return;
  }

  /* Swap the image data bytes within the words. */
  for (x=0; x<(size/2); x++) {
    temp1 = fd[x];
    temp2 = fd[x];
		
    temp3 = ((temp1 & 0x00ff) << 8);
    temp4 = ((temp2 & 0xff00) >> 8);
		
    fd[x] = (temp4 | temp3);
  }
}

/***********************************************************************
 *     Client task to send offsets to delay lines
 ***********************************************************************/
void target_task(void *cookie)
{
  int opleserver;
  unsigned char j;
  struct smessage mess; //JDM
  unsigned short messtype;
  unsigned int messlength;
  //  struct sockaddr_in serv_addr, cli_addr;
  int ople_setup;
  int test_counter=0;
  //  RTIME start_clock,now_clock;
  long start_time,now_time;
 
  // Init server's data structure
  printf("init jobs\n");
  init_jobs();
  //JDM  Setup OPLE socket as completely separate socket that stays open.
  opleserver= (int) open_server_socket(OPLE,2200,"OPLE");
  printf("opleserver = %d   ",opleserver);
  //JDM Get current offsets -- from now on, the server will be only keeper
  //of this information
  mess.type=OPLE_CURRENT_MANUAL_TARGETS;
  mess.length=0;
  mess.data=(char *)malloc(4);
  printf("Sending OPLE message number == %d\n",mess.type);
  if (!send_message(opleserver,&mess))
    {
      fprintf(stderr,"Failed to send OPLE CURRENT MANUAL TARGETS\n");
    }
  
//JDM Wait for answer (do I need to cast the char -> doubles?)
  ople_setup=0;  //kludgy way of dealing with ople server to get offsets initially!
  while(ople_setup == 0){
    while (client_read_socket(servers[opleserver].fd,(char *) &messtype,2) == 0){
      rt_task_sleep(1000000);
    };
    printf("message type = %d\n",messtype );
  
    while (client_read_socket(servers[opleserver].fd,(char *) &messlength,4) == 0){
      rt_task_sleep(1000000);
    };
   
    // allocate data buffer
    mess.data = (unsigned char *) realloc(mess.data, messlength);
   
    while (client_read_socket(servers[opleserver].fd,(char *) mess.data, messlength) == 0){
      rt_task_sleep(1000000);
    }
    printf("got the data for %d %d %d \n",messlength, messtype, OPLE_CURRENT_MANUAL_TARGETS);
  
    if (messtype == OPLE_CURRENT_MANUAL_TARGETS) {
      memcpy(dloffsets,mess.data,messlength);
      memcpy(last_dloffsets,mess.data,messlength);
      ople_setup = 1;
    }
  }

  //start_time = (long)   (  (double) (start_clock>>18) /3.8147 ); //convert to ms
 
  printf("\n dloffset 0= %lf 1= %lf 2= %lf 3= %lf 4= %lf 5= %lf\n",*dloffsets, *(dloffsets+1), *(dloffsets+2), *(dloffsets+3), *(dloffsets+4), *(dloffsets+5));
  
  // start_time=GETCLOCK();
  
  /* Loop to send offsets to DLs */
  while(!exc.endProg) {
    rt_task_sleep(50000000); //erg!
    
    // *******************************************************************************
    // JDM KEEP AFTER TEST     rt_task_sleep(500000000); // 0.5 second delay (for Now)

    /* Next set of code used for testing. */
    //start_clock=rt_timer_read();
    //rt_task_sleep(500000000); // 0.5 second delay (for now)
    //  rt_task_sleep(1000000000); // 1.0 second delay (for now)
    
/*  test_counter++; */
/*       for (j=0;j<NUMDLS;j++){ */
/* 	dloffsets[j]=j+ test_counter* 1e-6; */
/* 	} */
	
/* 	//	printf("DL %d :  %lf",j,dloffsets[j]); */
/* 	exc.DLOffset[j] = 0; */
/*        gdt.manualOffset[j] = 0; */
	
/*       // printf("\n"); */
/*        //	set_manual_offsets(opleserver, dloffsets); */
/*        // for (j=0;j<NUMDLS;j++){ */
/*        for (j=0;j<3;j++){ */
/* 	 now_time=GETCLOCK(); */
/* 	 rt_task_sleep(300000000); */
/*         set_manual_target(opleserver,j,dloffsets[j]); */
/*         printf("DELAY %i: %d (ms) \n",j,(int) (GETCLOCK()-now_time)); */
/*        } */
/* 	    printf("(%d) DL Offsets SENT: %lf %lf %lf %lf %lf %lf \n", */
/*              (int)(GETCLOCK()-start_time) ,  */
/* 	     dloffsets[0], dloffsets[1],dloffsets[2],dloffsets[3],dloffsets[4],dloffsets[5]); */
/* 	    printf("AVERAGE TIME (ms): %f \n",(float)(GETCLOCK()-start_time)/test_counter); */
/* mess.type=OPLE_CURRENT_MANUAL_TARGETS; */
/*   mess.length=0; */
/*   // mess.data=(char *)malloc(4);  //JDM commented out 2006Mar -- not needed, right? */
/*    //printf("Sending OPLE message number == %d\n",mess.type); */
/* /\*   if (!send_message(opleserver,&mess)) *\/ */
/* /\*     { *\/ */
/* /\*       fprintf(stderr,"Failed to send OPLE CURRENT MANUAL TARGETS\n"); *\/ */
/* /\*     } *\/ */

/*   // Now ask for offsets  */
/*   //rt_task_sleep(100000000); */
/*   ople_setup=1;  //kludgy way of dealing with ople server to get offsets initially! */
/*   while(ople_setup == 0){ */
/*     while (client_read_socket(servers[opleserver].fd,(char *) &messtype,2) == 0){ */
/*       rt_task_sleep(1000000); */
/*     }; */
/*     //  printf("message type = %d\n",messtype ); */
  
/*     while (client_read_socket(servers[opleserver].fd,(char *) &messlength,4) == 0){ */
/*       rt_task_sleep(1000000); */
/*     }; */
/*     printf("message length = %d\n", messlength ); */
/*     // allocate data buffer */
/*     mess.data = (unsigned char *) realloc(mess.data, messlength); */
/*     while (client_read_socket(servers[opleserver].fd,(char *) mess.data, messlength) == 0){ */
/*       rt_task_sleep(1000000); */
/*     } */
/*     if (messtype == OPLE_CURRENT_MANUAL_TARGETS) { */
/*       memcpy(dloffsets_test,mess.data,messlength); */
/*       ople_setup = 1; */
/*              printf("(%ld) DL Offsets RCVD: %lf %lf %lf %lf %lf %lf \n",(long)(GETCLOCK()-start_time),  */
/* 	     dloffsets_test[0], dloffsets_test[1],dloffsets_test[2],dloffsets_test[3], */
/*          dloffsets_test[4],dloffsets_test[5]); */
/*     }} */
  
  // ********************** END OF TEST *****************************************
    
    if(exc.OffsetsReady || exc.manOffsetsReady){
     
   for (j=0;j<NUMDLS;j++){
	
	if( (exc.DLOffset[j] == exc.DLOffset[j])&&(exc.OffsetsReady)){
	    dloffsets[j]=dloffsets[j] + exc.DLOffset[j]; //increment dl
	    exc.DLOffset[j]=0;
	}
        if ( (gdt.manualOffset[j] == gdt.manualOffset[j])&&(exc.manOffsetsReady)){ 
             dloffsets[j]=dloffsets[j] + gdt.manualOffset[j];
	     gdt.manualOffset[j] = 0;
	}
	
	//	printf("DL %d :  %lf",j,dloffsets[j]);
	//exc.DLOffset[j] = 0;
	//gdt.manualOffset[j] = 0;
	
      }
      // printf("\n");
      //JDM I need to modify this to only update DLs that change since there is
      // overhead. 300ms per channel
        for (j=0;j<NUMDLS;j++){ 
	  if (dloffsets[j] != last_dloffsets[j]){ // if changed, then update
	    printf("Sending %lf to DL %d\n", dloffsets[j],j);
            set_manual_target(opleserver,j,dloffsets[j]);
	    last_dloffsets[j]=dloffsets[j];
	    // rt_task_sleep(330000000);    // 1/3 second delay (used to work)
	    rt_task_sleep(1000000000);
	  }
	}

      //printf("Offsets sent\n");
      exc.OffsetsReady = FALSE;
      exc.manOffsetsReady = FALSE;
    }
  }
  //JDM close out the opleserver socket which had remained open.
  close_server_socket(opleserver);
  fflush(NULL);
}
/* end of target_task */


/***********************************************************************
 *     VME VERSION Client task to send offsets to delay lines
 ***********************************************************************/
void vme_target_task(void *cookie)
{
  int opleserver;
  int ople_socket; // vme
  unsigned char j;
  struct smessage mess; //JDM
  unsigned short messtype;
  unsigned int messlength;
  //  struct sockaddr_in serv_addr, cli_addr;
  int ople_setup;
  int test_counter=0;
  long start_time,now_time;
 
  // Init server's data structure
  printf("init jobs to talk with VME!! \n");
  init_jobs();
  //JDM  Setup OPLE socket as completely separate socket that stays open.
  opleserver= (int) open_server_socket(OPLE,2200,"OPLE");
  ople_socket = (int) open_connect_socket(OPLE_MACHINE, OPLE_SOCKET);
  // chara code uses open_connect_socket instead.. whats the difference?
  printf("opleserver = %d",opleserver);
  printf("ople_socket (vme) = %d",ople_socket);

  //JDM Get current offsets -- from now on, the server will be only keeper
  //of this information
  mess.type=OPLE_CURRENT_MANUAL_TARGETS;
  mess.length=0;
  mess.data=(char *)malloc(4);
  printf("Sending OPLE message number == %d\n",mess.type);
  if (!send_message(opleserver,&mess))
    {
      fprintf(stderr,"Failed to send OPLE CURRENT MANUAL TARGETS\n");
    }
  
  ople_setup=0;  // Keep listening to socket until we get the right message!
  while(ople_setup == 0){
    while (client_read_socket(servers[opleserver].fd,(char *) &messtype,2) == 0){
      rt_task_sleep(1000000);
    };
    printf("message type = %d\n",messtype );
  
    while (client_read_socket(servers[opleserver].fd,(char *) &messlength,4) == 0){
      rt_task_sleep(1000000);
    };
    printf("message length = %d\n", messlength );
    // allocate data buffer
    mess.data = (unsigned char *) realloc(mess.data, messlength);
    while (client_read_socket(servers[opleserver].fd,(char *) mess.data, messlength) == 0){
      rt_task_sleep(1000000);
    }
    if (messtype == OPLE_CURRENT_MANUAL_TARGETS) {
      memcpy(dloffsets,mess.data,messlength);
      ople_setup = 1;
    }
  }

   printf("Current OPLE Offsets: %lf  %lf  %lf  %lf  %lf  %lf \n",*dloffsets, *(dloffsets+1), *(dloffsets+2), *(dloffsets+3),*(dloffsets+4),*(dloffsets+5));
   start_time=GETCLOCK();

  /* Loop to send offsets to DLs */
  while(!exc.endProg) {
    // *******************************************************************************
    // JDM KEEP AFTER TEST     rt_task_sleep(500000000); // 0.5 second delay (for now)

    /* Next set of code used for testing. */
    //start_clock=rt_timer_read();
    //rt_task_sleep(500000000); // 0.5 second delay (for now)
    //  rt_task_sleep(1000000000); // 1.0 second delay (for now)
    
 test_counter++;
      for (j=0;j<NUMDLS;j++){
	dloffsets[j]=j+ test_counter* 1e-6;
	}
	
	//	printf("DL %d :  %lf",j,dloffsets[j]);
	exc.DLOffset[j] = 0;
       gdt.manualOffset[j] = 0;
	
      // printf("\n");
       //	set_manual_offsets(opleserver, dloffsets);
       // for (j=0;j<NUMDLS;j++){
       for (j=0;j<3;j++){
	 now_time=GETCLOCK();
	 rt_task_sleep(300000000);
        vme_set_manual_target(ople_socket,j,dloffsets[j]);
        printf("DELAY %i: %d (ms) \n",j,(int)(GETCLOCK()-now_time));
       }
	    printf("(%ld) DL Offsets SENT: %lf %lf %lf %lf %lf %lf \n",
             (long)(GETCLOCK()-start_time) , 
	     dloffsets[0], dloffsets[1],dloffsets[2],dloffsets[3],dloffsets[4],dloffsets[5]);
	    printf("AVERAGE TIME (ms): %f \n",(float)(GETCLOCK()-start_time)/test_counter);
mess.type=OPLE_CURRENT_MANUAL_TARGETS;
  mess.length=0;
  // mess.data=(char *)malloc(4);  //JDM commented out 2006Mar -- not needed, right?
   //printf("Sending OPLE message number == %d\n",mess.type);
/*   if (!send_message(opleserver,&mess)) */
/*     { */
/*       fprintf(stderr,"Failed to send OPLE CURRENT MANUAL TARGETS\n"); */
/*     } */

  // Now ask for offsets 
  //rt_task_sleep(100000000);
  ople_setup=1;  //kludgy way of dealing with ople server to get offsets initially!
  while(ople_setup == 0){
    while (client_read_socket(servers[opleserver].fd,(char *) &messtype,2) == 0){
      rt_task_sleep(1000000);
    };
    //  printf("message type = %d\n",messtype );
  
    while (client_read_socket(servers[opleserver].fd,(char *) &messlength,4) == 0){
      rt_task_sleep(1000000);
    };
    printf("message length = %d\n", messlength );
    // allocate data buffer
    mess.data = (unsigned char *) realloc(mess.data, messlength);
    while (client_read_socket(servers[opleserver].fd,(char *) mess.data, messlength) == 0){
      rt_task_sleep(1000000);
    }
    if (messtype == OPLE_CURRENT_MANUAL_TARGETS) {
      memcpy(dloffsets_test,mess.data,messlength);
      ople_setup = 1;
      printf("(%ld) DL Offsets RCVD: %lf %lf %lf %lf %lf %lf \n",(long)(GETCLOCK()-start_time), 
	     dloffsets_test[0], dloffsets_test[1],dloffsets_test[2],dloffsets_test[3],
         dloffsets_test[4],dloffsets_test[5]);
    }}
  
  // ********************** END OF TEST *****************************************
    if(exc.OffsetsReady || exc.manOffsetsReady){
      for (j=0;j<NUMDLS;j++){
	
	if(exc.DLOffset[j] == exc.DLOffset[j]){

	  dloffsets[j]=dloffsets[j] + exc.DLOffset[j]; //increment dl
	}
        if (gdt.manualOffset[j] == gdt.manualOffset[j]){ 
             dloffsets[j]=dloffsets[j] + gdt.manualOffset[j];
	}
	
	//	printf("DL %d :  %lf",j,dloffsets[j]);
	exc.DLOffset[j] = 0;
	gdt.manualOffset[j] = 0;
	
      }
      // printf("\n");
      //JDM I need to modify this to only update DLs that change since there is
      // overhead. 300ms per channel
      set_manual_offsets(opleserver, dloffsets);

      //printf("Offsets sent\n");
      exc.OffsetsReady = FALSE;
      exc.manOffsetsReady = FALSE;
    }
  }
  //JDM close out the opleserver socket which had remained open.
  close_server_socket(opleserver);
  close(ople_socket);
  fflush(NULL);
	   }
/* end of vme_target_task */



/***********************************************************************
 *     Task to transfer data from astropci on interrupt
 ***********************************************************************/
void sem_task(void *cookie) 
{
  RT_SEM switch_sem;
  //  RT_SEM dataReady;
  KEY_STATE ks = S_IDLE;
  KEY_STATE ks_next = S_IDLE;

  struct smessage message;
  int pixshift = 8;
  int sem_count= 0;
  int err = 0;
  RTIME now = 0, previous = 0;
  RTIME sem_now = 0, sem_prev = 0, sem_prev2 = 0;
  RTIME sem_first = 0, sem_last = 0, sem_preexp;
  float reset_gap=0.;
  unsigned int m = 0;
  unsigned int frameCount = 1, firstCrap = 0;
  unsigned int totalFrameCount = 0;
  unsigned short *tmpBuffer;
  float *tmpBuffer1,*oldBuffer;
  // char path[30];
  unsigned int coaddCount = 1;
  unsigned int coaddFrame = 1;
  unsigned int nBkgnd = 0;
  float *totalBuffer, *bkgndBuffer, *pSpec2DBuffer, *imBuffer;
  fftwf_complex *fft;
  fftwf_plan plan = NULL;
  RTIME timeout = TIMEOUT;
  // unsigned short *filebuffer;
  // float opdtest;
  float gdtGain=.6;
  int j,loop_counter=0;
  int frameOffset = 0;
  float gdtNoiseWt,gdtNoiseOpd,gdtFinalWeights[BASELINES];
  double dummyOffset;
  float medianFilter[BASELINES][NUMMEDIAN], sorted[NUMMEDIAN];
  unsigned char mediancount = 0;
  unsigned char pipeName[5];
  // unsigned char device[20];
  int fileOpen = FALSE;
  int loopMsg = FALSE, bg_go = FALSE;
  float t_exec, t_int; // average frame time in ms
  header_struct header;

  bzero(medianFilter, BASELINES * NUMMEDIAN * sizeof(float));
  daqOut.coaddCount=0;
  sprintf(pipeName,"pipe%i",pipeMinor);
  
  while(rt_pipe_create(&pipe_desc,pipeName,pipeMinor)) {
    pipeMinor++;
    sprintf(pipeName,"pipe%i",pipeMinor);
    printf("Trying to open %s...\n",pipeName);
  }
  printf("Create %s:\n",pipeName);
    
  sprintf(exc.rtpDev,"../spooler/spooler /dev/rtp%i &",pipeMinor);
  
  /* Create the accumulator image buffer */
  totalBuffer = (float *)malloc(MAX_SIZE);
  if (totalBuffer == NULL){
    printf("Cannot allocate totalBuffer \n");
    exit (1);
  }

  /* Create the background frame buffer */
  bkgndBuffer = (float *)malloc(MAX_SIZE);
  if (bkgndBuffer == NULL){
    printf("Cannot allocate bkgndFrame \n");
    exit (1);
  }
  
  /* Create the accumulate power spectrum image buffer */
  pSpec2DBuffer = (float *)malloc(MAX_SIZE);
  if (pSpec2DBuffer == NULL){
    printf("Cannot allocate pSpec2DBuffer\n");
    exit (1);
  }
  
  imBuffer = (float *)malloc(MAX_SIZE);
  if (imBuffer == NULL){
    printf("Cannot allocate imBuffer\n");
    exit (1);
  }

  fft = fftwf_malloc(sizeof(fftwf_complex) * 65536); //Only 1 quad

  /* Create the file-save buffer */
 
/*   filebuffer = (unsigned short *)malloc(MAX_SIZE); */
/*   if (filebuffer == NULL){ */
/*     printf("Cannot allocate coaddedFrame \n"); */
/*     exit (1); */
/*   } */

  /* Create a temporary buffer */
  tmpBuffer = (unsigned short *)malloc(MAX_SIZE);
  if (tmpBuffer == NULL){
    printf("Cannot allocate temporary buffer \n");
    exit (1);
  }
  /* Create a temporary buffer */
  tmpBuffer1 = (float *)malloc(MAX_SIZE);
  if (tmpBuffer1 == NULL){
    printf("Cannot allocate temporary1 buffer \n");
    exit (1);
  }


  /* Create a buffer to hold the old image for the difference */
  oldBuffer = (float *)malloc(MAX_SIZE);
  if (oldBuffer == NULL){
    printf("Cannot allocate old buffer \n");
    exit (1);
  }


  err = rt_sem_bind(&switch_sem, "dispsem");
  if (err) {
    fprintf(stderr,"semaphore_task: cannot bind semaphore: %s\n",
	    strerror(-err));
    return;
  } else fprintf(stdout,"semaphore_task: semaphore bound to kernel task: \n");


  
  // Main loop
  while (!exc.endProg) {

    //data acquisition loop until collected framesPerReset

    // this will measure the execution time of the RT loop
    previous = now;
    now = rt_timer_read();
    rt_task_sleep(1000000);  /* Sleep for 1 ms */
                            /* ideally pending on 1ms interrupt */
    loop_counter++;
    if ( (loop_counter % 1000) == 0)        loopMsg = TRUE;
                                       else loopMsg = FALSE;
    
/*    // blocking semaphore from interrupt handler */
/*     err = rt_sem_p(&switch_sem, timeout); */
/*     if (err && (ks == C_RUN)) { */
/*       printf("Timeout on data interrupt!!!\n"); */
/*       exc.cmd = C_IDLE; */
/*     } */

      /* Get new state from Finite State Machine lookup table */
      ks_next = FSM[ks][exc.cmd];
      if (loopMsg)
	//printf("Getting new state from FSM: %i, %i, %i", ks_next,ks,exc.cmd);

      /*-----------------------------------------------------------*/
      /* State 1: IDLE STATE                                       */
      /* "Do nothing"                                              */
      /*-----------------------------------------------------------*/

      if ((S_IDLE == ks_next) && (S_IDLE == ks)){
	if (loopMsg) printf("IDLE state. Waiting command...\n");
	sem_count = 0;
      }

 
      if ((S_IDLE == ks_next) && !(S_IDLE == ks)) {
        printf("Transition to S_IDLE\n");
        exc.cmd = C_IDLE;
      }

      /*-----------------------------------------------------------*/
      /* Transition from S_IDLE to S_INIT state                    */
      /* "Synchronise local vars with sh_mem when idle"            */
      /* Does major memory reallocation, etc. SLOW                 */
      /*-----------------------------------------------------------*/

      if ((S_INIT == ks_next) && (S_IDLE == ks)){
	printf("S_IDLE -> S_INIT state\n");

	m = 0; 
	frameCount = 1;
	totalFrameCount = 0;
	coaddFrame = 0;
  	coaddCount = 1;

	/* Will zero out background whenever INIT happens (sensible) */
        bkgndBuffer = (float *)realloc(bkgndBuffer, daq.rows*daq.cols*daq.nreads*daq.framesPerReset*sizeof(float)); /* this is huge */
        
        if (bkgndBuffer == NULL){
	    printf("Cannot allocate bkgndFrame \n");
	     exit (1);
	 }
          bzero(bkgndBuffer,daq.rows*daq.cols*daq.nreads*daq.numberOfFrames*sizeof(float) );
	  nBkgnd = 0;
	  printf("Reset Background at main INIT\n");  
 		
	if(fft == NULL) {
	  fft = fftwf_malloc(sizeof(fftwf_complex) * daq.cols * daq.rows/4); //Only 1 quad
	} else {
	  fftwf_free(fft);
	  fft = fftwf_malloc(sizeof(fftwf_complex) * daq.cols * daq.rows/4); //Only 1 quad
	}

	/* Create the accumulator image buffer */
        tmpBuffer = (unsigned short *)realloc(tmpBuffer,exc.byte_count_short);
	tmpBuffer1 = (float *) realloc(tmpBuffer1,exc.byte_count_float);

	totalBuffer = (float *)realloc(totalBuffer, exc.byte_count_float);
	if (totalBuffer == NULL){
	  printf("Cannot allocate totalBuffer \n");
	  exit (1);
	}

  
	/* Create the accumulate power spectrum image buffer */
	pSpec2DBuffer = (float *)realloc(pSpec2DBuffer, exc.byte_count_float);
	imBuffer = (float *)realloc(imBuffer, exc.byte_count_float);

	//Create plan for DFT (see fftw3 man)
	plan = fftwf_plan_dft_r2c_2d(daq.rows/2, daq.cols/2,
				     imBuffer, fft,
				     FFTW_MEASURE);
      

	/* Create a buffer to hold the old image for the difference */
	oldBuffer = (float *)realloc(oldBuffer, exc.byte_count_float);
	if (oldBuffer == NULL){
	  printf("Cannot allocate old buffer \n");
	  exit (1);
	}
	bzero(totalBuffer, exc.byte_count_float);
	bzero(pSpec2DBuffer, exc.byte_count_float);

      }  //  if ((S_INIT == ks_next) && (S_IDLE == ks))


      /*-----------------------------------------------------------*/
      /* Transition to RUN STATE                                   */
      /* "Acquire frames and run GDT"                              */
      /*-----------------------------------------------------------*/
  
      if ((S_RUN == ks_next) && (S_RUN != ks)){
	printf("Transition to S_RUN State\n");
      }

      if (S_RUN == ks) {
	exc.daqRunning = TRUE;
      } else {
	exc.daqRunning = FALSE;
      }

      /*-----------------------------------------------------------*/
      /* State 3: RUN STATE                                        */
      /* "Acquire frames and run GDT"                              */
      /* Will stay in this STATE until timeout or done             */
      /*-----------------------------------------------------------*/
     
      // Note -- the user is expected to have done an C_INIT at least once!
      if ((S_RUN == ks_next) && (S_RUN == ks)){
	sem_preexp = rt_timer_read();
        //Baby Init
  	  sem_count = 0;
	  m = 0; 
	  frameCount = 1;
	  firstCrap=0; // kludge....
	  totalFrameCount = 0;
	  coaddFrame = 0;
	  coaddCount = 1;
	  bzero(totalBuffer, exc.byte_count_float);
	  bzero(pSpec2DBuffer, exc.byte_count_float);

	  if (exc.takeBkgnd) bg_go=TRUE;  
	  /* needed so background accumulation doesn't start in the middle 
	     of read */

      
	  /* Send header info to the spooler process */
	  message.type = HEADER;
	  message.length = sizeof(header);
	  fileOpen = TRUE; // only do this if saving is ON??
            if (fileOpen) {

	    /* assemble structures into the header struct */
	    //printf("Gather header info\n");

	    header.rows=daq.rows;
	    header.cols=daq.cols;
	    header.rowOffset=daq.rowOffset;
	    header.colOffset=daq.colOffset;  
	    header.nreads=daq.nreads;
	    header.numberOfCoadds=daq.numberOfCoadds;
	    header.framesPerReset=daq.framesPerReset;
	    header.exptime=daq.exptime;
	    header.numberOfFrames=daq.numberOfFrames;
	    header.do_image_save=daq.do_image_save;
	    header.voffset=daq.voffset;
	    header.vreset=daq.voffset;  
	    header.vd=daq.vd;
	    header.ictl=daq.ictl;
	    memcpy(header.image_directory,daq.image_directory,30);
	    memcpy(header.image_filename,daq.image_filename,20);
	    /*   sprintf(header.object, "%s",'null'); */
	    /*   memcpy('Nemo',header.observer,4); */
	    /*   header.bandwidth=0.3; */
	    header.tint=t_int;
	    header.lambda=exc.lambda;
	    header.bandwidth=exc.bandwidth;
	    /*   memcpy('Comment',header.comment,7); */


	    printf("%s\n",daq.image_directory);


	    rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
	    rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
	    rt_pipe_write(&pipe_desc, (unsigned char *) &header, message.length, P_NORMAL);
	  }
      

       /* Start the exposure */

	
       printf("Starting exposure.\n");
                            
       if (sem_count == 0) {
	 reset_gap = (float)(rt_timer_read() - sem_prev2) / ( (double) 1000000 );
	 printf("Reset Gap (ms): %f \n",reset_gap);
       }

	if (doCommand(pci_fd, TIM_ID, SEX, DON) == _ERROR) {
	    printf("\tERROR: Starting exposure failed -> 0x%X\n", getError());
	} else exc.cmd = C_IDLE;
      
      while (sem_count < daq.numberOfFrames){
	sem_prev2=rt_timer_read();        
	sem_prev= sem_now;
	// blocking semaphore from interrupt handler 
	err = rt_sem_p(&switch_sem, timeout); 
        if (err && (ks == C_RUN)) { 
          printf("Timeout on data interrupt!!!\n");
          exc.cmd = C_IDLE;
	  break;
        }
        // Do timing from reset to reset.
        if (sem_count == 0) {
	  sem_last=sem_first; 
	  sem_first = rt_timer_read();
	  printf("Start2Start (ms): %f \n",(float)(sem_first-sem_preexp) / ( (double) 1000000));
	  printf("First2First (ms): %f \n",(float)(sem_first-sem_last) / ( (double) 1000000));
	}
        sem_now = rt_timer_read();
	sem_count++;
              
        // Replace this with an average?
            
	if ( (sem_count % (daq.numberOfFrames / 2 + 1)) == 0) {
	  t_int = (float)(sem_now - sem_prev) / ((double) 1000000); 
	 t_exec =(float)(sem_now - sem_prev2) / ((double) 1000000);
/* 	  	  printf("Total frame count: %d frame saved %d irq exec time %ld.%06ld ms %ld.%06ld task\n", */
/* 		 sem_count,	    */
/* 		 totalFrameCount,     */
/* 		 (long)(sem_now - sem_prev) / 1000000, */
/* 		 (long)(sem_now - sem_prev) % 1000000, */
/* 		 (long)(sem_now - sem_prev2) / 1000000, */
/* 		 (long)(sem_now - sem_prev2) % 1000000); */
	 printf("Sem Count: %i  T_int (ms): %f  T_exec (ms): %f \n",sem_count,t_int,t_exec);
       }
  
	memcpy( (short *) tmpBuffer, mem_fd + m + pixshift, exc.byte_count_short);
	
	if (deinterlace(daq.cols * daq.nreads, daq.rows, tmpBuffer) == _ERROR) {
	  printf("\tERROR: DEINTERLACE failed");
	} 
        swap_memory(tmpBuffer, exc.byte_count_short);

	/* Send data through pipe to the spooler process */
	message.type = DATA;
	message.length = exc.byte_count_short;//exc.byte_count_short;
	if(fileOpen){
	  rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
	  rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
	  rt_pipe_write(&pipe_desc, (unsigned char *) tmpBuffer, message.length, P_NORMAL);
	}

        if (exc.takeBkgnd && bg_go) {
	  //printf("Accumulating Background: %i\n",frameCount);
          frameOffset =(int) (frameCount -1)*daq.rows*daq.cols*daq.nreads;
          for (j=0;j<(daq.rows*daq.cols*daq.nreads);j++) {
            *(bkgndBuffer+frameOffset+j) +=(float) *(tmpBuffer+j);
	  }
        }

	// Convert tmpBuffer (ints) to tmpBuffer1 (float)
        if (exc.useBkgnd == TRUE && !(exc.takeBkgnd) && nBkgnd > 0) {  /* Apply background frame to data! */
          frameOffset =(int) (frameCount -1)*daq.rows*daq.cols*daq.nreads;
          for (j=0;j<(daq.rows*daq.cols*daq.nreads);j++) {
            *(tmpBuffer1+j) =(float) *(tmpBuffer+j) -((float)*(bkgndBuffer+frameOffset+j))/nBkgnd;
	  }} else {
          for (j=0;j<(daq.rows*daq.cols*daq.nreads);j++) {
            *(tmpBuffer1+j) =(float) *(tmpBuffer+j);
	  }
	  }
        
	//	if (frameCount == 30) {
        //  printf("resetBkgnd %i  useBkgnd %i  addBkgnd%i   nBkgnd %i \n",exc.resetBkgnd,exc.useBkgnd, exc.takeBkgnd, nBkgnd);
	//}

	if (coaddCount > 2 ) {
	  psAccum(tmpBuffer1, oldBuffer, imBuffer, pSpec2DBuffer,  daq.rows, daq.cols, daq.nreads,  plan, exc.pixelCount, fft,bkgndBuffer);
	}

       	linearFit(tmpBuffer1, totalBuffer, daq.rows, daq.cols, daq.nreads, daq.numberOfCoadds, coaddCount, m);

	/* I have collected the prescribed number of Coadds */
	/* do things -- like copy image to daq and calculate opds */
     
	if (coaddCount >= daq.numberOfCoadds && sem_count < daq.numberOfFrames) {
        coaddCount = 0;
	  
	//remove following if statement if camera reset transient
	// is improved.. JDM.. skips output of first coadd frame
	if (firstCrap == 1) {
	memcpy(daqOut.coadd,  totalBuffer, (daq.cols*daq.rows)*sizeof(float)); 
	daqOut.coaddCount++;
	} 
	else {
	  printf("Skipping first coadd crap \n");
	  firstCrap=1;
	}

        memcpy(daqOut.pSpec2D, pSpec2DBuffer, (daq.cols/4+1)*daq.rows/2*sizeof(float) );        
        

	gdtNoiseOpd=centroid(daqOut.pSpec2D,daq.rows/2,daq.cols/4+1,exc.bkgndNoiseCol,
			      exc.lambda,exc.bandwidth,&gdtNoiseWt);
      
        for (j=0;j<BASELINES;j++){

	  medianFilter[j][mediancount] = ((float)exc.OPDSigns[j])*centroid(daqOut.pSpec2D,daq.rows/2,daq.cols/4+1,exc.cpeak[j],
	    exc.lambda,exc.bandwidth,exc.gdtWeight+j);

	    /* really dum median filter of NUMMEDIAN values */
	    memcpy((float *) sorted, medianFilter[j],NUMMEDIAN * sizeof(float));
	    bubbleSort(sorted, NUMMEDIAN);
	    // JDM Forget median
      	    //exc.gdtOpd[j] = sorted[NUMMEDIAN / 2];
            exc.gdtOpd[j] =((float)exc.OPDSigns[j])*
	      centroid(daqOut.pSpec2D,daq.rows/2,daq.cols/4+1,exc.cpeak[j],
	      exc.lambda,exc.bandwidth,exc.gdtWeight+j);
	    // exc.gdtWeight[j]=exc.gdtWeight[j]/gdtNoiseWt;
	    gdtFinalWeights[j]=exc.gdtWeight[j]/exc.gdtThreshold[j];
	    if (gdt.activeBaselines[j] == FALSE) gdtFinalWeights[j]=0.1;
	    // printf("%d OPD(mu):%f4.1  Wt:%f4.1",j,exc.gdtOpd[j],gdtFinalWeights[j]);
	}
	 
          bootStrapping4(gdtFinalWeights,exc.gdtOpd,exc.gdtOptOpd);
	  if (DL_DEBUG == 1) {
	   printf("\n");
	   printf("weights: %lf %lf %lf %lf %lf %lf\n",gdtFinalWeights[0],gdtFinalWeights[1],gdtFinalWeights[2],gdtFinalWeights[3],gdtFinalWeights[4],gdtFinalWeights[5]);
	   printf("opds   : %lf %lf %lf %lf %lf %lf\n",exc.gdtOpd[0],exc.gdtOpd[1],exc.gdtOpd[2],exc.gdtOpd[3],exc.gdtOpd[4],exc.gdtOpd[5]);
           printf("optopds: %lf %lf %lf \n",exc.gdtOptOpd[0],exc.gdtOptOpd[1],exc.gdtOptOpd[2]);
	  } // (DL_DEBUG == 1)
      
	  // These delays are in terms of BEAMS. NOW Convert to delay lines
	  exc.DLOffset[exc.BeamOrder[0]]=0.0/1000000./2; // also convert to meters.
	  for (j=1;j<NUMTELS;j++){
	    exc.DLOffset[exc.BeamOrder[j]]=-1.0*exc.gdtOptOpd[j-1] /1000000./2;
	    //offset is opposite in sign to OPD and half the value. j-1?
	  }
	  if (DL_DEBUG == 1) {
	   printf("dl0: %lf %lf %lf %lf %lf %lf:\n",exc.DLOffset[0]*1e6,exc.DLOffset[1]*1e6,exc.DLOffset[2]*1e6,exc.DLOffset[3]*1e6,exc.DLOffset[4]*1e6,exc.DLOffset[5]*1e6);
	  }
      
	  //Now apply reference cart
	  dummyOffset=exc.DLOffset[exc.RefTel];
	  for (j=0;j<NUMTELS;j++){  // Only update DLs that are listed in the BEAMOrder
	    exc.DLOffset[exc.BeamOrder[j]]=gdtGain*(exc.DLOffset[exc.BeamOrder[j]]-dummyOffset);
	  }
	  // Extra Feature. Zero offsets to delay line if
	  // all baselines using that DL are NOT ACTIVE.
	  if (gdt.activeBaselines[0] == FALSE && gdt.activeBaselines[1] == FALSE 
	      && gdt.activeBaselines[2] == FALSE) exc.DLOffset[exc.BeamOrder[0]]=0.0;
	  if (gdt.activeBaselines[0] == FALSE && gdt.activeBaselines[3] == FALSE 
	      && gdt.activeBaselines[4] == FALSE) exc.DLOffset[exc.BeamOrder[1]]=0.0;
	  if (gdt.activeBaselines[1] == FALSE && gdt.activeBaselines[3] == FALSE 
	      && gdt.activeBaselines[5] == FALSE) exc.DLOffset[exc.BeamOrder[2]]=0.0;
	  if (gdt.activeBaselines[2] == FALSE && gdt.activeBaselines[4] == FALSE 
	      && gdt.activeBaselines[5] == FALSE) exc.DLOffset[exc.BeamOrder[3]]=0.0;
	  // printf("dl0: %lf %lf %lf %lf %lf %lf:\n",exc.DLOffset[0]*1e6,exc.DLOffset[1]*1e6,exc.DLOffset[2]*1e6,exc.DLOffset[3]*1e6,exc.DLOffset[4]*1e6,exc.DLOffset[5]*1e6);
	  
	  if (DL_DEBUG == 1) {
	   printf("tesls: %d %d %d %d %d %d",S1,S2,W1,W2,E1,E2);
	   printf("GDT (S1 S2 E1 E2 W1 W2) mu: %lf %lf %lf %lf %lf %lf\n ",exc.DLOffset[0]*1e6,
		 exc.DLOffset[1]*1e6,exc.DLOffset[2]*1e6,exc.DLOffset[3]*1e6,exc.DLOffset[4]*1e6,exc.DLOffset[5]*1e6);
	  }
	  if (gdt.gdtLock) exc.OffsetsReady = TRUE;
	  bzero(totalBuffer, exc.byte_count_float);
          bzero(pSpec2DBuffer, exc.byte_count_float);
	  /* mediancount goes from 0 to 2 and back to 0 at any cycle */
	  if (mediancount >= NUMMEDIAN) {
	    mediancount = 0;
	  } 
	  else { 
	    mediancount++;
	  } 
	} // Coadd finished
  
	m += exc.pixelCount;
       	frameCount++;
	coaddCount++;
	totalFrameCount++;
  
	/* Determine where each new image starts on a 1024 address boundary (512 in
	   this case because we are working with pixels and not bytes). */
	if ((m % 512) != 0)
	  m = 512*((int)(m/512) + 1);

	/* Loop back to start of kernel image buffer (pixels[0]), and
	   re-initialize the frame count. */
	if (frameCount >= exc.framesPerBuffer) {
	  m = 0;
	  frameCount = 1;
	}
	memcpy( (float *) oldBuffer, tmpBuffer1, exc.byte_count_float);
  

       
  } // continue while until done reading frames
 
    // Ending the S_RUN State..
      
    if (fileOpen) {
	  /* Send END command to the spooler process */
	  message.type = END;
	  message.length = exc.byte_count_short;
	  fileOpen = FALSE;
	  rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
    }
    if (exc.takeBkgnd && bg_go) { /* Stop accumulating Background */
      bg_go = FALSE;
      nBkgnd++;
      exc.takeBkgnd = FALSE;
      printf("Finished Background. Current nBkgnd: %i\n",nBkgnd);
    }
    if (exc.continuousReadout) exc.cmd = C_RUN;
                           else exc.cmd = C_IDLE;     
      }

   

      /************************************************/
      /*  Do misc commands                            */
      /*    only executed outside of single exposure  */
      /************************************************/ 
      if (exc.endProg) break;
 
      // e.g., reset background
      if (exc.resetBkgnd == TRUE) {
        nBkgnd =0;
        bzero(bkgndBuffer,daq.rows*daq.cols*daq.nreads*daq.numberOfFrames*sizeof(float) );
	exc.resetBkgnd = FALSE;
	printf("Reset Background inside mini INIT: \n");
      }

      /* enter the next state: */
      ks = ks_next;

      }  // loop timer
  
  err = rt_sem_unbind(&switch_sem);
  if (err) {
    fprintf(stderr,"cannot unbind semaphore: %s\n",
	    strerror(-err));
  } else fprintf(stdout,"semaphore unbound: \n");

/*   free(filebuffer); */
  free( bkgndBuffer);
  free( totalBuffer);
  free( tmpBuffer);
  free( oldBuffer);
  free( imBuffer);
  free( pSpec2DBuffer);
  fftwf_destroy_plan(plan);
  fftwf_free(fft);

  /* Send KILL command to the spooler process */
  message.type = KILL;
  rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);

  err = rt_pipe_delete(&pipe_desc); 
  if (err) {
    fprintf(stderr,"Failed to delete pipe, code %d\n", err);
  } else fprintf(stderr,"Delete pipe\n");
  fflush(NULL);
  exit(0);
}

// signal-handler, to ensure clean exit on Ctrl-C
void clean_exit(int dummy)
{
  int err = 0;
  exc.endProg = TRUE;
  fprintf(stdout,"cleanup\n");
  usleep(2000000);
  if (exc.daqRunning) err = rt_task_delete(&sem_task_ptr);
  if (err) {
    fprintf(stderr,"astropci: failed to delete sem_task, code %d\n", err);
  } else fprintf(stdout,"Delete sem_task\n");
}

void error(char *msg)
{
  perror(msg);
  exit(1);
}

int main (int argc, char *argv[])
{
  // int opleserver;
  struct smessage mess; //JDM
  //unsigned short messtype;
  //unsigned int messlength;
  message_struct message;
  unsigned int configWord;
  int sockfd, newsockfd, portno, clilen, validate = 0, trigger = 0, cds = 0;
  char  pci_file[40] =  "/home/epedrett/dsp/pciInt.lod", tim_file[40] =  "/home/epedrett/dsp/tim_2005Sep16a.lod";
  struct sockaddr_in serv_addr, cli_addr;
  int n;
  //int ople_setup;
  int err = 0;
  //int shm_fd = 0;
  unsigned char i;
  int status;
 

   if (argc < 2) {
    fprintf(stderr,"ERROR, no port provided\n");
    exit(1);
  }


  pci_fd = open("/dev/astropci0", O_RDWR);
  
  /* Check for Reply or Image Buffer allocation failure. */
  if (pci_fd == -1 && errno == ENOMEM) {
    printf("Required Buffers (Reply or Image) \n Could Not Be Allocated (errno = %d) \n%s", errno, strerror(errno));
    exit(EXIT_FAILURE);
  }
  
  /* Check that the device can be opened. */
  else if (pci_fd == -1) {
    printf("Device Could Not Be Opened (errno = %d) \n%s", errno, strerror(errno));
    exit(EXIT_FAILURE);
  }

  /* bind user memory to kernel buffer */
  mem_fd = create_memory(pci_fd, BUFFER_SIZE);
 
  /* Disable paging for this program's memory. */
  mlockall(MCL_CURRENT|MCL_FUTURE);
  fprintf(stdout,"start\n");
  // install signal handler
  signal(SIGTERM, clean_exit);	
  signal(SIGINT, clean_exit);

  /* Create the accumulator image buffer */
  message.data = (float *)malloc(256);
  if (message.data == NULL){
    printf("Cannot allocate message.data \n");
    exit (1);
  }
  mess.data= (float *)malloc(1);

  err = rt_task_spawn(&sem_task_ptr, "Sem_task",
		      STACK_SIZE, HIGH_PRIORITY, TASK_MODE_1, &sem_task, NULL);
  if (err) {
    fprintf(stderr,"astropci: failed to create sem_task, code %d\n", err);
    //exit (1);
  } else fprintf(stdout,"sem_task created\n");

    err = rt_task_spawn(&target_task_ptr, "Target_task",
		      STACK_SIZE, STD_PRIORITY, TASK_MODE_1, &target_task, NULL);

/* err= rt_task_spawn(&target_task_ptr, "Target_task", */
/* 		      STACK_SIZE, STD_PRIORITY, TASK_MODE_0, &vme_target_task, NULL); */

  if (err) {
    fprintf(stderr,"astropci: failed to create target_task, code %d\n", err);
    //exit (1);
  } else fprintf(stdout,"target_task created\n");



  
 sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    //error("ERROR opening socket");
    exc.endProg = TRUE;
    usleep(2000000);
  }
  bzero((char *) &serv_addr, sizeof(serv_addr));
  portno = atoi(argv[1]);
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = INADDR_ANY;
  serv_addr.sin_port = htons(portno);
  if (bind(sockfd, (struct sockaddr *) &serv_addr,
	   sizeof(serv_addr)) < 0) {
    //error("ERROR on binding");
    exc.endProg = TRUE;
    usleep(2000000);
  }
  listen(sockfd,5);
  clilen = sizeof(cli_addr);

  
  //system(exc.rtpDev);
  

  while (!exc.endProg) {
    newsockfd = accept(sockfd,
		       (struct sockaddr *) &cli_addr,
		       &clilen);
    if (newsockfd < 0) 
      error("ERROR on accept");

    n = read(newsockfd, (unsigned short *) &message.type, 2);
    if (n < 0) {
      error("ERROR reading socket type");
      err = 1;
    } else {

      switch (message.type) {

      case EN:
	//Length of nessage = 4 chars (int). this could differ in other architecture 
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {
	  n = read(newsockfd, (unsigned int *) message.data, message.length);
	  if (n < 0) error("ERROR reading from socket");
	  err = raise(SIGINT);
	  if (err) {
	    fprintf(stderr,"Cannot raise SIGINT: %s\n",
		    strerror(-err));
	  }
	}
	break;
      
      case AB:
	/* Do not execute if rt task NOT running */
	if(!exc.daqRunning) break;
	printf("Abort Exposure...");

	status = (getHstr(pci_fd) & HTF_BITS) >> 3;
	// Send abort exposure/readout according to the HSTR contents.
	if (status == READOUT) {
	  err = hcvr(pci_fd, ABORT_READOUT, UNDEFINED, DON);
	} else {
	  err = doCommand(pci_fd, TIM_ID, AEX, DON);
	}
	if (err == _ERROR){
	  printf("failed. -> Could not abort.\n");
	}
	else
	  //exc.cmd = C_IDLE;
	  printf("done.\n");
	break;

      case IO:
	/* Do not execute if rt task running */
	if(exc.daqRunning) break;
	/* -----------------------------------
	   LOAD PCI FILE
	   ----------------------------------- */
	printf("Loading PCI file %s...", pci_file);
	if (loadPciFile(pci_fd, pci_file) == _ERROR) {
	  printf("failed. -> Could not load pci file.\n");
	  //return _ERROR;
	}
	else {
	    printf("done.\n");
	}

	/* -----------------------------------
	   SET (HARDWARE) BYTE SWAPPING
	   ----------------------------------- */
	daq.byteSwapping = setHardwareByteSwapping(pci_fd);



	/* -----------------------------------
	   RESET CONTROLLER
	   ----------------------------------- */
	printf("Resetting controller...");
	if (hcvr(pci_fd, RESET_CONTROLLER, UNDEFINED, SYR) == _ERROR) {
	  printf("failed. -> Could not reset controller.\n");
	  //return _ERROR;
	}
	else
	  printf("done.\n");


	/* -----------------------------------
	   LOAD TIMING FILE/APPLICATION
	   ----------------------------------- */

	printf("Loading timing file %s...", tim_file);
	loadFile(pci_fd, tim_file, validate);
	printf("done.\n");

	break;

      case CC: /* Send Coadd Counter so external program knows if frame is new*/

	  // allocate data buffer
	message.length = sizeof(int);
	  message.data = (unsigned char *) realloc(message.data, sizeof(int));
          memcpy((unsigned char *) message.data, &daqOut.coaddCount, sizeof(int));          
	  // send data through socket  

	  n = write(newsockfd, (unsigned char *) message.data, message.length);
	   if (n < 0) error("ERROR writing to socket");
      
	break;


      case RD:

	//Length of nessage = 4 chars (int). this could differ in other architecture
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {

	  // allocate data buffer
	  message.data = (unsigned char *) realloc(message.data, message.length);
          memcpy((unsigned char *) message.data, daqOut.coadd, message.length);          
	  // send data through socket  

	  n = write(newsockfd, (unsigned char *) message.data, message.length);
	
	  if (n < 0) error("ERROR writing to socket");
	}
	// close(fd);
	break;

      case RC:
        //outside service request configuration
	//Length of nessage = 4 chars (int). this could differ in other architecture
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {

	  // allocate data buffer
	  message.data = (unsigned char *) realloc(message.data, message.length);
          memcpy((unsigned char *) message.data, &daq, message.length);          
	  // send data through socket  

	  n = write(newsockfd, (unsigned char *) message.data, message.length);
	
	  if (n < 0) error("ERROR writing to socket");
	}
	// close(fd);
	break;

      case PS:

	//Length of nessage = 4 chars (int). this could differ in other architecture
	// read data length
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	// printf("PS Request length = %d\n",message.length);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {

	  // allocate data buffer
	  message.data = (unsigned char *) realloc(message.data, message.length);
          memcpy((unsigned char *) message.data , daqOut.pSpec2D, message.length);

	  // send data through socket  
	  n = write(newsockfd, (unsigned char *) message.data, message.length);
	
	  if (n < 0) error("ERROR writing to socket");
	}
	// close(fd);
	break;
 
	 case XX:
	   // Sets continuous readout to be FALSE
	   exc.continuousReadout = FALSE;
	   printf("Stopping Continuous Readout at next opportunity...\n");
	   break;
	
      case SX:
	/* Do not execute if rt task running */
	if(exc.daqRunning) break;

	/* Set frames per buffer */
	exc.framesPerBuffer = nFramesPerBuffer(mem_fd, BUFFER_SIZE, daq.rows, daq.cols * daq.nreads, BITS_PER_PIXEL);
	printf("Setting FPB to %d\n", (int) exc.framesPerBuffer);
	if (doCommand1(pci_fd, TIM_ID, FPB, exc.framesPerBuffer, DON) == _ERROR) {
	  printf("\tERROR: could not set frames-per-buffer\n");
	}

	//sleep(1);
	/* Set the number of frames */
	printf("Setting number of frames to %d\n", (int) daq.numberOfFrames);
	if (doCommand1(pci_fd, TIM_ID, SNC, daq.numberOfFrames, DON) == _ERROR) {
	  printf("\tERROR: could not set number of frames\n");
	}

	/* Set the exposure time */
	printf("Setting exposure time (ms): %d\n", daq.exptime);
	if (doCommand1(pci_fd, TIM_ID, SET, daq.exptime, DON) == _ERROR) {
	  printf("\tERROR: Setting exposure time failed -> 0x%X\n", getError());	    
	}

/* 	/\* Start the exposure *\/ */
/* 	printf("Starting exposure.\n"); */
/* 	if (doCommand(pci_fd, TIM_ID, SEX, DON) == _ERROR) { */
/* 	  printf("\tERROR: Starting exposure failed -> 0x%X\n", getError()); */
/* 	} */

	if (daq.continuousReadout) exc.continuousReadout=TRUE;

	exc.cmd = C_RUN;

	//n = write(newsockfd, "NO", 2);
	//if (n < 0) error("ERROR writing to socket");

	break;

      case SB:
	/* Do not execute if rt task running */
	if(exc.daqRunning) break;

        printf( "Setting PICNIC bias:\n");
        printf( "VOFFSET: %6.3f V\n", daq.voffset); 
        printf( "VRESET: %6.3f V\n" , daq.vreset);
        printf( "VD: %6.3f V\n" , daq.vd);
        printf( "ICTL: %6.3f V\n" , daq.ictl);
        
        if (doCommand4(pci_fd, TIM_ID, SBN, 0, DAC2, VID,
                       (int) (VOLTS2WORD * daq.voffset), DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
	}	
	usleep(500000);
        if (doCommand4(pci_fd, TIM_ID, SBN, 0, DAC3, VID,
                       (int) (VOLTS2WORD * daq.vreset), DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
        }
	usleep(500000);
        if (doCommand4(pci_fd, TIM_ID, SBN, 0, DAC4, VID,
                       (int) (VOLTS2WORD * daq.vd), DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
        }
	usleep(500000);
        if (doCommand4(pci_fd, TIM_ID, SBN, 0, DAC5, VID,
                       (int) (VOLTS2WORD * daq.ictl), DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
	}
	break;
      
      case AQ:
	/* Do not execute if rt task running */
	if(exc.daqRunning) break;
		//Length of nessage = 4 chars (int). this could differ in other architecture 
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {
	  message.data = (unsigned char *) realloc(message.data, message.length);
	  n = read(newsockfd, message.data, message.length);
	  if (n < 0) {
	    error("ERROR reading data from socket");
	    err = 1;
	  } else {
	    daq = *(daq_struct *) message.data;
	    printf("bufferSize = %d\n",BUFFER_SIZE);
	    printf("image_filename = %s\n",daq.image_filename);
	    exc.pixelCount = daq.rows * daq.cols * daq.nreads;
	    exc.byte_count_float = exc.pixelCount * sizeof(float);
	    exc.byte_count_short = exc.pixelCount * sizeof(unsigned short);
	    message.data = (unsigned char *) realloc(message.data, message.length);
	    exc.endProg = FALSE;
	    //spawn data acquisition task
	    printf("daqRunning = %d\n",exc.daqRunning);
	    printf("daq.continuousRead = %d\n",daq.continuousReadout);
	    printf("daq.doImageSave = %d\n",daq.do_image_save);
	    printf("daq message length, n  = %d %d\n",message.length,n);
	    /* Does not appear that dataBuffer is used anymore.. */
	    /* dataBuffer = (unsigned short *)realloc(dataBuffer, daq.framesPerReset * exc.byte_count_short); */
/* 	    if (dataBuffer == NULL){ */
/* 	      printf("Cannot allocate dataBuffer \n"); */
/* 	      exit (1); */
/* 	    } */

	    exc.cmd = C_INIT;
	  }
	}
	break;

      case CN:  // Change name of next file.
		//Length of nessage = 4 chars (int). this could differ in other architecture 
	printf("Request to Changen Name\n");
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {
	  message.data = (unsigned char *) realloc(message.data, message.length);
	  n = read(newsockfd, message.data, message.length);
	  if (n < 0) {
	    error("ERROR reading data from socket");
	    err = 1;
	  } else {
	    daq = *(daq_struct *) message.data;
	    message.data = (unsigned char *) realloc(message.data, message.length);
	    strcpy(exc.image_filename,daq.image_filename);  /* update name */
	    printf("Changing next image name to: %c\n", exc.image_filename);
	  }
	}
	break;

      case SD:  // Save Data. Actually this updates all daq parameters!!
		//Length of nessage = 4 chars (int). this could differ in other architecture 
	printf("Request to Change Name\n");
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {
	  message.data = (unsigned char *) realloc(message.data, message.length);
	  n = read(newsockfd, message.data, message.length);
	  if (n < 0) {
	    error("ERROR reading data from socket");
	    err = 1;
	  } else {
	    daq = *(daq_struct *) message.data;
	    message.data = (unsigned char *) realloc(message.data, message.length);
	  	    printf("Saving Images: %d\n",daq.do_image_save);
	  }
	}
	break;

      case SC:
	/* Do not execute if rt task running */
	if(exc.daqRunning) break;
	
	/* -----------------------------------
	   POWER ON
	   ----------------------------------- */
	printf("Doing power on...");
	if (doCommand(pci_fd, TIM_ID, PON, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
	  //return _ERROR;
	}
	else
	  printf("done.\n");
	    

	/* -----------------------------------
	   SET DIMENSIONS
	   ----------------------------------- */

	printf("Setting image columns %d ...", daq.cols);
	if (doCommand2(pci_fd, TIM_ID, WRM, (Y | 1), daq.cols * daq.nreads, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
	  //return _ERROR;
	}
	else
	  printf("done.\n");

	printf("Setting image rows %d ...", daq.rows);
	if (doCommand2(pci_fd, TIM_ID, WRM, (Y | 2), daq.rows, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());
	  //return _ERROR;
	}
	else
	  printf("done.\n");

	/* --------------------------------------
	   READ THE CONTROLLER PARAMETERS INFO
	   -------------------------------------- */
	configWord = doCommand(pci_fd, TIM_ID, RCC, UNDEFINED);


	/* --------------------------------------
	   SELECT SUBARRAY MODE
	   -------------------------------------- */
	printf("Setting subarray mode: row = %d col = %d...\n", daq.rowOffset, daq.colOffset);
	/* Row and column offset are swapped due to a documentation error */
	if (doCommand2(pci_fd, TIM_ID, SSA, daq.colOffset, daq.rowOffset, DON) == _ERROR){
	  printf("failed -> 0x%X\n", getError());
	}

	/* --------------------------------------
	   SET NUMBER OF READS PER PIXEL
	   -------------------------------------- */
	printf("Setting number of reads per pixel = %d...\n", daq.nreads);
	if (doCommand1(pci_fd, TIM_ID, SNR, daq.nreads, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());	      
	}

	/* --------------------------------------
	   SET TRIGGER MODE
	   -------------------------------------- */
	printf("Setting trigger mode = %d...\n", trigger);
	if (doCommand1(pci_fd, TIM_ID, STM, trigger, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());	      
	}

	/* --------------------------------------
	   SET READOUT MODE TO CDS
	   -------------------------------------- */
	printf("Setting CDS mode = %d...\n", cds);
	if (doCommand1(pci_fd, TIM_ID, CDS, cds, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());	      
	}


	/* --------------------------------------
	   SET NUMBER OF FRAMES PER RESET
	   -------------------------------------- */
	printf("Setting number of frames per reset = %d...\n", daq.framesPerReset);
	if (doCommand1(pci_fd, TIM_ID, SFR, daq.framesPerReset, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());	      
	}

	/* --------------------------------------
	   SET READ SIGNAL TOGGLE
	   -------------------------------------- */
	printf("Enable READ signal toggle =  %d...\n", daq.readToggle);
	if (doCommand1(pci_fd, TIM_ID, SRD, daq.readToggle, DON) == _ERROR) {
	  printf("failed -> 0x%X\n", getError());	      
	}

	break;

	//JDM Add writing to ople server (test code)
      case OP:
	//Length of nessage = 4 chars (int). this could differ in other architecture 
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {
	  message.data = (unsigned char *) realloc(message.data, message.length);
	  n = read(newsockfd, message.data, message.length);
	  if (n < 0) {
	    error("ERROR reading data from socket");
	    err = 1;
	  } else {
	    memcpy(gdt.dlOffsets, message.data, message.length);
            // JDM should only update a delay line that is clicked.
	    // currently the python gui send all offsets -- should change!
	    for(i = 0;i < NUMDLS; i++){
	      gdt.manualOffset[i]+=gdt.dlOffsets[i];
	      // printf("dl received %d %lf\n",i,gdt.dlOffsets[i]);
	    }
	    exc.manOffsetsReady = TRUE;
	  }
	}
	break;

    case GI:
	//Length of nessage = 4 chars (int). this could differ in other architecture 
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {
	  message.data = (unsigned char *) realloc(message.data, message.length);
	  n = read(newsockfd, message.data, message.length);
	  if (n < 0) {
	    error("ERROR reading data from socket");
	    err = 1;
	  } else {
	    memcpy(&gdt.gdtLock, message.data,4);
	    memcpy(gdt.activeBaselines, message.data + 4,message.length-4);
	    printf("%d\n",(gdt.gdtLock));
	    printf("Receiving active baselines from client %i %i %i %i %i %i\n",
		   gdt.activeBaselines[0],gdt.activeBaselines[1],gdt.activeBaselines[2],gdt.activeBaselines[3],gdt.activeBaselines[4],gdt.activeBaselines[5]);	
	  }
	}
		break;




      case OF:
	//Length of nessage = 4 chars (int). this could differ in other architecture
	// read data length
	printf("Received Request OF..");
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	// printf("PS Request length = %d\n",message.length);
	if (n < 0) {
	  error("ERROR reading message length");
	  printf("ERROR in OF");
	  err = 1;
	} else {
	  // allocate data buffer
	  message.data = (unsigned char *) realloc(message.data, message.length);
          memcpy((unsigned char *) message.data, dloffsets, message.length);

	  // send data through socket  
	  n = write(newsockfd, (unsigned char *) message.data, message.length);
	  //	  printf("Sending DL Offsets to client %lf %lf %lf %lf %lf %lf\n",dloffsets[0],dloffsets[1],dloffsets[2],dloffsets[3],dloffsets[4],dloffsets[5]);	
	  if (n < 0) error("ERROR writing to socket");
	}
	break;

      case FI:
	//Length of nessage = 4 chars (int). this could differ in other architecture
	// read data length
	n = read(newsockfd, (unsigned int *) &message.length, 4);
	// printf("PS Request length = %d\n",message.length);
	if (n < 0) {
	  error("ERROR reading message length");
	  err = 1;
	} else {

	  // allocate data buffer
	  message.data = (unsigned char *) realloc(message.data, message.length);
          memcpy((unsigned char *) message.data, exc.gdtOpd, message.length / 2);
          memcpy((unsigned char *) message.data + message.length / 2, exc.gdtWeight, message.length / 2);

	  // send data through socket  
	  n = write(newsockfd, (unsigned char *) message.data, message.length);
	
	  if (n < 0) error("ERROR writing to socket");
	}
	break;



/* 	dloffsets[5]=dloffsets[5]+.0001; //increment dl by 100nm */
/* 	set_manual_target(opleserver,5,dloffsets[5],0,0); */

	break;


      case TB:
	exc.takeBkgnd = TRUE;
	printf("Take Background Now Set for %i\n",exc.takeBkgnd);
	break;

      case RB:
	exc.resetBkgnd = TRUE;
	exc.useBkgnd = FALSE;
	printf("Reset Background Now Set for %i\n",exc.resetBkgnd);
	break;

      case UB:
	if (exc.useBkgnd == TRUE) {exc.useBkgnd = FALSE; }
	else {
	  exc.useBkgnd = TRUE;
	}
	printf("Use Background Now Set for %i\n",exc.useBkgnd);
	break;

      default:
	  
	break;
      }

    }
    if (!err) {
      n = write(newsockfd,"OK",2);
      if (n < 0) error("ERROR writing to socket");
    } else {
      n = write(newsockfd,"KO",2);
      if (n < 0) error("ERROR writing to socket");
    }
      
    //printf("Here is the message: %i\n",message.length);
    //n = write(newsockfd,"I got your message",18);
    //if (n < 0) error("ERROR writing to socket");
    close(newsockfd);
  }

  // Free image memory
  free(message.data);
  close(sockfd);
  if (mem_fd != NULL) 
    munmap((char *)mem_fd, BUFFER_SIZE);
  close(pci_fd);
  fflush(NULL);
  fprintf(stdout,"exit\n");

  //JDM close out the opleserver socket which had remained open.
  // moved to target task EP
/*   close_server_socket(opleserver); */ 
  /* USED? jdm      
    free(dataBuffer);
  */
  return 0;
}


/************************************************************************/
/*  using reverse_byte_order() from control/libs/rwfits/fits_util.c     */
/************************************************************************/
/* reverse_byte_order()                                                 */
/*                                                                      */
/* Reverses the byte order from the pointer location including the      */
/* given number of bytes.                                               */
/************************************************************************/

void reverse_byte_order(void *p, int nbytes)
{
        unsigned char *c1,*c2;
        int     i;
        unsigned char *np;

        /* Allocate enough memory to do this */

#ifdef DEBUG2
        printf("nbytes = %d\n",nbytes);
        fflush(stdout);
#endif
        np = (unsigned char *)malloc(nbytes);
	// mem_test(np);  // test to see if out of memory -- not needed!!

        /* Make a copy of the bytes */

        for(c1 = p, c2 = np, i=0; i<nbytes; i++,c1++,c2++)
        {
                *c2 = *c1;
        }

        /* Now copy it back in reverse order */

        for(c1 = p, c2 = np+nbytes-1, i=0; i<nbytes; i++, c1++, c2--)
        {
                *c1 = *c2;
        }

#ifdef DEBUG2
        for(c1 = p, c2 = np, i=0; i<nbytes; i++,c1++,c2++)
        {
                printf("%d %d\n",*c2,*c1);
        }
        printf("\n");
        fflush(stdout);
#endif

        /* Release the temp memory space */

        free(np);

} /* reverse_byte_order() */

/********************************************************************************/
/*  open_connect_socket()        taken from /control/libs/charaui/src/socket.c  */
/************************************************************************/
/* open_connect_socket()                                                */
/*                                                                      */
/* Attempts to open up a socket port for connection. Requires a         */
/* machine name and a port number. Returns the socket file descriptor   */
/* if successful or a -1 if it fails.                                   */
/* The corresponding function close_connect_socket() is a simple        */
/* macro for close().                                                   */
/* This may not work so well for STANDALONE machines...                 */
/************************************************************************/

int open_connect_socket(char *machine, int port)
{
        struct sockaddr_in server;      /* Server address */
        struct hostent *hp;             /* Pointer to host data */
        int     sock;                   /* Socket number */
        int     buf_size;               /* Size of buffer */

	/* removing error checking!! 2006Mar30 JDM */
        /* First see if we can identify the host. */

        if((hp = gethostbyname(machine)) == 0)
        {
                printf("Host %s unknown.",machine);
                return -1;
        }

        /* We can not connect to ourselves. */

/*         if (strcmp(hp->h_name,myname) == 0 && port == myport) */
/*         { */
/*                 return (WARNING,"We can not connect to ourselves.\n"); */
/*         } */

        /* Create a socket */

        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        {
                printf("Failed to create socket.\n");
                return -1;
        }

        /* Force buffer sizes to SOCKET_BUFFER */

        buf_size = SOCKET_BUFFER;

        if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF,
                (char *)&buf_size, sizeof(buf_size)) < 0)
        {
                close(sock);
                printf("Failed to change send buffer size to %d\n",
                        buf_size);
                return -1;
        }
     if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF,
                (char *)&buf_size, sizeof(buf_size)) < 0)
        {
                close(sock);
                printf("Failed to change receive buffer size to %d\n",
                        buf_size);
                return -1;
        }

        /* Create server address */

        server.sin_family = AF_INET;
        strncpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
        server.sin_port = htons(port);

        /* Let's try and connect now.... */

        if (connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0)
        {
                close(sock);
                printf("Failed to connect to %s port %d.",machine,port);
                return -1;
        }

        /* All is well it seems */

        return sock;

} /* open_connect_socket() */

