/************************************************************************/
/* astrogtk_init_window.c                                               */
/*                                                                      */
/* Routine for initling the basic CHARA astrogtk functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : May   2006                                                  */
/************************************************************************/

#include "astrogtk.h"
#include "../../cliserv/include/chara_messages.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../charalib/chara.h"
#include "../slalib/slalib.h"

/* Globals */

int    last_reference_cart = 0;
static GtkWidget *astrogtk_init_window;
static char astrogtk_init_window_up = FALSE;
static GtkWidget *pop_select_menus[NUM_SCOPES];
static GtkWidget *pop_menu_items[NUM_SCOPES][NUM_POPS];
static GtkWidget *beam_select_menus[NUM_SCOPES];
static GtkWidget *beam_menu_items[NUM_SCOPES][NUM_BEAMS];
static GtkWidget *ref_select_menu;
static GtkWidget *ref_menu_items[NUM_SCOPES];
struct s_init {int server; char doit; char send_it;} ok, cancel;
int (*config_callback)(void) = NULL;

#define ASTRO_CONTROL_WIDTH	200

/************************************************************************/
/* create_astrogtk_init_window()					*/
/*									*/
/* ROutine for creating the astrogtk init window. This should be self*/
/* contained and give you all you need to init the basic astrogtk 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_astrogtk_init_window (GtkButton *thebutton, gpointer *data)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *button;
	GtkWidget *label;
	GtkWidget *pop_menu;
	GtkWidget *beam_menu;
	GtkWidget *ref_menu;
	struct astro_init_window_data *the_data;
	int	  i,j;

	/* Do nothing if we exist already */

	if (astrogtk_init_window_up) return astrogtk_init_window;

	/* Get the data */

	the_data = (struct astro_init_window_data *)data;

	/* Setup standard things */

	ok.server = the_data->server;
	ok.doit = TRUE;
	ok.send_it = the_data->get_config;
	cancel.server = the_data->server;
	cancel.doit = FALSE;
	cancel.send_it = FALSE;

	/* Find out what the current configuration is */

	if (the_data->get_config && 
		!astrogtk_get_current_config(the_data->server))
	{
		create_message_window("Failed to get current configuration.");
		return NULL;
	}

	/* and the current reference cart */

	if (the_data->get_config &&
		!astrogtk_get_current_reference_cart(the_data->server))
	{
		create_message_window("Failed to get current reference cart.");
		return NULL;
	}

	/* Create the window itself */

	astrogtk_init_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (astrogtk_init_window),
		the_data->name);
	gtk_container_set_border_width (GTK_CONTAINER 
		(astrogtk_init_window), 1);

	/* Box to put things in */

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (astrogtk_init_window), vbox);
	gtk_widget_show(vbox);

	/* Set up things for all the scopes */

	for(i=0; i<NUM_SCOPES; i++)
	{
		/* Make an hbox to put things in */

		hbox = gtk_hbox_new(FALSE, 0);
		gtk_container_add (GTK_CONTAINER (vbox), hbox);
		gtk_widget_show(hbox);

		/* Now put the label for the scope */

		label = gtk_label_new(scope_types[i]);
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);

		/* And the menu for the pops */

		pop_menu = gtk_option_menu_new();
		gtk_widget_show (pop_menu);
		pop_select_menus[i] = gtk_menu_new ();
		gtk_box_pack_start(GTK_BOX(hbox), pop_menu, TRUE, TRUE, 0);
		for(j=0; j<NUM_POPS; j++)
		{
			pop_menu_items[i][j] =
			    gtk_menu_item_new_with_label(pop_types[j]);
			gtk_widget_show(pop_menu_items[i][j]);
			gtk_menu_append(GTK_MENU (pop_select_menus[i]),
			    pop_menu_items[i][j]);
		}
		gtk_option_menu_set_menu (GTK_OPTION_MENU (pop_menu),
			pop_select_menus[i]);
		gtk_option_menu_set_history(GTK_OPTION_MENU (pop_menu),
			beam_config.pops[i]);

		/* And the menu for the beams */

		beam_menu = gtk_option_menu_new();
		gtk_widget_show (beam_menu);
		beam_select_menus[i] = gtk_menu_new ();
		gtk_box_pack_start(GTK_BOX(hbox), beam_menu, TRUE, TRUE, 0);
		for(j=0; j<NUM_BEAMS; j++)
		{
			beam_menu_items[i][j] =
			    gtk_menu_item_new_with_label(beam_types[j]);
			gtk_widget_show(beam_menu_items[i][j]);
			gtk_menu_append(GTK_MENU (beam_select_menus[i]),
			    beam_menu_items[i][j]);
		}
		gtk_option_menu_set_menu (GTK_OPTION_MENU (beam_menu),
			beam_select_menus[i]);
		gtk_option_menu_set_history(GTK_OPTION_MENU (beam_menu),
			beam_config.beams[i]);
	}

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* Now put the label for the reference cart */

	label = gtk_label_new("Reference Cart");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	/* And the menu for the pops */

	ref_menu = gtk_option_menu_new();
	gtk_widget_show(ref_menu);
	ref_select_menu = gtk_menu_new ();
	gtk_box_pack_start(GTK_BOX(hbox), ref_menu, TRUE, TRUE, 0);
	for(j=0; j<NUM_SCOPES; j++)
	{
		ref_menu_items[j] =
		    gtk_menu_item_new_with_label(scope_types[j]);
		gtk_widget_show(ref_menu_items[j]);
		gtk_menu_append(GTK_MENU (ref_select_menu),ref_menu_items[j]);
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (ref_menu),
		ref_select_menu);
	if (the_data->server != -1)
		gtk_option_menu_set_history(GTK_OPTION_MENU (ref_menu),
			reference_cart[the_data->server]);
	else
		gtk_option_menu_set_history(GTK_OPTION_MENU (ref_menu),
			last_reference_cart);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* CANCEL button */

	button = gtk_button_new_with_label ("CANCEL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_init_ok_callback), (gpointer)&cancel);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* OK button */

	button = gtk_button_new_with_label ("OK");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_init_ok_callback), (gpointer)&ok);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* Now, we can show this window if we like */

	gtk_widget_show(astrogtk_init_window);

	/* Make sure we only get one version of this window */

	astrogtk_init_window_up = TRUE;

	/* That should be all */

	return astrogtk_init_window;

} /* create_astrogtk_init_window() */

/************************************************************************/
/* astrogtk_init_ok_callback()						*/
/*									*/
/* Callback function for astrogtk init OK button. Stop background	*/
/* processing and destroy astrogtk init window.				*/
/************************************************************************/

void astrogtk_init_ok_callback(GtkButton *button, gpointer data)
{
	int	i,j;
	GtkWidget *item;
	struct smessage mess;
	struct s_init *action;
	int	really_destroy = TRUE;

	if (!astrogtk_init_window_up) return;

	action = (struct s_init *)data;

	if (action->doit)
	{
	    for(i=0; i<NUM_SCOPES; i++)
	    {
		item = gtk_menu_get_active((GtkMenu *)pop_select_menus[i]);
		for(j=0; j<NUM_POPS && pop_menu_items[i][j] != item; j++);
		beam_config.pops[i] = j;

		item = gtk_menu_get_active((GtkMenu *)beam_select_menus[i]);
		for(j=0; j<NUM_BEAMS && beam_menu_items[i][j] != item; j++);
		beam_config.beams[i] = j;
	    }

	    item = gtk_menu_get_active((GtkMenu *)ref_select_menu);
	    for(j=0; j<NUM_SCOPES && ref_menu_items[j] != item; j++);
	    if (action->server != -1) reference_cart[action->server] = j;
	    last_reference_cart = j;

	    if (action->send_it)
	    {
		mess.type = ASTROMOD_INIT;
		mess.length = sizeof(beam_config);
		mess.data = (unsigned char *)&beam_config;

		if (!send_message(action->server, &mess))
		    create_message_window(
			"Failed to send ASTROMOD_INIT message.");

		mess.type = ASTROMOD_SET_REFERENCE_CART;
		mess.length = sizeof(reference_cart[action->server]);
		mess.data = (unsigned char *)&(reference_cart[action->server]);

	    	if (!send_message(action->server, &mess))
		    create_message_window(
			"Failed to send ASTROMOD_SET_REFERENCE_CART message.");
	    }

	    if (config_callback != NULL) really_destroy = (*config_callback)();
	}

	if (really_destroy)
	{
		gtk_widget_destroy(astrogtk_init_window);
		astrogtk_init_window_up = FALSE;
	}

} /* astrogtk_init_ok_callback() */

/************************************************************************/
/* astrogtk_get_current_config()					*/
/*									*/
/* Try and find out what the current config is right now.		*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_config(int server)
{
        struct smessage mess;
        time_t  current_time;
	int	old_beam;

        /* The current targets are not valid */

	old_beam = beam_config.beams[0];
        beam_config.beams[0] = BEAM_NONE;

        /* Send message requesting targets */

        mess.type = ASTROMOD_CURRENT_CONFIG;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
                create_message_window(
                        "Failed to send ASTROMOD_CURRENT_CONFIG message.");
		beam_config.beams[0] = old_beam;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(beam_config.beams[0] == BEAM_NONE)
        {
                process_server_socket(server);
                if (time(0) > current_time + 2)
		{
			beam_config.beams[0] = old_beam;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_config() */

/************************************************************************/
/* astrogtk_get_current_reference_cart()				*/
/*									*/
/* Try and find out what the current reference_cart is right now.	*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_reference_cart(int server)
{
        struct smessage mess;
        time_t  current_time;
	int	old_reference_cart;

        /* The current targets are not valid */

	old_reference_cart = reference_cart[server];
        reference_cart[server] = -1;

        /* Send message requesting targets */

        mess.type = ASTROMOD_WHICH_REFERENCE_CART;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
                create_message_window(
                       "Failed to send ASTROMOD_WHICH_REFERENCE_CART message.");
		reference_cart[server] = old_reference_cart;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(reference_cart[server] == -1)
        {
                process_server_socket(server);
                if (time(0) > current_time + 2)
		{
			reference_cart[server] = old_reference_cart;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_reference_cart() */

/************************************************************************/
/* set_config_callback()						*/
/*									*/
/* Allows calling program to set a callback function for configuration	*/
/* changes.								*/
/************************************************************************/

void set_config_callback(int (*callback)(void))
{
	config_callback = callback;

} /* set_config_callback() */


/************************************************************************/
/* unset_config_callback()						*/
/*									*/
/* Allows calling program to remove a callback function for		*/
/* configuration changes.						*/
/************************************************************************/

void unset_config_callback(void)
{
	config_callback = NULL;

} /* set_config_callback() */
