/************************************************************************/
/* astro_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : May 2006							*/
/************************************************************************/

#include "astrogtk.h"
#include "../../cliserv/include/chara_messages.h"

/* Globals */

int	chara_number[MAX_SERVERS]; /* Chara number of current star */
int	reference_cart[MAX_SERVERS];   /* Which reference cart */
struct s_astromod_init beam_config;

/************************************************************************/
/* setup_astro_messages()						*/
/*									*/
/* Guess what this does.						*/
/************************************************************************/

int setup_astro_messages(int server)
{
	int	i;

	/* Here we set up the local messages */

	if (!add_message_job(server, ASTROMOD_INIT,
		message_astromod_init)) return FALSE;

	if (!add_message_job(server, ASTROMOD_SET_STAR,
		message_astromod_set_star)) return FALSE;

	if (!add_message_job(server, ASTROMOD_SET_REFERENCE_CART,
		message_astromod_set_reference_cart)) return FALSE;

	chara_number[server] = NOSTAR;
	reference_cart[server] = S1;
	for(i=0; i<NUM_SCOPES; i++)
	{
		beam_config.beams[i] = BEAM_1;
		beam_config.pops[i] = POP_1;
	}

	return TRUE;

} /* setup_astro_messages() */

/************************************************************************/
/* message_astromod_init()						*/
/*									*/
/* Responds to ASTROMOD_INIT message from server.			*/
/************************************************************************/

int message_astromod_init(int server, struct smessage *mess)
{
	struct s_astromod_init	*data;
	int	i;

	/* Make sure data is right length */

	if (mess->length != sizeof(*data))
	{
	    create_message_window(
	    "Recieved ASTROMOD_INIT message with wrong amount of data.");
	}

	data = (struct s_astromod_init *)mess->data;

	for(i=0; i<NUM_SCOPES; i++)
	{
		beam_config.beams[i] = data->beams[i];
		beam_config.pops[i] = data->pops[i];
	}

	return NOERROR;

} /* message_astromod_init() */

/************************************************************************/
/* message_astromod_set_star()						*/
/*									*/
/* Responds to ASTROMOD_SET_STAR message from server.			*/
/************************************************************************/

int message_astromod_set_star(int server, struct smessage *mess)
{
	int	*data;

	/* Make sure data is right length */

	if (mess->length != sizeof(*data))
	{
	    create_message_window(
	    "Recieved ASTROMOD_WHICH_STAR message with wrong amount of data.");
	}

	data = (int *)mess->data;

	chara_number[server] = *data;

	return TRUE;

} /* message_astromod_set_star() */

/************************************************************************/
/* message_astromod_set_reference_cart()				*/
/*									*/
/* Responds to ASTROMOD_SET_REFERENCE_CART message from server.		*/
/************************************************************************/

int message_astromod_set_reference_cart(int server, struct smessage *mess)
{
	int	*data;

	/* Make sure data is right length */

	if (mess->length != sizeof(*data))
	{
	    create_message_window(
"Recieved ASTROMOD_WHICH_REFERENCE_CART message with wrong amount of data.");
	}

	data = (int *)mess->data;

	reference_cart[server] = *data;

	return TRUE;

} /* message_astromod_set_reference_cart() */
