/************************************************************************/
/* testui.c								*/
/*                                                                      */
/* Test program for astromod UI interface.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "../../cliserv/include/chara_messages.h"
#include "astromod.h"
#include "astromod_ui.h"
#include <time.h>

int time_status(void);		/* Example background job below */
int astromod_status(void);

void open_function(void)
{
	astromod_init(NULL, NULL);
	set_no_star(0, NULL);
	background_start(0,NULL);
}

int main(int argc, char **argv)
{
	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA USER INTERFACE ASTROMOD TEST ROUTINE");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "ASTROMODUI VERSION 0.0";
	initialise_ui("menus.ini","help.ini","astroui");
	set_user_open_function(open_function);

	/* Setup background job(s) */

	background_add(time_status);
	background_add(astromod_status);

	/* Setup the messages */

	setup_astromod_messages();

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);
}

/************************************************************************/
/* time_status()							*/
/* 									*/
/* Displays the time in the status window.				*/
/************************************************************************/

int time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Local Time : ");
	wstandend(status_window);
	wprintw(status_window,"     %2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* time_status() */

/************************************************************************/
/* astromod_status()							*/
/* 									*/
/* Displays the astromod in the status window.				*/
/************************************************************************/

int astromod_status(void)
{
	int year, month, day;
	struct  tm      *gmt_now;
	time_t	current_time;
	double	ut;
	double  mjd;
	double az,el,ref_el,hour_angle;
	char	s[256];
	int	i;
	
	wstandout(status_window);
	mvwaddstr(status_window,1,0,"Star Name  : ");
	wstandend(status_window);
	wprintw(status_window,"%13s",star.name);

	time(&current_time);
	gmt_now = gmtime(&current_time);

	/* Note... not really Y2K cool.... */

        year = gmt_now->tm_year + 1900;
        if (year < 1950) year+=100;
	month = gmt_now->tm_mon+1;
	day = gmt_now->tm_mday;
	ut = ((double)gmt_now->tm_sec/(24.0*60.0*60.0) +
	      (double)gmt_now->tm_min/(24.0*60.0) +
	      (double)gmt_now->tm_hour/(24.0)) * M_PI * 2.0;
	mjd = astromod_mjd(year, month, day, ut);

	wstandout(status_window);
	mvwaddstr(status_window,2,0,"Mod Jul Dat: ");
	wstandend(status_window);
	wprintw(status_window,"%13.6f",mjd);
	
	wstandout(status_window);
	mvwaddstr(status_window,3,0,"Atomic Time: ");
	wstandend(status_window);
	wprintw(status_window,"%13.6f",astromod_atomic_time(mjd));
	
	astromod_azel_mjd(star, mjd, &az, &el, &ref_el, &hour_angle);

	wstandout(status_window);
	mvwaddstr(status_window,4,0,"Az/El      : ");
	wstandend(status_window);
	wprintw(status_window,"  %5.1f/%5.1f",az/D2R, el/D2R);

	wstandout(status_window);
	mvwaddstr(status_window,5,0,"Refract El : ");
	wstandend(status_window);
	wprintw(status_window,"  %11.1f",ref_el/D2R);

	wstandout(status_window);
	mvwaddstr(status_window,6,0,"Hour Angle : ");
	wstandend(status_window);
	wprintw(status_window,"%13s",time_to_string(hour_angle,s));

	wstandout(status_window);
	mvwaddstr(status_window,0,40,"Reference  : ");
	wstandend(status_window);
	wprintw(status_window,"%15s",ople_types[reference_cart]);
	for(i=0; i<NUM_DELAY_LINES; i++)
	{
		wstandout(status_window);
		mvwprintw(status_window,i+1,40,"%2s OPLE    : ", ople_types[i]);
		wstandend(status_window);
		wprintw(status_window,"%15.10f",
			astromod_cart_position(i, reference_cart, 
				1.0, az, el, star.chara_number));
	}

	return NOERROR;

} /* astromod_status() */

int test_create_astromod_data_file_name(int argc, char **argv)
{
	char s[256];

	message(system_window,create_astromod_data_file_name("directory/","test",s));

	return NOERROR;

}
