/************************************************************************/
/* astrocalc.c								*/
/*									*/
/* Routines for astronomical calculations.				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/*	Author: Theo ten Brummelaar					*/
/*									*/
/************************************************************************/

/*
 * Include files
 */	

#include "chara.h"
#include <math.h>

/************************************************************************/
/* calc_uv()								*/
/*									*/
/* Attempts to calculate the UV points given Dec, Hour angle 		*/
/* in radians and the Bx, By, and Bz of the baseline in meters.		*/
/* Wavelength specified in meters also. Result in cycles/arcsec.	*/
/************************************************************************/

void calc_uv(double dec, double ha, double Bx, double By, double Bz, 
		double lambda, double *u, double *v)
{
	*u = (Bx * cos(ha) - 
		By * sin(CHARA_LATITUDE) * sin(ha) +
		Bz * cos(CHARA_LATITUDE) * sin(ha)) / (206265.0 * lambda);

	*v = (Bx * sin(dec) * sin(ha) +
		By * (sin(CHARA_LATITUDE) * sin(dec) * cos(ha) + 
			cos(CHARA_LATITUDE) * cos(dec)) -
		Bz * (cos(CHARA_LATITUDE) * sin(dec) * cos(ha) - 
			sin(CHARA_LATITUDE) * cos(dec))) / (206265.0 * lambda);

} /* calc_uv() */
