/************************************************************************/
/* testastro.c                                                          */
/*                                                                      */
/* Test astrometric reading functions.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Dave Berger		                                        */
/* Date   :                                                             */
/************************************************************************/


#include <unistd.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "chara.h"

int main(int argc, char **argv)
{

	/*
	 * Internals
	 */
	double	S1_pos;
	double	S2_pos;
	double	E1_pos;
	double	E2_pos;
	double	W1_pos;
	double	W2_pos;
	double	now;
	TIME	timestruct;
	int	secs_since_midnight, time_used;
	char	name[81];
	char	hd[81];
	double	met_wavelength,n_air_obs,n_air_met;
	char	fixed_cart[81];
	int	i;

	/*
	 * Get the current time in seconds since midnight.
	 * Then round it off ot the previous whole number 
	 * of ASTROMOD_TICK_TIMES.
	 */
	now = GetTime(&timestruct);
	secs_since_midnight = (int)timestruct.ut;
	secs_since_midnight -= secs_since_midnight % ASTROMOD_TICK_TIME;


	if ( (open_astromod (".", *++argv)) )
	{
	   fprintf (stderr, "Could not open Astromod.\n");
	   exit (-1);
	}

	/* What is the name of the star? */

	if (get_star_name(name) != 0) 
		printf("UNKNOWN STAR.\n");
	else
		printf("NAME: %s\n",name);

	if (get_star_hd(hd) != 0) 
		printf("UNKNOWN HD.\n");
	else
		printf("HD: %s\n",hd);

	if (get_fixed_cart(fixed_cart) == 0)
		printf ("Fixed cart = %s\n", fixed_cart);
	else
		printf("Failed to get fixed cart.\n");

	if ((i=get_met_wavelength(&met_wavelength)) == 0)
		printf("Metrology wavelength = %e\n",met_wavelength);
	else
		printf("Failed to get metrology wavelength (%d)\n",i);

	time_used = secs_since_midnight;

	printf("Time now = %d\n", time_used);

	printf("Type negative time to stop\n");

	while(1)
	{
		printf("Enter a time : ");
		scanf("%d",&time_used);
		if (time_used < 0 ) break;

		if (get_cart_pos (time_used, "S1", &S1_pos) != 0 )
			S1_pos = 0.0;
		if (get_cart_pos (time_used, "S2", &S2_pos) != 0 )
			S2_pos = 0.0;
		if (get_cart_pos (time_used, "E1", &E1_pos) != 0 )
			E1_pos = 0.0;
		if (get_cart_pos (time_used, "E2", &E2_pos) != 0 )
			E2_pos = 0.0;
		if (get_cart_pos (time_used, "W1", &W1_pos) != 0 )
			E1_pos = 0.0;
		if (get_cart_pos (time_used, "W2", &W2_pos) != 0 )
			W2_pos = 0.0;
		if (get_n_air_obs (time_used, &n_air_obs) != 0)
			n_air_obs = 0.0;
		if (get_n_air_met (time_used, &n_air_met) != 0)
			n_air_met = 0.0;

		printf (
		"time = %d N_obs = %15.10f N_met = %15.10f\n",
			time_used, n_air_obs, n_air_met);
		printf("\tS1 pos = %15.10f\n",S1_pos);
		printf("\tS2 pos = %15.10f\n",S2_pos);
		printf("\tE1 pos = %15.10f\n",E1_pos);
		printf("\tE2 pos = %15.10f\n",E2_pos);
		printf("\tW1 pos = %15.10f\n",W1_pos);
		printf("\tW2 pos = %15.10f\n",W2_pos);
	}

	/* Clean up */

	exit(0);
}
