/************************************************************************/
/* testft.c 								*/
/*									*/
/* Test programme for simple X library.  				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 2005   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"
#include "chara.h"
#include "../interferometry/interferometry.h"

#define WIDTH	600
#define HEIGHT	300
#define SAMPLE  0.001	/* 1mS per sample */
#define N_ZERO	10
#define SLEEP	1

int main(int argc, char **argv)
{
	Window	theWindow;
	XImage	*theImage;
	char *picture1;
	char *picture2;
	float	*fringe;
	int	i;
	float	min,max;
	int	n, fft_n;
	float	Nph, sig,dL;
	float	pos;
	float	bias;
	float	frg_freq;
	float	frg_width;
	float	frg_vel;
	float	L;
	float	*fft;
	float	*ps;
	float	*freqs;
	float	*bp;
	float	*lp;
	float	*hp;
	float	*filt;
	float	*copy_fft;
	int	sub_mean;
	float	*inv;
	float	*env;
	float	mean;
	float	weight;
	int	frg_pos;
	float	*loc;
	float	iamp,iper;

	/* Go get the command line stuff */

	if (argc < 10 || (argc != 10 && argc != 12))
	{
	    fprintf(stderr,
 "usage: %s n fft_n bias Nph Sig dL(um) frg_freq frg_width sub_mean (I P)\n",
		argv[0]);
	    exit(-1);
	}

	sscanf(argv[1],"%d",&n);
	sscanf(argv[2],"%d",&fft_n);
	sscanf(argv[3],"%f",&bias);
	sscanf(argv[4],"%f",&Nph);
	sscanf(argv[5],"%f",&sig);
	sscanf(argv[6],"%f",&dL);
	sscanf(argv[7],"%f",&frg_freq);
	sscanf(argv[8],"%f",&frg_width);
	sscanf(argv[9],"%d",&sub_mean);
	if (argc > 10)
	{
		sscanf(argv[10],"%f",&iamp);
		sscanf(argv[11],"%f",&iper);
		iper *= 1e-6;
	}
	else
	{
		iamp = 0.0;
	}
		
	min = max = 0.0;

	/* Allocate memory */

	fringe = vector(1,n);
	loc = vector(1,n);

	/* Now fill out the fringe data */

	frg_vel = K_BAND_CEN * frg_freq;
	L = frg_vel * SAMPLE * n;
	dL *= 1e-6;
	printf("#TESTFT\n#=======\n\n");
	printf("#Fringe velocity = %.1f\n", frg_vel*1e6);
	printf("#Scan range      = %.1f\n", L*1e6);
	printf("#Total Fringe Tm = %.1f\n", n*SAMPLE*1e3);
	printf("#Total FFT Time  = %.1f\n", fft_n*SAMPLE*1e3);
	printf("#Nyquist         = %.1f\n", 1.0/(2.0*SAMPLE));

	for(i=1; i<=n; i++) 
	{
		pos = (float)i*L/(float)n - L/2 - dL;
		fringe[i] = iamp * sin((pos/iper)*2.0*M_PI) +
				bias + detect_fringe_value(Nph,1.0,sig,pos,
				1.0, K_BAND_CEN,K_BAND_WIDTH,0.0);
	}

	/* Initailze X */

        i = initX(NULL);
        if (i<=0)
        {
                fprintf(stderr,"Failed to open display.\n");
                exit(-1);
        }

	/* Now make the window */

	theWindow = openWindow("TEST FRINGE",200,100,WIDTH,HEIGHT);

	/* Create a picture of the fringe */

	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,fringe,n,&min,&max);
	printf("#Min = %f Max = %f\n",min,max);

	/* Display the picture */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#THE FRINGE ITSELF.\n");
	sleep(SLEEP);

	/* Now calculate the FFT of the fringe */

	fft = fft_fringe(fringe, n, fft_n, sub_mean);

	/* And the power spectrum and frequency range */

	ps = ps_fringe(fft, fft_n);
	freqs = fft_freqs(fft_n, SAMPLE);

	/* Work out the fringe weight */

	printf("#PS fringe weight = %f\n", 
		fringe_weight(ps, freqs, fft_n, frg_freq, frg_width));

	/* 
	 * Display the power spectrum
	 * NOTE: unlike most NRC vectors this one starts at 0 instead of
	 * 1, but since the 0 elemnt containts the DC part of things
	 * we do not plot it.
	 */

	min = max = 0.0;
	picture1 = plot_funct(WIDTH,HEIGHT,ps,fft_n/2,&min,&max);
	printf("#Min = %f Max = %f\n",min,max);
	XDestroyImage(theImage);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#Nyquist = %.1f\n", freqs[fft_n/2]);
	printf("#THE POWER SPECTRUM.\n");
	sleep(SLEEP);

	/* Create the filters */

	bp = make_bandpass_filter(freqs, fft_n, frg_freq, frg_width);
	lp = make_lowpass_filter(freqs, fft_n, 0.75*frg_freq, frg_width);
	hp = make_highpass_filter(freqs, fft_n, 1.25*frg_freq, frg_width);
	filt = vector(0, fft_n/2);

	/* And plot them over the power spectrum */

	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,ps,fft_n/2,&min,&max);
	XDestroyImage(theImage);
	set_plot_color(PLOT_RED);
	for(i=0; i<=fft_n/2; i++) filt[i] = bp[i] * max;
	picture2 = plot_funct(WIDTH,HEIGHT,filt,fft_n/2,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);

	set_plot_color(PLOT_GREEN);
	for(i=0; i<=fft_n/2; i++) filt[i] = lp[i] * max;
	picture2 = plot_funct(WIDTH,HEIGHT,filt,fft_n/2,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);

	set_plot_color(PLOT_DARK_BLUE);
	for(i=0; i<=fft_n/2; i++) filt[i] = hp[i] * max;
	picture2 = plot_funct(WIDTH,HEIGHT,filt,fft_n/2,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#FILTERS: RED-Bandpass, GREEN-Low pass, BLUE-High pass.\n");
	sleep(SLEEP);

	/* Now we see how well the inverse FFT works. */

	inv = invfft_fringe(fft, n, fft_n);

	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,inv,n,&min,&max);
	XDestroyImage(theImage);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#STRAIGHT INVERSE.\n");
	sleep(SLEEP);

	/* Now, is that very different from the original? */


	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,fringe,n,&min,&max);
	XDestroyImage(theImage);
	set_plot_color(PLOT_GREEN);
	picture2 = plot_funct(WIDTH,HEIGHT,inv,n,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);
	for(i=1; i<=n ;i++) inv[i] -= fringe[i];
	set_plot_color(PLOT_RED);
	picture2 = plot_funct(WIDTH,HEIGHT,inv,n,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#Min = %f Max = %f\n",min,max);
	printf("#DIFFERENCE.\n");
	sleep(SLEEP);

	/* Now we make a copy of the fft and apply filters */
	
	copy_fft = vector(1, fft_n);
	for(i=1; i<=fft_n; i++) copy_fft[i] = fft[i];
	use_filter(copy_fft, bp, fft_n);
	free_vector(inv, 1, n);
	inv = invfft_fringe(copy_fft, n, fft_n);

	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,inv,n,&min,&max);
	XDestroyImage(theImage);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#BANDPASS FILTERED SIGNAL.\n");
	sleep(SLEEP);

	for(i=1; i<=fft_n; i++) copy_fft[i] = fft[i];
	use_filter(copy_fft, lp, fft_n);
	free_vector(inv, 1, n);
	inv = invfft_fringe(copy_fft, n, fft_n);

	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,inv,n,&min,&max);
	XDestroyImage(theImage);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#LOWPASS FILTERED SIGNAL.\n");
	sleep(SLEEP);

	for(i=1; i<=fft_n; i++) copy_fft[i] = fft[i];
	use_filter(copy_fft, hp, fft_n);
	free_vector(inv, 1, n);
	inv = invfft_fringe(copy_fft, n, fft_n);

	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,inv,n,&min,&max);
	XDestroyImage(theImage);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#HIGHPASS FILTERED SIGNAL.\n");
	sleep(SLEEP);

	/* And now, the envelope of the fringe */

	if (!sub_mean)
	{
	    printf(
	    "#No mean subtraction means envelope stuff will not work well.\n");
	    printf("#We will instead force the bandpass filter.\n");
	    use_filter(fft, bp, fft_n);
	}

	env = envelope_fringe(fft, n, fft_n, &frg_pos);

	printf("#ENVELOPE\n");
	printf("#Position = %d\n", frg_pos);

	/* We add the mean of the fringe signal for plotting reasons */

	mean = 0.0;
	for(i=1; i<=n; i++) mean += fringe[i];
	mean /= (float)n;
	for(i=1; i<=n; i++) env[i] += mean;

	min = max = 0.0;
	set_plot_color(PLOT_WHITE);
	picture1 = plot_funct(WIDTH,HEIGHT,fringe,n,&min,&max);
	printf("#Fringe Min = %f Max = %f\n", min, max);
	XDestroyImage(theImage);

	set_plot_color(PLOT_RED);
	picture2 = plot_funct(WIDTH,HEIGHT,env,n,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);

	/* Where does it think the fringes are? */

	for(i=0; i<=n; i++) loc[i] = 0.0;
	loc[frg_pos] = 1.0;

	set_plot_color(PLOT_GREEN);
	min = max = 0.0;
	picture2 = plot_funct(WIDTH,HEIGHT,loc,n,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	/* FInally, do the group delay tracking part */

	printf("#Result from GDT = %d\n",
		gdt_fringe_track(fft, n, fft_n, SAMPLE, frg_freq, frg_width,
                &weight, &frg_pos));

	printf("#GDT\n");
	printf("#Weight = %f\n#Position = %d\n",weight, frg_pos);

	/* Plot a line to show where this might be */

	for(i=0; i<=n; i++) loc[i] = 0.0;
	loc[frg_pos] = 1.0;

	set_plot_color(PLOT_DARK_BLUE);
	min = max = 0.0;
	picture2 = plot_funct(WIDTH,HEIGHT,loc,n,&min,&max);
	merge_plots(picture1, picture2, WIDTH, HEIGHT);
	free(picture2);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, WIDTH, HEIGHT, theBitmapPad, 0);
	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,WIDTH,HEIGHT);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	printf("#FRINGE POSITION AND ENVELOPE.\n");
	printf(
"#WHITE-Signal, RED-Envelope, GREEN-Envelope Max, BLUE-GDT position.\n");
	getchar();

	/* Clean up */

	XDestroyImage(theImage);
	free_vector(fringe,1,n);
	free_vector(inv, 1, n);
	free_vector(fft, 1, fft_n);
	free_vector(env, 1, fft_n);
	free_vector(copy_fft, 1, fft_n);
	free_vector(ps, 0, fft_n/2);
	free_vector(bp, 0, fft_n/2);
	free_vector(lp, 0, fft_n/2);
	free_vector(hp, 0, fft_n/2);
	free_vector(filt, 0, fft_n/2);
	free_vector(freqs, 0, fft_n/2);
	XDestroyWindow(theDisplay, theWindow);
	quitX();
	exit(0);
}
