/************************************************************************/
/* charaui.h								*/
/*                                                                      */
/* General Header File.							*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

#ifndef __CHARAUI__
#define __CHARAUI__

/*
 * General include files 
 */

#include <stdio.h>
#include <curses.h>
#include <time.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdarg.h>
#include <ctype.h>

/*
 * Constants
 */

#define MAXLEVEL	20	/* Maximum menu level */
#define MAXSTRING	41	/* Maximum string length for prompts and names*/
#define MAXENTRY	10	/* Maximum number of entries in a menu */
#define MAXMENUS	100	/* Maximum number of menus */
#define MAXAUTO		100	/* Maximum number of ftns in auto list */
#define MAXSCRIPT	100	/* Maximum number of lines in script */

/* Check whgat these are.... do we use them at all or are they roberts? */

#define MAXFIELDS (MAIN_LINES-4)
#define MAXPAGES	50
#define MAX_WAIT_TIME	5
#define MAX_ATTEMPTS	2

/*
 * The constants above are the initial values for the global variables below.
 */

#define YES		1
#define NO		0
#ifndef TRUE
#define TRUE		1
#define FALSE		0
#endif
#define OFF		0
#define ON		1
#define EVEN		2
#define ODD		3

#define LOCK_DIR	"/var/lock"
#define LOCK_START	"LCK.."

#define F_HELP		"help"		/* get help */
#define F_SHELL		"shell"		/* operating system shell */

#define BMAIN_LINES	12
#define BMAIN_COLS	79
#define BMAIN_BEGIN_Y	9
#define BMAIN_BEGIN_X	0

/* Note that this window forces blank columns at begin and end of lines */

#define MAIN_LINES		(BMAIN_LINES-2)
#define MAIN_COLS		(BMAIN_COLS-4)
#define MAIN_BEGIN_Y		(BMAIN_BEGIN_Y+1)
#define MAIN_BEGIN_X		(BMAIN_BEGIN_X+2)

#define STATUS_LINES    	(21 - BMAIN_LINES)
#define STATUS_COLS		BMAIN_COLS
#define STATUS_BEGIN_Y		0
#define STATUS_BEGIN_X		0

#define HEADING_LINES		2
#define HEADING_COLS		MAIN_COLS
#define HEADING_BEGIN_Y		MAIN_BEGIN_Y
#define HEADING_BEGIN_X		MAIN_BEGIN_X

#define SUB_MAIN_LINES		(MAIN_LINES - HEADING_LINES)
#define SUB_MAIN_COLS		MAIN_COLS
#define SUB_MAIN_BEGIN_Y	(MAIN_BEGIN_Y + HEADING_LINES)
#define SUB_MAIN_BEGIN_X	MAIN_BEGIN_X

#define SYSTEM_LINES		2
#define SYSTEM_COLS		80
#define SYSTEM_BEGIN_Y		21
#define SYSTEM_BEGIN_X		0

#define COMMAND_LINES		1
#define COMMAND_COLS		70
#define COMMAND_BEGIN_Y		23
#define COMMAND_BEGIN_X 	0
#define LENGTH_CL               68 


#define MODE_LINES		1
#define MODE_COLS		(80 - COMMAND_COLS)
#define MODE_BEGIN_Y		23
#define MODE_BEGIN_X 		COMMAND_COLS

/* Each window dynamically allocated and is the same size as main window */

#define MENU_LINES		MAIN_LINES
#define MENU_COLS		MAIN_COLS
#define MENU_BEGIN_Y		MAIN_BEGIN_Y
#define MENU_BEGIN_X		MAIN_BEGIN_X

#define BOX_WIDTH	78	/* MUST be even */

#undef  KEY_HELP		/* It's also in curses.h */
#define KEY_HELP	'?'	/* Menu help key */
#ifdef JUNK
#define KEY_MENU_UP	'\b'    /* Move up to calling menu */
#else
#define KEY_MENU_UP	KEY_BACKSPACE    /* Move up to calling menu */
#endif
#define KEY_RISE	'^'	/* Go up to main menu (ESC key) */
#define KEY_SWAP	'\t'	/* Swap back to last menu */
#define MENU		(-1)	/* Can't be an ascii character */
#define KEY_ESC         '\033'

#define PARA_DELIM	".paragraph"
#define NEWLINE_DELIM	".nl"
#define CENTRE_DELIM	".center"
#define TAB_DELIM	".tab"
#define TABSTOP		8

#define HELP_HEADING	"Help is available for the following topics"

#define TAG		't'
#define UNTAG		'u'
#define VIEW		'v'
#define DELETE_KEY	'd'
#define PRINT		'p'

#define CHAR_TAG	'>'

/*
 * There is a potential inconsistancy or conflict with charaui.h here!
 */

#define	INTEGER		0
#define FLOAT		1
#define	STRING		2
#define ENUMERATED	3	

#define	PICK_CHOICE_MESS_1 "Type <SPACE BAR> to switch between choices."
#define PICK_CHOICE_MESS_2 "Type <ENTER> to select a choice or <ESC> to exit."

#define KEY_REFRESH_L	'\014'
#define KEY_REFRESH_R	'\022'

#define PROMPT	    ">"
#define MENU_INFO1  "<?>         Help"
#define MENU_INFO2  "<BACKSPACE> Previous menu"
#define MENU_INFO3  "<^>         MAIN menu"
#define MENU_INFO4  "<TAB>       %s menu"
#define MENU_INFO5  "<TAB>       Operational after <^>"
#define MENU_INFOB  "                                      "

#define MOVE_LINE_D	"line down      <DOWN ARROW> or <j>"
#define MOVE_LINE_U	"line up        <UP ARROW>   or <k>"
#define MOVE_PAGE_D	"page down      <PAGE DOWN>  or <f> or <SPACE BAR>"
#define MOVE_PAGE_U	"page up        <PAGE UP>    or <b>"
#define MOVE_BEGIN	"beginning      <HOME>       or <^>"
#define MOVE_END	"end            <END>        or <$>"

#define MOVE_KEY_HELP	"<h> help on movement keys  <ESC> finished   "
#define CONT_TEXT 	"Press <ESC> to continue."
#define ENUM_ED_TEXT 	\
	"Change choice with <SPACE BAR>. Press <ENTER> to confirm choice."
#define TAG_TEXT	"tag item       <t>"
#define UNTAG_TEXT	"untag item     <u>"
#define VIEW_TEXT	"view item      <v>"
#define DELETE_TEXT	"delete item    <d>"
#define PRINT_TEXT	"print item     <p>"

#define	NUM_TEXT_LINES	750		/* max number of lines in text file */
#define MAX_ITEMS	750		/* max number of files */
#define FILENAME	81		/* length of filename */
#define DESCRIPTION	MAIN_COLS	/* length of descriptive title */
#define LINE_LENGTH	81		/* length of line from file */


#define MAX_CONNECTIONS 20		/* Maximum number of socket connects */
#define BUFFER_SIZE     1024		/* For serial read/writes */
#define SOCKET_BUFFER   65536		/* For socket read/writes */

#define ALLOWED_DOMAIN "chara-array.org"/* allowed domain for socket commands */
#define SOCKET_COMMAND_STR "COMMAND_COMPLETE:%d\n"

#define DEFAULT_COMMAND_PORT 2000
#define DEFAULT_DATA_PORT 2100
#define DEFAULT_SERVER_PORT 2200
/*
 * Macro functions
 */

/*
 * Name changed from clear_screen to ui_clear_screen to avoid
 * conflict with /usr/5include/term.h
 */

/* #define kbhit()			char_waiting(fileno(stdin)) */

#define serial_char_waiting(port) char_waiting(port.fd)

#define serial_putstr(port,s)	write(port.fd,s,strlen(s))

#define ui_clear_screen()	system("clear")

#define menu_alloc()	(struct smenu *)malloc(sizeof(struct smenu))
#define list_alloc()	(struct slist *)malloc(sizeof(struct slist))

#define	working()	werase(system_ol_window); \
			mvwaddstr(system_ol_window,0,0,"Working. . ."); \
			touchwin(system_ol_window); \
			wrefresh(system_ol_window)

#define not_working()	touchwin(system_window);wrefresh(system_window)

#define clean_command_line()	wmove(command_window,0,2); \
				wclrtoeol(command_window); \
				wrefresh(command_window)

#define fill(x) 	for(i=0; i<BOX_WIDTH-2; i++) putchar(x)
#define blank_line()	putchar(CHAR_VERT); fill(' '); putchar(CHAR_VERT); \
			putchar('\n')

#define no_socket()	if (active_socket != -1)\
			{\
			   return error(ERROR,\
				"Command %s not allowed via socket.\n"\
				,argv[0]);\
			}

#define socket_test_args(num,use) if(active_socket != -1 && argc-1 != num)\
			{\
			    return error(ERROR,"usage: %s %s\n",argv[0],use);\
			}
					
#define close_connect_socket(fd) close(fd)

/*
 * Globals and structures.
 */

extern int	active_socket;		/* 
				 	 * The current active socket. 
					 * Set to -1 if no active socket.
					 */

extern char *active_machine;		/* 
				 	 * The current active machine. 
					 * Set to 0 if no active machine.
					 */

/*
 * Global variable to let other functions, like the error function
 * know that a background job is running.
 */

extern bool background_function;


/* A simmilar variable for the error function */

extern bool inside_error_function;

#ifndef FUNCTS
extern struct {				/* Structure for the look up 	*/
		char 	*name;		/* table for user callable 	*/
		int	(*function)(int argc, char **argv);  
					/* functions as defined in the  */
	} functions[];			/* file functs.c		*/
#endif

struct smenu 
{
        char    name[MAXSTRING];            /* The name of the menu */ 

        struct    {        
                char   prompt[MAXSTRING];   /* Prompt string for menu message */
                int    pt_funct;            /* Index to functions table	      */
                struct smenu *pt_menu;	    /* Pointer to next menu	      */

            } entry[MAXENTRY];		    /* One set of these for each entry*/

};

struct sserial 
{
	int    fd;		     /* Port file descriptor */
	char   name[81];	     /* Port name */
	char   lock_file[81];	     /* lock_file name */
        struct termios orig_termios; /* original termio data */
        struct termios curr_termios; /* current termio data */
};

extern struct smenu *menus[MAXMENUS];

extern int autolist[MAXAUTO]; /*Array of function indexes for auto proceedure */
extern char autoargs[MAXAUTO][81]; /* Arguments for auto list functions */

extern struct smenu	*gotonext[MAXLEVEL+1],
			*comefrom[MAXLEVEL+1],*store[MAXLEVEL+1];

extern int	ptgoto;		/* Index to gotonext array */
extern int	ptcome;		/* Index to comefrom array */
extern int	move_down_menu_tree;/*Flag for menu movement (away from main) */
extern int 	move_up_menu_tree;/* Flag for menu movement (towards main)   */
extern char	*TITLE;		/* Title shown on screen */

struct scred_page	{

	char			*title;		/* Page title */
	int			number_fields;	/* Number of fields in page */
	bool			page_ok;	/* TRUE if all fields are OK */
	
	struct sfield 	{
		
		int	field_length;		/* Length of this field */
		int 	field_type;		/* Types defined in lineedit.h*/
		char	**enum_strings;		/* List of strings for 
							enumerated types */
		char	*prompt;		/* Prompt for this field */
		int	y_pos_prompt;		/* Position of */
		int	x_pos_prompt;		/* prompt in window */
		int	y_pos_field;		/* Position of */
		int	x_pos_field;		/* Field in window */
		bool	(*setup)();		/* Function that places current
						   field value into string 
						   or integer so it can be
						   edited. Returns TRUE if
						   you are currently alowed
						   to edit this field */
		bool	(*check_value)();	/* Function that places edited
						   value back into the 
						   apropriate structure and 
					           tests the value. Returns
						   TRUE if it is OK */
		bool	field_ok;		/* TRUE if field OK */
			
			} fields[MAXFIELDS];
			
				} ;

/*
 * NOTE : The form of a setup function is as follows :
 *
 * bool	setup_data(str,check)
 *
 * char	*str; FOR STRING,FLOAT and INTEGER OR
 * int	*str; FOR ENUMERATED TYPES
 * bool check;
 * {
 *	if (check)
 *	{
 *		if (not allowed to edit this field)
 *		{
 *			issue warning;
 *			return FALSE;
 *		}
 *		Place appropriate message in system window;
 *	}
 *
 * 	put data into str;
 *	return TRUE;
 * }
 *
 * The form of a check function is as follows :
 *
 * bool check_data(szValue)
 *
 * char	*str; OR
 * int	str; AS ABOVE
 * {
 *	if (!(field valid)) return FALSE;
 *	put str into data;
 *	return TRUE;
 * }
 * 
 */

/*
 * The screen editor structure itself
 */

struct	scred	{
		int	number_pages;		/* Number of pages in job */

		struct scred_page *pages[MAXPAGES];	/* Pages in scred job */
		} ;

/*
 * Window declarations
 */

extern WINDOW  *status_window;
extern WINDOW	*bmain_window;
extern WINDOW	*main_window;
extern WINDOW	*heading_window;
extern WINDOW	*sub_main_window;
extern WINDOW 	*system_window;
extern WINDOW 	*system_ol_window;
extern WINDOW	*command_window;
extern WINDOW	*mode_window;

extern WINDOW	*active_window;		/* to be refreshed by fti_error() */

extern WINDOW  *active_menuwindow;	/* 
				 * To be refreshed by some functions
				 * normally if called by a socket.
				 */
extern struct slist	{
			struct sitem {
				char	description[DESCRIPTION];
				char	filename[FILENAME];
				bool	tagged;
				     } *item[MAX_ITEMS];

			int	number_items;

		} *help_list; 

/*
 * This structure holds formatted text from files. Functions that use
 * the structure must format the text prior to putting each line into the
 * structure. Also, there is a variable within the structure that keeps
 * count of how may lines (items) there are. This is used by the scroll_text
 * routine.
 */

extern struct sdisplay	{
			int	number_items;
			char	*string[NUM_TEXT_LINES];

		} *display;


/*
 * The following structure is for circular buffers 
 */

typedef	struct sbuffer{
			unsigned char	*bottom;
			unsigned char	*top;
			unsigned char 	*in;
			unsigned char	*out;

		} BUFFER;

/* 
 * These macros let you play with buffers
 */

#define	buffer_not_full(x)	(x->in != x->out)
#define buffer_full(x)		(x->in == x->out)
#define buffer_not_empty(x)	((x->out != x->in - 1) && \
				((x->out != x->top) || (x->in != x->bottom)))
#define buffer_empty(x)		((x->out == x->in - 1) || \
				((x->out == x->top) && (x->in == x->bottom)))

#define buffer_in(x,y)		{if (buffer_not_full(x)) \
					 {\
						*(x->in) = (y);\
						if (++(x->in) > x->top) \
							x->in = x->bottom;\
					 }\
				}
	
#define buffer_out(x,y)		{if (buffer_not_empty(x)) \
					 {\
						if (++(x->out) > x->top) \
							x->out = x->bottom;\
						(y) = *(x->out);\
					 }\
				}
/*
 * The following struct is for the scripting stuff.
 */

typedef	struct sscript{
			char	function[LENGTH_CL];
			char	arguments[LENGTH_CL];
			int	on_message;
			int	on_noerror;
			int	on_warning;
			int	on_error;
			int	on_fatal;
			int	on_goto;

		} SCRIPT;

/*
 * A linked list of scripts
 */

typedef struct sscripts{
				struct sscripts *last;
				struct sscripts *next;
				char name[LENGTH_CL];
				struct sscript lines[MAXSCRIPT];

			} SCRIPTS;

/* Global begining of linked list */

extern SCRIPTS	first_script;
extern SCRIPTS	last_script;

/* 
 * Error messages
 */

#define MESSAGE 1 /* Use error system for putting up a message */
#define NOERROR	0 /* No error has occured */
#define WARNING (-1) /* A warning, nothing too dangerous has happened */
#define ERROR	(-2) /* Some kind of major error.Prog does not have to reboot */
#define FATAL	(-3) /* A fatal error has occured. The programme must exit(1)*/ 

/*
 * WARNINGS
 */

/* Warning messages can only fit on 1 line. */

#define WARN_1 "Cannot call main menu from the command line. Use <ESC>."
#define WARN_2 "Command unknown : %s"
#define WARN_3 "No help available."
#define WARN_4 "Topic not found : %s"
#define WARN_5 "Initialisation file not found : %s"
#define WARN_6 "Problem loading help file %s : limited help available."
#define WARN_7 "Cannot open file : %s"
#define WARN_8 "Text line too long to center in window. Text not centered."
#define WARN_9 "Too many lines in help file. Information truncated."
#define WARN_10 "Selection not found in list : %s"
#define WARN_11 "No environment variable for shell. Shell not available."
#define WARN_12 "Too many entries in AUTOLIST. Truncated at %s."
#define WARN_13 "Too many entries in AUTOLIST. Truncated at GOTO %s."
#define WARN_14 "Exceeded maximum menu level of %d."
#define WARN_15 "Menu key unrecognised."
#define WARN_16 "Default out of range for enumerated type."
#define WARN_17 "Argument unknown: %s"
#define WARN_18 "Unable to create %s : Check syntax."
#define WARN_19 "Too many help files, some are ignored."
#define WARN_20 "Illegal port number of %d. Port number must be 0 or 1."

/*
 * FATAL
 */

#define FATAL_1 "Out of memory space for %s. Is the terminal 80x24?"

#define FATAL_2 "Cannot open menu initialisation file : %s\n\
This file must exist in the same directory as FTI.EXE.\n\
Consult your manual for further details on creating menus."

#define FATAL_3 "Exceeded maximum number of %d menus.\n\
Consult your manual for further details on creating menus."

#define FATAL_4 "Menu defined with same name as a function : %s\n\
Consult your manual for further details on creating menus."

#define FATAL_5 "Menu or function defined more than once : %s\n\
Consult your manual for further details on creating menus or functions."

#define FATAL_6 "No MAIN menu defined.\n\
Consult your manual for further details on creating menus."

#define FATAL_7 "Cannot find menu : %s\n\
Consult your manual for further details on creating menus."

#define FATAL_8 "Too many menu entries defined for menu : %s\n\
Consult your manual for further details on creating menus."

#define FATAL_9 "Menu refers to MAIN menu : %s\n\
Consult your manual for further details on creating menus."

#define FATAL_10 "Menu refers to itself : %s\n\
Consult your manual for further details on creating menus."

#define FATAL_11 "Menu %s reference does not exist : %s\n\
Consult your manual for further details on creating menus."

#define FATAL_12 "Cannot have GOTO as first entry in AUTOLIST.\n\
Consult your manual for further details on creating the AUTOLIST."

#define FATAL_13 "AUTOLIST GOTO reference does not exist : %s\n\
Consult your manual for further details on creating the AUTOLIST."

#define FATAL_14 "Function not allowed in AUTOLIST : %s\n\
Consult your manual for further details on creating the AUTOLIST."

#define FATAL_15 "AUTOLIST reference is not a function : %s\n\
Consult your manual for further details on creating the AUTOLIST."

/*
 * Defines for coms port
 * What's this about again ????

int	port;
#define PORT_1   1
#define PORT_2   2
*/

/*
 * Stuff for binary server routines.
 */

#define TIME_OUT	5

#define SERVER_SOCKET_BUFFER   65535		/* For socket read/writes */

#define MAX_JOBS	4000

/* Here we define the message structure */

#ifndef __CLISERV__
#ifndef __SMESSAGE__
struct smessage {
			unsigned short int	type;
			unsigned int		length;
			unsigned char    	*data;
		};

typedef struct smessage message_struct;
#define __SMESSAGE__
#endif
#endif

/* Globals */

extern int client_socket;		/* 
				 	 * The current active client socket. 
					 * Set to -1 if no active socket.
					 */

extern char *client_machine;		/* 
				 	 * The current active client machine. 
					 * Set to 0 if no active machine.
					 */

extern char *server_name;		/* The name of our server */

extern int number_text_lines;      	/* ...in file */ 

/*
 * Prototypes
 */

/* title.h */

extern void	wait_for_title(void);
extern void 	center_line(char *string);
extern void 	put_line(char *string);

/* startui.c */

extern void	start_ui(void);
extern void 	initialise_ui(char *menufile, char *helpfile,char *server_name);
extern void     set_user_open_function(void (*function)(void));

/* uiend.c */

extern int      ui_end(int argc, char **argv);
extern void	sig_end(int sig);
extern void     set_user_close_function(void (*function)(void));

/* window.c */

extern void 	init_curses(void);
extern void	close_curses(void);
extern WINDOW 	*set_up_menu_window(
	struct 	smenu *thismenu,
	struct 	smenu *lastmenu,
	int	     level);
extern void 	close_menu_window(WINDOW *menuwindow);
extern void 	refresh_menu_window(
	WINDOW 	*menuwindow,
	int	level);
extern int	ask_yes_no(char *string1,char *string2);
extern void	heading(WINDOW *win,char *string1,char *string2);
extern int 	message(WINDOW *win, char *fmt, ...);
extern int	printd(char *fmt, ...);
extern void	clear_display(void);
extern void	refresh_screen(void);
extern void 	plain_screen_on(void);
extern void 	plain_screen_off(void);

/* uiauto.c */

extern void	init_auto(void);
extern int	ui_auto(int argc, char **argv);
extern int	ui_auto_socket(void);


/* help.c */

extern int 	ui_help(int argc,char **argv);
extern int 	commands(int argc,char **argv);
extern void 	init_help(char *helpfilename);
extern void 	text_format(WINDOW *win, char *filename);
extern void 	fill_text_format(WINDOW *win, FILE *filename);
extern bool 	inc_number_text_lines(void);
extern void 	scroll_text(WINDOW *win);
extern void 	init_display(void);
extern int 	scripts(int argc, char **argv);

/* error.c */

extern int error(int level, char *fmt, ...);
extern int break_into_lines( 
	char	*output,
	int	sizeof_output,
	char	*input,
	int	max_line_width);
extern int error_messages_on(int argc, char **argv);
extern int error_messages_off(int argc, char **argv);
extern void set_user_error_function(void (*function)(void));

/* strcmpi.c */

extern int     strcmpi(char *string1, char *string2);

/* list.c */

extern int 	ui_list(
	char	*title,		/* ...for list of items */
	char	*sub_title,	/* ...for list of items */
	struct	slist *list,	/* pointer to list structure */
	char	*last_choice,	/* default place to start in list */
	char	**choice,	/* selected by user */
	bool	tag,		/* user is able to tag | untag items */
	bool	view,		/* user is able to view item */
	bool	can_delete,	/* user is able to delete item */
	bool	print);		/* user is able to print item */
extern void 	highlight(
	WINDOW 	*win,	/* window to operate in */
	int 	line,	/* position in window */
	int 	column,	/* position in window */
	char 	*text,	/* text to highlight */
	bool	tagged);	/* ...or not */
extern void 	unhighlight(
	WINDOW 	*win,	/* window to operate in */
	int 	line,	/* position in window */
	int 	column,	/* position in window */
	char 	*text,	/* text to unhighlight */
	bool	tagged);	/* ...or not */
extern void 	down_key(
	WINDOW	*win,		/* window to operate in */
	int	*line,		/* position in window */
	int	*item,		/* item counter to increment */
	int	number_items,	/* total number of items */
	char	*next_text,	/* text for next line */
	bool	tagged);		/* ...or not */
extern void 	up_key(
	WINDOW	*win,		/* window to operate in */
	int	*line,		/* position in window */
	int	*item,		/* item counter to increment */
	int	number_items,	/* total number of items */
	char	*next_text,	/* next line of text */
	bool 	tagged);		/* ...or not */
extern void 	key_help(
	bool	tag,	/* user is able to tag | untag items */
	bool	view,	/* user is able to view item */
	bool	can_delete,	/* user is able to delete item */
	bool	print);	/* user is able to print item */
extern void	help_line(bool scroll_flag);
extern int	find_list_item(struct slist *list,char *filename);
extern int	alloc_list_item(struct slist *list, int position);
extern void	free_list_item(struct slist *list, int position);

/* uiinit.c */

extern struct 	smenu *init_menu_structure(char *menufile);
extern void 	passone(FILE *structure_file);
extern struct 	smenu *checkone(void);
extern void 	passtwo (FILE *structure_file);
extern void 	passthree(FILE *structure_file);
extern char 	*getline(char *string, int n, FILE *stream);
extern char	*split_string(char *string, char **rest);

/* background.c */

extern int get_command(void);
extern bool kbhit(void);
extern int char_waiting(int fd);
int write_ready(int fd);
int exception(int fd);
extern int background(void);
extern int reset_status_window_refresh(int argc, char **argv);
extern int background_on(int argc, char **argv);
extern int background_off(int argc, char **argv);
extern int background_start(int argc, char **argv);
extern int background_stop(int argc, char **argv);
extern int background_add_name(int (*job)(void), char *name);
extern int background_add(int (*job)(void));
extern int background_del(int (*job)(void));
extern int bypass_key_poll(int argc,char **argv);
extern void bypass_key_poll_off(int sig);
extern void call_wrefresh_curscr(int sig) ;
extern int go_to_sleep(int argc, char **argv);
extern void wake_up(int sig);
void set_top_job(int (*new_top_job)(void));

/* background_mirror.c */

int open_mirror_socket(int argc, char **argv);
int close_mirror_socket(int argc, char **argv);
int mirror_background(void);
int print_status(int argc, char **argv);

/* scred.c */

extern int	edit_page(struct scred_page *page);
extern bool	scred(struct scred *page_list);
extern void	copy_scred_page(struct scred_page *page_target,
			struct scred_page *page_source);
extern void	copy_scred_field(
struct scred_page	*page_target,	/* Target page */
int			field_target,	/* Target field in target page */	
struct scred_page	*page_source,	/* Source page */
int			field_source);	/* Source field in source page */	
extern bool	return_true(void);
extern int	quick_edit(
char	*item,		/* string describing what is being changed */
char	*status,	/* Default value */
void	*value,		/* pointer to string to be played with  */
char	*strings[],	/* string array for enumerated types */
int	value_type);	/* STRING, FLOAT, INTEGER or ENUMERATED */

/* lineedit.c */

extern int	line_edit(
WINDOW	*win,			/* Window that string lives in */
int	y, int x,		/* Position in window to place string */
char	*string,		/* The string to edit */
int	length,			/* The length of the string including NULL */
int	value_type,		/* Type of value string will hold ie :
				 * INTEGER, FLOAT or STRING */
bool	(*value_check)(),	/* Function to test value of string */
bool	insert_on);		/* TRUE if we are to go straight into insert
			         * mode, FALSE if we are to go back to what ever
				 * mode we were in before.
				 */

extern void 	show_string(
WINDOW *win,		/* Window to use */
int	y, int x,	/* Position in window to use */
char	*string,	/* The string to show */
int	length,		/* The length of the string */
int	cursor,		/* The cursor position */
char	blank);		/* The blank character to use */

extern void 	show_char(
WINDOW	*win,		/* Window to use */
int	position,	/* Current position in string */
int	cursor,		/* Current cursor position */
int	c);		/* Character to show */

extern int 	pick_choice(

WINDOW 	*win,			/* Window that value lives in */
int	y, int x,		/* Position in window to place value */
int	*value,			/* Variable to be set and default */
char	*strings[],		/* The array of strings to show */
bool	(*value_check)());	/* Function to test value */

/* command.c */

extern struct	smenu *command_processor(char first_char);
extern int	get_user_input(WINDOW *menuwindow, struct smenu **nextmenu);
extern bool check_command(char *command_line);
extern int	call_function(int function_pt, char *args);

/* uifind.c */

extern struct 	smenu *find_menu(char *menu_name);
extern int find_function(char *function_name);
extern struct sscripts *find_script(char *script_name);

/* uimenu.c */

extern void 	menu(
	struct 	smenu	*thismenu,	/* Pointer to current menu */
	struct 	smenu	*lastmenu,	/* Pointer to last menu */
	int	level);			/* Current menu level */
extern void 	put_store_reversed_into_gotonext(void);
extern void put_comefrom_into_store(void);

/* menuhelp.c */

extern void    get_menu_help(struct smenu *thismenu);

/* uibuf.c */

extern BUFFER	*set_up_buffer(int buf_size);
void		remove_buffer(BUFFER *buf);

/* shell.c */

extern int	ui_shell(int argc, char **argv);

/* uicoms.c */

extern struct sserial open_serial_port(char *port_name, bool blocking);
extern char 	*lock_file_name(char *port, char *s);
extern int	close_serial_port(struct sserial port);
extern int set_serial_baud_rate(struct sserial *port, speed_t speed);
extern int set_serial_parity(struct sserial *port, int parity);
extern int set_serial_xonxoff(struct sserial *port, int xonxoff);
extern int set_serial_hard_handshake(struct sserial *port, int hard_handshake);
extern int set_serial_bitlength(struct sserial *port, int bits);
extern int set_serial_stopbits(struct sserial *port, int bits);
extern unsigned char serial_getchar(struct sserial port);
extern unsigned char serial_putchar(struct sserial port, unsigned char c);
extern int serial_print(struct sserial port, char *fmt, ...);
extern int serial_scan(struct sserial port, char *fmt, ...);
extern char *serial_gets(struct sserial port, char *s, int n);
extern int     asccom(int argc, char **argv);

/* socket.c */

extern int open_command_socket(void);
extern void close_command_socket(void);
extern int     socket_print(int sock, char *fmt, ...);
extern int     all_socket_print(char *fmt, ...);
extern int socket_scan(int sock, char *fmt, ...);
extern int test_socket_scan(int argc, char **argv);
extern char *socket_gets(int sock, char *s, int n);
extern int process_command_socket(void);
extern char *get_machine_name(char *name);
extern int get_command_port(void);
extern int get_data_port(void);
extern void block_sockets(void);
extern void unblock_sockets(void);
extern int call_block_sockets(int argc, char **argv);
extern int call_unblock_sockets(int argc, char **argv);
extern int open_connect_socket(char *machine, int port);
extern int soccom(int argc, char **argv);
extern int ping(int argc, char **argv);
extern int start_socket_command_monitor(int argc, char **argv);
extern int stop_socket_command_monitor(int argc, char **argv);
extern int start_socket_command_log(int argc, char **argv);
extern int stop_socket_command_log(int argc, char **argv);
extern int     machines_connected(int argc, char **argv);
extern int open_data_socket(void);
extern void close_data_socket(void);
extern int read_data(void *buf, int n);
extern int read_data_fast(void *buf, int n);
extern int data_ready(void);
extern int write_data(void *buf, int n);
extern int write_data_fast(void *buf, int n);
extern int data_connected(void);
extern int data_open(void);
extern int send_socket_command(char *machine, int port, char *command);
extern int call_send_socket_command(int argc, char **argv);
extern void flush_sockets(void);
extern void sig_pipe_text_sockets(int sig);

/* script.c */

extern void init_scripts(void);
extern void delete_scripts(void);
extern struct sscripts *new_script(struct sscripts data);
extern int delete_script(struct sscripts *old);
extern int parse_script(int argc, char **argv);
extern int call_script(struct sscripts *script_pt);
extern int user_message(int argc, char **argv);
extern int user_ask_yes_no(int argc, char **argv);
extern int do_nothing(int argc, char **argv);

/* server.c */

int open_message_socket(void);
void close_message_socket(void);
int	process_message_socket(void);
char *get_server_name(char *name);
char *get_server_computer_name(char *name);
void flush_server_socket(int sock);
void flush_server_sockets(void);
int ui_send_message(int sock, struct smessage *message);
void    sig_pipe(int sig);
int ui_send_message_all(struct smessage *message);
int ui_read_socket(int fd, char *buf, size_t count);
void standard_jobs(void);
void ui_add_message_job(unsigned int type, 
	int (*job)(struct smessage *message));
void ui_run_message_job(struct smessage *message);
void set_server_name(char *name);
int message_noop(struct smessage *message);
int message_message(struct smessage *message);
int message_ping(struct smessage *message);
int message_all_noop(struct smessage *message);
int message_all_ping(struct smessage *message);
int message_status(struct smessage *message);
int message_signal(struct smessage *message);
int message_send_message_all_clients(struct smessage *message);
bool any_client_signal_set(void);
bool client_signal_set(unsigned char signal);
void clear_client_signal(unsigned char signal);
int ui_print_message(int sock, char *fmt, ...);
int ui_print_message_all(char *fmt, ...);
int send_server_stopping(void);
int ui_print_log(char *fmt, ...);
int get_server_port(void);

#endif
