/************************************************************************/
/* job_control.c							*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT LIB   				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : Oct 2001							*/
/************************************************************************/

/*
 * Include files
 */

#include "../include/cliserv.h"

short int message_array[MAX_JOBS];
unsigned char signal_array[256];

/*
 * Some local globals
 */

struct sservers servers[MAX_SERVERS];
static char init_jobs_has_been_run = FALSE;

/************************************************************************/
/* init_jobs()								*/
/*									*/
/* Initializes the servers jobs lists.					*/
/************************************************************************/

void init_jobs(void)
{
	int	i,j;

	/* Make sure we only do this once! */

	if (init_jobs_has_been_run) return;

	/* First set all jobs to NULL */

	for (j=0; j<MAX_SERVERS; j++)
	{
		servers[j].fd = -1;
		servers[j].name[0] = 0;
		for (i=0; i<MAX_JOBS; i++) servers[j].message_jobs[i] = NULL;
	}

	for(i=0; i<=MAX_JOBS; i++) message_array[i] = i;
	for(i=0; i<=256; i++) signal_array[i] = i;

	init_jobs_has_been_run = TRUE;

} /* init_jobs() */

/************************************************************************/
/* add_message_job()							*/
/*									*/
/* Adds a new message job to the setup. Error if job already		*/
/* exists.								*/
/* Returns TRUE if all goes well, FALSE otherwise.			*/
/************************************************************************/

int add_message_job(int server,
		unsigned int type,
		int (*job)(int server, struct smessage *message))
{
	/* Is the server reasonable? */

	if (server < 0 || server >= MAX_SERVERS) return FALSE;

	/* Is it a reasonable messgae type ? */

	if (type < 0 || type >= MAX_JOBS) return FALSE;

	/* Is this message place already taken ? */

	if (servers[server].message_jobs[type] != NULL) return FALSE;

	/* Otherwise put it in place */

	servers[server].message_jobs[type] = job;

	return TRUE;

} /* add_message_job() */

/************************************************************************/
/* remove_message_job()							*/
/*									*/
/* Removes a message job from the setup. 				*/
/************************************************************************/

void remove_message_job(int server, unsigned int type)
{
	if (server < 0 || server >= MAX_SERVERS) return;

	/* Is it a reasonable messgae type */

	if (type < 0 || type >= MAX_JOBS) return;

	/* Otherwise set it to NULL */

	servers[server].message_jobs[type] = NULL;

	return;

} /* remove_message_job() */

/************************************************************************/
/* run_message_job()							*/
/*									*/
/* Attempts to run message job.						*/
/* Returns TRUE if it went well, FALSE otherwise.			*/
/************************************************************************/

int run_message_job(int server, struct smessage *message)
{
	/* Does this server exist? */

        if (server < 0 || server >= MAX_SERVERS) return FALSE;

	/* Is it safe to run this message? */

	if (message->type >= MAX_JOBS || 
		servers[server].message_jobs[message->type] == NULL)
	{
		return FALSE;
	}

	/* Yes it is, go ahead */

	(*(servers[server].message_jobs[message->type]))(server, message);

	return TRUE;

} /* run_message_job() */

/************************************************************************/
/* message_do_nothing()                                                 */
/*                                                                      */
/* A do nothing message.                                                */
/************************************************************************/

int message_do_nothing(int server, struct smessage *mess)
{
        return TRUE;

} /* message_do_nothing() */
