/************************************************************************/
/* gtktest.c                                                            */
/*                                                                      */
/* Test program, speaks to two servers.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Oct 2001 			                                */
/************************************************************************/

#define USE_GTK
#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"
#include "../../../cliserv/socket_manager/server/socket_manager.h"
#include <stdio.h>
#include <errno.h>
#include <time.h>

/* prototypes */

int open_server(int server);
void message_callback( GtkWidget *widget, gpointer   data );
void reopen_callback( GtkWidget *widget, gpointer   data );
gint background_code(gpointer data);
void signal_callback( GtkWidget *widget, gpointer   data );

/* Some global variables */

int server[2] = {-1, -1};
int	  ports[2];
char	  *machines[2];
char	  *names[2];
unsigned char current_signal = 0;
struct s_process_sockets process1, process2;
int	  serv_index[2] = {0,1};

int main( int   argc,
	      char *argv[] )
{
	GtkWidget *window;
	GtkWidget *button;
	GtkWidget *hbox,*vbox;
	GtkWidget *label;
	int	  i;
	char	name[81];
	struct s_status_window status_info[2];

	/* Let GTK parse command line */

	gtk_init (&argc, &argv);

	/* Let's make sure we have the right arguments */

	if (argc != 5 && argc != 3)
	{
		fprintf(stderr,"usage: %s machine1 port1 machine2 port2\n",
			argv[0]);
		fprintf(stderr,"or\n");
		fprintf(stderr,"%s server1 server2\n", argv[0]);
		exit(-1);
	}

	/* Initialise mesage jobs */

	init_jobs();

	/* Try and open the servers */

	if (argc == 5)
	{
		names[0] = machines[0] = argv[1];
		sscanf(argv[2],"%d",ports);
		names[0] = machines[1] = argv[3];
		sscanf(argv[4],"%d",ports+1);
	}
	else
	{
		if (socket_manager_get_process(argv[1], &process1) != 0)
		{
		    fprintf(stderr,	
		    "Failed to get data on process 1 from socket manager.\n");
		    exit(-3);
		}
		names[0] = process1.name;
		machines[0] =  process1.machine;
		ports[0] = process1.messages;

		if (socket_manager_get_process(argv[2], &process2) != 0)
		{
		    fprintf(stderr,	
		    "Failed to get data on process 2 from socket manager.\n");
		    exit(-3);
		}
		names[1] = process2.name;
		machines[1] =  process2.machine;
		ports[1] = process2.messages;
	}

	/* Attempt to open the first socket */

	if (!open_server(0))
	{
		fprintf(stderr,"Failed to open port %d on %s\n",
			ports[0],machines[0]);
		exit(-2);
	}

	if (!open_server(1))
	{
		fprintf(stderr,"Failed to open port %d on %s\n",
			ports[1],machines[1]);
		exit(-2);
	}

	/* Create a new window */

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

	/* Set name of window to unit name */

	gtk_window_set_title (GTK_WINDOW (window), "Test GUI");
	set_client_name("Test GUI");

	/* Set a handler for delete_event that immediately exits GTK. */

	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			GTK_SIGNAL_FUNC (delete_event), NULL);

	/* Sets the border width of the window. */

	gtk_container_set_border_width (GTK_CONTAINER (window), 10);

	/* We create a hbox to pack widgets into. */

	hbox = gtk_hbox_new(FALSE, 0);

	/* Put the box into the main window. */

	gtk_container_add (GTK_CONTAINER (window), hbox);

	/* Create things for each server */

	for(i=0; i<2; i++)
	{
		/* Put a vbox into the hbox */

		vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_add (GTK_CONTAINER (hbox), vbox);

		/* Add error and circuit breaker label */

		sprintf(name,"%s",names[i]);
		label = gtk_label_new(name);
		gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);

		/* REOPEN button */

		button = gtk_button_new_with_label ("REOPEN");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (reopen_callback),
			(gpointer)(i+serv_index));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* NOOP button */

		button = gtk_button_new_with_label ("NOOP");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (noop_callback),(gpointer) (server+i));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* MESSAGE button */

		button = gtk_button_new_with_label ("MESSAGE");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(message_callback),
			(gpointer)(server+i));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* PING button */

		button = gtk_button_new_with_label ("PING");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (ping_callback), 
			(gpointer) (server+i));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* ALL_NOOP button */

		button = gtk_button_new_with_label ("ALL_NOOP");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (all_noop_callback), 
			(gpointer)(server+ i));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* ALL_PING button */

		button = gtk_button_new_with_label ("ALL_PING");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (all_ping_callback), 
			(gpointer) (server+i));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* A status button, will not work on all machines! */

		sprintf(status_info[i].name,"SERVER %d",i+1);
		status_info[i].server = server[i];
		button = gtk_button_new_with_label ("UI_STATUS");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (create_status_window), 
			(gpointer) (&(status_info[i])));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);
		
		/* UI_SIGNAL button */

		button = gtk_button_new_with_label ("SIGNAL");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (signal_callback), 
			(gpointer) (server+i));
		gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		/* And we need to show the vbox */

		gtk_widget_show(vbox);
	}

	/* Add a quit buttont */

	button = gtk_button_new_with_label ("QUIT");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (delete_event), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	/* OK, show the hbox */

	gtk_widget_show(hbox);

	/* and the window */

	gtk_widget_show (window);

        /* Now start up the background process. */

        gtk_timeout_add(50, background_code, 0);

	/* Rest in gtk_main and wait for the fun to begin! */

	gtk_main ();

	return(0);
}

/************************************************************************/
/* open_server()							*/
/************************************************************************/

int open_server(int n)
{
	char	name[256];

	sprintf(name,"%s:%d",machines[n], ports[n]);
	if ((server[n] = open_server_socket(machines[n], ports[n], name)) == -1)
		return FALSE;

        if (!add_message_job(server[n], NOOP, message_noop) ||
            !add_message_job(server[n], MESSAGE, message_message) ||
            !add_message_job(server[n], UI_STATUS, message_status) ||
            !add_message_job(server[n], PING, message_ping) ||
            !add_message_job(server[n], SERVER_STOPPING, 
			message_server_stopping)) 
	{
		return FALSE;
	}

	return TRUE;

} /* open_server() */

/************************************************************************/
/* message_callback()							*/
/*									*/
/************************************************************************/

void message_callback( GtkWidget *widget, gpointer   data )
{
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

	print_message(*server,"Hello my name is %s.", "Test Client");

} /* message_callback() */

/************************************************************************/
/* reopen_callback()							*/
/*									*/
/************************************************************************/

void reopen_callback( GtkWidget *widget, gpointer   data )
{
	int	n;

	n = *((int *) data);

	close_server_socket(server[n]);
	
	if (!open_server(n))
	{
		create_message_window("Failed to Reopen server.");
	}

} /* repoen_callback() */

/************************************************************************/
/* delete_event()							*/
/*									*/
/* Callback function for program exit.					*/
/************************************************************************/

gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
	if (number_of_messages_displayed > 0) return TRUE; 
	gtk_main_quit();
	close_server_socket(server[0]);
	close_server_socket(server[1]);
	return(FALSE);

} /* delete_event() */

/************************************************************************/
/* background_code()							*/
/*									*/
/* Called periodically by GTK.						*/
/************************************************************************/

gint background_code(gpointer data)
{
	process_server_socket(server[0]);
	process_server_socket(server[1]);
	return TRUE;

} /* background_code() */

/************************************************************************/
/* signal_callback()                                                    */
/*                                                                      */
/* Sends a UI_SIGNAL message to server.					*/
/************************************************************************/

void signal_callback( GtkWidget *widget, gpointer   data )
{
        int     *server;

        server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

	send_ui_signal(*server, current_signal++);

} /* signal_callback() */
