/************************************************************************/
/* uiclient.c								*/
/*                                                                      */
/* Main routine for the central scrutinizer. 				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "local_uiclient.h"
#include "../../../cliserv/include/chara_messages.h"

int the_server = -1;

int main(int argc, char **argv)
{

	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("UICLIENT 0.0");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.mtwilson.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "UICLIENT 0.0";
	initialise_ui("uiclient_menus.ini", "uiclient_help.ini","uiclient");

	/* Setup background job(s) */

	setup_background_jobs();

	/* Setup user close function */

        set_user_close_function(uiclient_close_function);

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);

} /* main() */

/************************************************************************/
/* uiclient_close_function()						*/
/*									*/
/************************************************************************/

void uiclient_close_function(void)
{
	close_server_socket(the_server);

} /* uichara_close_function() */

/************************************************************************/
/* call_open_server_socket_by_name()					*/
/*									*/
/* Opens a socket so we can start sending server data to the outside    */
/* world.								*/
/************************************************************************/

int call_open_server_socket_by_name(int argc, char **argv)
{
        char    name[81];
	char	s[81];

	if (server_socket_open(the_server))
	{
		error(MESSAGE,"Server socket already open.");
		return NOERROR;
	}

	if (argc < 2)   /* No name given */
        {
                strcpy(s,"       server");
                if (quick_edit("Server name",s, s,NULL,STRING)==KEY_ESC) 
                {
                        return NOERROR;
                }
		sscanf(s,"%s",name);
        }
        else
        {
                sscanf(argv[1],"%s",name);
        }

	/* Try and open the socket */

	if ((the_server = open_server_by_name(name)) == -1)
	{
		return error(ERROR,"Failed to open server %s", name);
	}

	/* Now let's add our jobs to this server */

	if (!set_up_standard_jobs(the_server)) 
		return error(ERROR,"Failed to set up standard jobs.");
	
	/* That is all */

	return NOERROR;

} /* call_open_server_socket_by_name() */

/************************************************************************/
/* call_open_server_socket()						*/
/*									*/
/* Opens a socket so we can start sending server data to the outside    */
/* world.								*/
/************************************************************************/

int call_open_server_socket(int argc, char **argv)
{
        int     port;   /* The port */
        char    s[81];  /* Edit string */
        char    machine[81];

	if (server_socket_open(the_server))
	{
		error(MESSAGE,"Server socket already open.");
		return NOERROR;
	}

	if (argc < 2)   /* No name given */
        {
                strcpy(s,"       ctrscrut");
                if (quick_edit("Machine",s, s,NULL,STRING)==KEY_ESC) 
                {
                        return NOERROR;
                }
                sscanf(s,"%s",machine);
        }
        else
        {
                sscanf(argv[1],"%s",machine);
        }

        if (argc < 3)   /* No port given */
        {
                sprintf(s,"3000");
                if (quick_edit("Port",s, s,NULL,INTEGER)==KEY_ESC) 
                {
                        return NOERROR;
                }
                sscanf(s,"%d",&port);
        }
        else
        {
                sscanf(argv[2],"%d",&port);
        }

	sprintf(s,"%s:%d",machine, port);

	/* Try and open the socket */

	if ((the_server = open_server_socket(machine, port, s)) == -1)
	{
		return error(ERROR,"Failed to open server socket %d on %s",
			port,machine);
	}

	/* Now let's add our jobs to this server */

	if (!set_up_standard_jobs(the_server)) 
		return error(ERROR,"Failed to set up standard jobs.");
	
	/* That is all */

	return NOERROR;

} /* call_open_server_socket() */

/************************************************************************/
/* call_close_server_socket()						*/
/*									*/
/* CLoses server socket.						*/
/************************************************************************/

int call_close_server_socket(int argc, char **argv)
{
	close_server_socket(the_server);
	the_server = -1;

	return NOERROR;

} /* call_close_server_socket() */

/************************************************************************/
/* call_send_noop()							*/
/*									*/
/* Sends a NOOP message to the server.					*/
/************************************************************************/

int call_send_noop(int argc, char **argv)
{
	return send_noop(the_server);

} /* int call_send_noop() */

/************************************************************************/
/* call_send_ping()							*/
/*									*/
/* Sends a PING message to the server.					*/
/************************************************************************/

int call_send_ping(int argc, char **argv)
{
	return send_ping(the_server);

} /* call_int send_ping() */

/************************************************************************/
/* call_send_message_message()						*/
/*									*/
/* Sends a MESSAGE message to the server.				*/
/************************************************************************/

int call_send_message_message(int argc, char **argv)
{
	char	s[80];

	if (argc < 2)   /* No name given */
        {
                strcpy(s,"Testing testing one two three.");
                if (quick_edit("Message",s, s,NULL,STRING)==KEY_ESC) 
                {
                        return NOERROR;
                }
        }
        else
        {
		strcpy(s,argv[1]);
        }

	return send_message_message(the_server, s);

} /* int call_send_message_message() */

/************************************************************************/
/* call_send_all_noop()							*/
/*									*/
/* Sends a ALL_NOOP message to the server.				*/
/************************************************************************/

int call_send_all_noop(int argc, char **argv)
{
	return send_all_noop(the_server);

} /* int call_send_all_noop() */

/************************************************************************/
/* call_send_all_ping()							*/
/*									*/
/* Sends a ALL_PING message to the server.				*/
/************************************************************************/

int call_send_all_ping(int argc, char **argv)
{
	return send_all_ping(the_server);

} /* int send_all_ping() */
