/************************************************************************/
/* calc_histogram.c	                                                */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Feb 2004							*/
/************************************************************************/

#include <stdlib.h>

/************************************************************************/
/* calc_histogram()							*/
/*									*/
/* calculates an histogram from data[0..n-1].				*/
/* Returns a vector[0..num-1] containing histogram, memory allocate.	*/
/* If min and max are both zero it worls out the range autmatically.	*/
/* Returns NULL on error.						*/
/************************************************************************/

int	*calc_histogram(int n, float *data, float *min, float *max, int num)
{
	int	i,j;
	int	*hist;
	float	step;

	/* Work out the range */

	if (*min == 0.0 && *max == 0.0)
	{
		*min = 1e32;
		*max = -1e32;
		for(i=0; i<n; i++)
		{
			if (data[i] > *max)
				*max = data[i];
			else if (data[i] < *min)
				*min = data[i];
		}
	}

	/* Create the array */

	if ((hist = (int *)malloc(num * sizeof(int))) == NULL) return NULL;

	/* Go through the data */

	step = (*max-*min)/(float)num;
	for(j=0;j<num;j++) hist[j] = 0;
	for(i=0;i<n;i++)
	{
		for(j=1;j<=num;j++)
		{
			if (data[i] <= *min + (float)j*step) break;
		}
		hist[j-1]++;
	}

	/* That is all */

	return hist;

} /* calc_histogram() */
