/************************************************************************/
/* non_nrc_lsq_fit.c							*/
/*                                                                      */
/* Non linear least squares fit routines that DO NOT depend on NRC 	*/
/* fuctions. NOT WIRTTEN BY ME!!!!					*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : David 'P', Andrew Booth, Theo ten Brummelaar		*/
/* Date   : 31st July 1996						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <values.h>
#include "nrc.h"

/* Tese routines supplied by Andrew Booth */
/* David P's least squares multivariate fitting program in C */
/* calfunc is an integer function taking a pointer to a double as an
   argument.  This latter is an array to be filled with the calculated
   'y' values.  var is an array of pointers to where the variable
   coefficient values for use by this function are stored.
   Funct returns the number of variable coefficients used.
   NB. there will be trouble if one of these variables is zero.

							A.J.B.  */
/* 
 * Turnned into an ANSI rouinte and errors included by THEO.
 * residie calculations also added.
 */

double dcalcsd(int n,int nvar,double *y1,double *y2);
float calcsd(int n,int nvar,float *y1,float *y2);

void dlsmvf(int    nvar,
	    double *var[],
	    int    ndat,
	    double *ydat,
	    int    (*calfunc)(),
	    double *maxres,
	    double *rmsres,
	    double per_rms_err,
	    int	   max_nit,
	    int	   show)

{       
	int nit;
	int i, j, k;
	double *theory, sd1, sd2, dff, dfftst, *dth,
	       **diffarr, *dvar, **crlmat;
	double val, coef1, coef2, oldv;
	double  res;

	per_rms_err /= 100.0;

	theory = (double*)calloc(ndat,sizeof(double));
	dth = (double*)calloc(ndat,sizeof(double));
	diffarr = (double**)calloc(nvar+1,sizeof(double));
	dvar  = (double*)calloc(nvar+1,sizeof(double));
	crlmat = (double**)calloc(nvar+1,sizeof(double));
	for (i = 0; i < nvar+1; i++)
	{       *(diffarr+i) = (double*)calloc(ndat,sizeof(double));
		*(crlmat+i) = (double*)calloc(nvar+1,sizeof(double));
	}

	for(i=0;i<nvar;i++)
	{
		if (*var[i] == 0.0) *var[i] = 1e-20;
	}

	(*calfunc)(theory);
	sd1 = dcalcsd(ndat,nvar,theory,ydat);  sd2 = 0;
	if (show) 
	{
		printf("initial sd of fit from data = %G\n",sd1);
		fflush(stdout);
	}
	nit = 1;
	do
	{       if (show) printf("iteration No %d\n",nit++);
		for (j = 0; j < ndat; j++)      /* calc errors */
			*(*(diffarr+nvar)+j) = *(theory+j) - *(ydat+j);
		for (i = 0; i < nvar; i++)      /* find differantials */
		{       oldv = *var[i];
		    for (dfftst = 1.0, dff = 0.05; dfftst > 0.05; dff /= 10.0)
			{       *var[i] = (1.0+dff)*oldv; /* choose the */
				(*calfunc)(dth);          /* dx value to */
				dfftst = 0.0;             /* give dy < 5% */
				for (j = 0; j < ndat; j++)
				{       val = *(dth+j) - *(theory+j);
					*(*(diffarr+i)+j) = val/dff/oldv;
					val = fabs((double)val) / *(theory+j);
					if (dfftst < val) dfftst = val;
				}
			}
			*var[i] = oldv;
		}
		for (i = 0; i < nvar; i++)      /* find new coefficients */
		{       coef1 = 0;
			for (j = 0; j < ndat; j++)
			{       val = *(*(diffarr+i)+j);
				coef1 += val*val;
			}
			for (j = i+1; j < nvar+1; j++)
			{       coef2 = 0;
				for (k = 0; k < ndat; k++)
				 coef2 += *(*(diffarr+i)+k) * *(*(diffarr+j)+k);
				coef2 /= coef1;
				*(*(crlmat+i)+j) = coef2;
				for (k = 0; k < ndat; k++)
				 *(*(diffarr+j)+k) -= coef2 * *(*(diffarr+i)+k);
			}
		}
		for (i = 0; i < nvar+1; i++)
			*(dvar+i) = *(*(crlmat+i)+nvar);
		for (i = nvar-2; i >= 0; i--)
		{       coef1 = 0;
			for (j = i+1; j < nvar; j++)
				coef1 += *(*(crlmat+i)+j) * *(dvar+j);
			*(dvar+i) -= coef1;
		}
		for (i = 0; i < nvar; i++)      /* adjust coefficients */
			*var[i] -= *(dvar+i);

		(*calfunc)(theory);
		sd2 = sd1;  sd1 = dcalcsd(ndat,nvar,theory,ydat);
		/* print out coefficients & changes */
		if (show)
		{
			printf("coefficient  %%change\n");
			for (i = 0; i < nvar; i++)
				printf("%8.3g     %.2f\n",
					*var[i],*(dvar+i)*100/ *var[i]);
			printf("st dev of fit from data = %G\n",sd1);
			fflush(stdout);
		}
	} while(fabs(sd2-sd1) > per_rms_err*sd2 && nit <= max_nit);
	for (i = 0; i < nvar+1; i++)
	{       free(*(diffarr+i));
		free(*(crlmat+i));
	}

	/* Find the maximum and rms residue */

        *rmsres = *maxres = 0;
	(*calfunc)(theory);

        for(i=0;i<ndat;i++)
        {
                res = ydat[i] - theory[i];
                *rmsres += (res * res);
                if (fabs(res) > fabs(*maxres)) *maxres = res;
        }
        *rmsres = sqrt((double)*rmsres/(double)ndat);

	free(theory);  free(dth);  free(diffarr);  free(dvar);  free(crlmat);
}

double dcalcsd(int	n,
	       int	nvar,
	       double	*y1,
	       double	*y2)
{       
	int nn = n;
	double dy, sd = 0;

	while(nn-- > 0)
	{       dy = *y1++ - *y2++;
		sd += dy*dy;
	}
	return sqrt(sd/(n-nvar));
}


/*
 * Float versions
 */

void lsmvf(int    nvar,
	   float *var[],
	   int    ndat,
	   float *ydat,
	   int    (*calfunc)(),
	   float *maxres,
	   float *rmsres,
	   float per_rms_err,
	   int	   max_nit,
	   int	   show)

{       
	int nit;
	int i, j, k;
	float *theory, sd1, sd2, dff, dfftst, *dth,
	       **diffarr, *dvar, **crlmat;
	float val, coef1, coef2, oldv;
	float  res;

	per_rms_err /= 100.0;

	theory = (float*)calloc(ndat,sizeof(float));
	dth = (float*)calloc(ndat,sizeof(float));
	diffarr = (float**)calloc(nvar+1,sizeof(float));
	dvar  = (float*)calloc(nvar+1,sizeof(float));
	crlmat = (float**)calloc(nvar+1,sizeof(float));
	for (i = 0; i < nvar+1; i++)
	{       *(diffarr+i) = (float*)calloc(ndat,sizeof(float));
		*(crlmat+i) = (float*)calloc(nvar+1,sizeof(float));
	}

        for(i=0;i<nvar;i++)
        {
                if (*var[i] == 0.0) *var[i] = 1e-20;
        }

	(*calfunc)(theory);
	sd1 = calcsd(ndat,nvar,theory,ydat);  sd2 = 0;
	if (show) 
	{
		printf("initial sd of fit from data = %G\n",sd1);
		fflush(stdout);
	}
	nit = 1;
	do
	{       if (show) printf("iteration No %d\n",nit++);
		for (j = 0; j < ndat; j++)      /* calc errors */
			*(*(diffarr+nvar)+j) = *(theory+j) - *(ydat+j);
		for (i = 0; i < nvar; i++)      /* find differantials */
		{       oldv = *var[i];
		    for (dfftst = 1.0, dff = 0.05; dfftst > 0.05; dff /= 10.0)
			{       *var[i] = (1.0+dff)*oldv; /* choose the */
				(*calfunc)(dth);          /* dx value to */
				dfftst = 0.0;             /* give dy < 5% */
				for (j = 0; j < ndat; j++)
				{       val = *(dth+j) - *(theory+j);
					*(*(diffarr+i)+j) = val/dff/oldv;
					val = fabs((double)val) / *(theory+j);
					if (dfftst < val) dfftst = val;
				}
			}
			*var[i] = oldv;
		}
		for (i = 0; i < nvar; i++)      /* find new coefficients */
		{       coef1 = 0;
			for (j = 0; j < ndat; j++)
			{       val = *(*(diffarr+i)+j);
				coef1 += val*val;
			}
			for (j = i+1; j < nvar+1; j++)
			{       coef2 = 0;
				for (k = 0; k < ndat; k++)
				 coef2 += *(*(diffarr+i)+k) * *(*(diffarr+j)+k);
				coef2 /= coef1;
				*(*(crlmat+i)+j) = coef2;
				for (k = 0; k < ndat; k++)
				 *(*(diffarr+j)+k) -= coef2 * *(*(diffarr+i)+k);
			}
		}
		for (i = 0; i < nvar+1; i++)
			*(dvar+i) = *(*(crlmat+i)+nvar);
		for (i = nvar-2; i >= 0; i--)
		{       coef1 = 0;
			for (j = i+1; j < nvar; j++)
				coef1 += *(*(crlmat+i)+j) * *(dvar+j);
			*(dvar+i) -= coef1;
		}
		for (i = 0; i < nvar; i++)      /* adjust coefficients */
			*var[i] -= *(dvar+i);

		(*calfunc)(theory);
		sd2 = sd1;  sd1 = calcsd(ndat,nvar,theory,ydat);
		/* print out coefficients & changes */
		if (show)
		{
			printf("coefficient  %%change\n");
			for (i = 0; i < nvar; i++)
				printf("%8.3g     %.2f\n",
					*var[i],*(dvar+i)*100/ *var[i]);
			printf("st dev of fit from data = %G\n",sd1);
		}
	} while(fabs((double)(sd2-sd1)) > per_rms_err*sd2 && nit <= max_nit);
	for (i = 0; i < nvar+1; i++)
	{       free(*(diffarr+i));
		free(*(crlmat+i));
	}

	/* Find the maximum and rms residue */

        *rmsres = *maxres = 0;
	(*calfunc)(theory);

        for(i=0;i<ndat;i++)
        {
                res = ydat[i] - theory[i];
                *rmsres += (res * res);
                if (fabs((double)res) > fabs((double)*maxres)) *maxres = res;
        }
        *rmsres = sqrt((double)*rmsres/(float)ndat);

	free(theory);  free(dth);  free(diffarr);  free(dvar);  free(crlmat);
}

float calcsd(int	n,
	     int	nvar,
	     float	*y1,
	     float	*y2)
{       
	int nn = n;
	float dy, sd = 0;

	while(nn-- > 0)
	{       dy = *y1++ - *y2++;
		sd += dy*dy;
	}
	return sqrt((double)sd/(n-nvar));
}
