/************************************************************************/
/* testfits.c								*/
/*									*/
/* Tests writing a FITs file.  						*/
/*									*/
/* Usage: testfits file							*/ 
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1997 							*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"
#ifdef DISPLAY
#include "../simapleX/simpleX.h"
#endif

int main(int argc, char **argv)
{
	/* FITs file data */

	fits_data data;
	float	mult;

	/* Ensure command line is OK */

	if (argc < 2)
	{
		fprintf(stderr,"usage: %s file <mult>\n",argv[0]);
		exit(-1);
	}
	if (argc > 2)
	{
		sscanf(argv[2],"%f",&mult);
	}
	else
	{
		mult = 1.0;
	}

	printf("TESTFITS : %s\n\n",argv[1]);

        /* Set up defaults */

        data.nheader = 0;
        data.header = (char **)malloc(MAXHEADER+1)-1;
        mem_test(data.header);
        data.nx = 4;
        data.ny = 2;
        data.data = matrix(1,4,1,2);

	data.data[1][1] = mult * 1;
	data.data[1][2] = mult * 2;
	data.data[2][1] = mult * 3;
	data.data[2][2] = mult * 4;
	data.data[3][1] = mult * 5;
	data.data[3][2] = mult * 6;
	data.data[4][1] = mult * 7;
	data.data[4][2] = mult * 8;
	
	print_fits_header(data);
	write_fits(data, argv[1]);
	free_fits_data(data);
	exit(0);
}
