#!/usr/bin/python

from matplotlib.mlab import bivariate_normal
from matplotlib.mlab import meshgrid
from pylab import *
import mpfit2 as mpfit
from numarray import *
from numarray import linear_algebra
from random import randrange

def model(p,fjac=None,x=None,y=None,err=None):
	
	base,amp,sigmax,sigmay,mux,muy = p

	model = base + amp * exp(-0.5*(((x[:,0]-mux)**2)/(sigmax**2) + ((x[:,1]-muy)**2)/(sigmay**2))) 
	status = 0
	return([status, (y-model)/err])
	
def gauss_fit(intensity=0,x_pos=0,y_pos=0):

	x = zeros(len(intensity.flat))
	y = zeros(len(intensity.flat))
	z = intensity.flat
	
	for i in range(len(y_pos)):
		for j in range(len(x_pos)):
			x[i*len(x_pos)+j] = x_pos[j]
			y[i*len(x_pos)+j] = y_pos[i]
			#z[i*len(x_pos)+j] = intensity[i][j]	

	#p contains the guesses
    	
    	# gets the index of the maximum of z (1d array)
    	iMax=argmax(z)
    	# gets the x coordinate of the maximum
    	mux1=x[iMax]
    	# gets the y coordinate of the maximum
    	muy1=y[iMax]
    	#guess sigma is ~5 microns
	sigmax1 = 5.  #steps per micron
	sigmay1 = 5.
	#guess amp is highes value
	amp1 = z.max()
	#guess that the base is the lowest value
	base1 = z.min()

	#actual values used in the function
	#p1 = array([base0,amp0,sigmax0,sigmay0,mux0,muy0])

	#guess values 
	guess = [base1,amp1-base1,sigmax1,sigmay1,mux1,muy1]
 
	err = 1.0
	
	parinfo=[]
	for i in range(6): 
		parinfo.append({'value':guess[i],'fixed':False,'limited':[0,0],'limits':[0.,0.],'mpprint':1})
	
    	xy=transpose(reshape(concatenate((x,y)),(2,shape(x)[0])))
    	
	fa={'x':xy, 'y':z, 'err':err}
    	
	print 'guesses: ', guess
##         eps = 1.e-6

	m=mpfit.mpfit(model,guess,parinfo=parinfo,functkw=fa,quiet=0)
	
    	print 'fit mparams: ', m.params	
	
	base,amp,sigmax,sigmay,mux,muy = m.params
		
	#for the fit use more data points, using delta
	delta = (x_pos[1]-x_pos[0])/5.
	x_fit = arange(x_pos.min(),x_pos.max()+delta,delta)
	y_fit = arange(y_pos.min(),y_pos.max()+delta,delta)
	x_fit_mesh,y_fit_mesh = meshgrid(x_fit,y_fit) 
	intensity_fit = base + amp*exp(-0.5*(((x_fit_mesh-mux)**2)/(sigmax**2) + ((y_fit_mesh-muy)**2)/(sigmay**2)))
	
	corner_delta = (x_pos[1]-x_pos[0])*0.5
	
	return intensity_fit,m.params
