/*
 * spooler.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define _GNU_SOURCE

#include <sys/time.h>
#include <sys/types.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <math.h>
#include <rtai/pipe.h>

#include "common.h"
#include "spooler.h"
#include <fitsio.h>
#include "cliserv.h"
#include "chara_messages.h"

/* Common variables */
typedef struct s_star star;
typedef struct sshutter_conf shutter;
typedef struct s_uv suv;
typedef struct s_time_now timenow;
typedef struct s_ut_sid ut_sid;
typedef struct s_process_sockets processSocket_struct;

boolean end = FALSE;
int fd, sockfd[NUM_THREADS];               /* File descriptors for pipe and sockets */
pthread_attr_t attr;
pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
int threadNum;

/* sem_t sem_fits; */

/*****************************************************/
/* signal-handler, to ensure clean exit on Ctrl-C    */
/*****************************************************/
void cleanExit(int dummy)
{
  end = TRUE;
  usleep(1000000);
  exit(0);
}/*  end cleanExit */


/*****************************************************/
/* Print out cfitsio error messages and exit program */
/*****************************************************/
void printerror( int status)
{
    if (status)
    {
       fits_report_error(stderr, status); /* print error report */
       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

void error(char *msg)
{
    perror(msg);
    exit(0);
}

/*****************************************************/
 /* Remove leading char from string                   */
/*****************************************************/
char *rmlead(char *str)
{
      char *obuf;

      for (obuf = str; obuf && *obuf && isspace(*obuf); ++obuf)
            ;
      if (str != obuf)
            strcpy(str, obuf);
      return str;
} /* end rmlead */

/*****************************************************/
/* format string for headers from charalibs          */
/*****************************************************/
char *fmtString(char *buffer)
{
  int idx;
  rmlead(buffer);
  if (*buffer == 45) {
    idx = 3;
      } else {
    idx = 2;
  }
  *(buffer+idx) = (char)58;
  *(buffer+idx+3) = (char)58;
  return(buffer);
} /* end fmtString */


/*****************************************************/
/* Open a server for querying status                 */
/*****************************************************/
void openServer(void *hostData)
{
  FILE *tmpFile;                   /* Temporary file to ckeck open pipe */
  int sockfd, portno, n, i;
  struct sockaddr_in serv_addr;
  struct hostent *server;
  int devNum=0;                    /* Open pipe number */
  char devName[50];                /* Pipe's device name */
  threadData *host;

/* Dereference data to right type */
  host = (threadData *) hostData;

  if(host->socknum < (threadNum - 1)){
    //init_jobs();
    host->fd = (int) open_server_socket(host->hostname,host->port,host->hostname);
    if (host->fd >= 0){
      printf("Open %s port %d\n",host->hostname, host->port);
    } else {
      printf("error opening %s\n",host->hostname);
    }
  } else { 
    devNum=0;
    do{
      /* Check existence of pipe */
      sprintf(devName,"/proc/rtai/registry/pipes/pipe%d", devNum);
      tmpFile = fopen(devName, "r");
      if(tmpFile == NULL) {
	sprintf(devName,"/proc/rtai/registry/pipes/pipe%d", devNum-1);
	printf("Opening %s\n",devName);	      
	break;
      }
      fclose(tmpFile);
      printf("Skipping %s\n",devName);
      devNum ++;
    } while(devNum < 23);
  
    /* Open pipe */
    host->fd = open(devName, O_RDONLY);
  }
} /* end openServer */

/*****************************************************/
/* Query server for status                           */
/*****************************************************/
int queryServer(int sockfd, void *serverarg)
{
  message_struct *msg;
  timenow *timen;

  msg = (message_struct *) serverarg;
  //printf("%d:Sending  message number == %d\n",sockfd,msg->type);
  if (!send_message(sockfd,msg)) {
    printf("%d : ERROR writing to socket \n",sockfd);
    return -1;
  } else {
    return 0;
  }
} /* end queryServer */

/*****************************************************/
/* Read server for status                           */
/*****************************************************/
int readServer(int sockfd, void *serverarg)
{
  unsigned short messtype;
  unsigned int messlength;
  message_struct *mess;
  
  mess = (message_struct *) serverarg;

  while (messtype != mess->type){
    
    // printf("%d",client_read_socket(servers[sockfd].fd,(char *) &messtype,2));
    //fflush(stdout);

    while (client_read_socket(servers[sockfd].fd,(char *) &messtype,2) == 0){
      usleep(10000); //jdm increased from 1000 to 10000 musec
    };
    //printf("%d: message type = %d\n",sockfd,messtype);
    //fflush(stdout);
    while (client_read_socket(servers[sockfd].fd,(char *) &messlength,4) == 0){
      usleep(10000); //jdm increased from 1000 to 10000 musec
    };
       
    //printf("%d: message length = %d\n",sockfd,messlength);
    //fflush(stdout);
    if (messlength == 0) {
      printf("%d: message type = %d: Message length zero.. (server probably down)\n",sockfd,messtype);
      return -1;
    }
    while (client_read_socket(servers[sockfd].fd,(char *) mess->data, messlength) == 0){
      usleep(10000); //jdm increased from 1000 to 10000 musec
    }
    //printf("%d: got the data for %d %d \n",sockfd ,messlength, messtype);
  }
  //printf("%d: Got right message %d *************\n",sockfd, messtype);

  return 0;
} /* end readServer */


/*****************************************************/
/* Tread reads socket and writes to a buffer         */
/*****************************************************/
void *socketThread(void *threadarg)
{
  int  threadId,tstatus;
  threadData *tdata;
  tdata = (threadData *) threadarg;
  size_t mystacksize;
  message_struct *mess;

  pthread_attr_getstacksize (&attr, &mystacksize);

  threadId = tdata->socknum;
  mess = (message_struct *) tdata->messageBack;

  //printf("Starting %s Thread id: %d \n",tdata->hostname,threadId);

  while (!end) {
    
    // printf("B Starting %s Thread id: %d \n",tdata->hostname,threadId);
    if (tdata->fd >= 0) {

    // printf("C Starting %s Thread id: %d \n",tdata->hostname,threadId);
      /* Query server for info*/
      //JDM      
      pthread_mutex_lock( &mutex1 );
      if (0 == queryServer(tdata->fd, tdata->message)) {
	//printf("%d:%s Ask server for data...\n", threadId,tdata->hostname);
	tstatus = readServer(tdata->fd, tdata->messageBack);
        //printf("Status of %s Thread id: %d : STATUS %d\n",tdata->hostname,threadId, tstatus);
      }
      //JDM 
      pthread_mutex_unlock( &mutex1 );
      usleep(600000);  //jdm increased from 200000 to 600000 musec
    } else {
 
      bzero(mess->data, mess->length);
    }
  }
   pthread_exit((void *) 0);
} /* end socketThread */

/*****************************************************/
/* Tread reads pipe and writes to disk               */
/*****************************************************/
void *spoolThread(void *threadarg)
{
  struct smessage message;      /* Messages passed from astropci_usr.c */
  header_struct header;         /* Structure passed from astropci_usr.c */
  FILE *tmpFile;                /* Temporary file to ckeck existence of fits files */
  int pixelCount;               /* Length of message from astropci_usr.c  */
  boolean fileOpen = FALSE;     /* Is fits file open? */
  int length = 0;               /* Total message space to malloc */
  double start1, start2;        /* Vars for timing thread execution */  
  clock_t cstart1, cstart2;        /* Vars for timing thread execution */


  int fileNum = 0;              /* last open fits file */
  char fileName[50];            /* Fits filename */
  char gdate[10];               /* System date */
  char fitsdate[19];            /* System UT date and time */  
  char dir1[50], dir2[50];      /* Dirs names to create file structure */
  char target[12];              /* Target Name */
  time_t rawTime;               /* Structures used to get System time and */
  struct tm *gmt;               /* UT time */
  double *delay;                /* Delay lines position  */
  star *theStar;                /* Star info structure */
  suv *uv;                      /* uv info structure */
  int loop;                     /* loop throug uv */
  char keyword[30];
  int result;
  float *counts;

/* Fits definitions in Header  */
/* Copied from cfitsio example */
  long naxis=NAXIS_LEN, naxes[NAXIS_LEN], fpixel = 1, firstpix[NAXIS_LEN] = {1,1,1};
  fitsfile *fptr;               /* FITS file pointer, defined in fitsio.h */
  int status = 0;               /* CFITSIO status value MUST be initialized to zero!  */
  pythonconfig_struct *pyconfig;
  int  threadId;
  threadData *tdata;
  size_t mystacksize;
  char complete[18];
  threadData *thread_data_array;
  message_struct *mess;
  ut_sid *theUtSid;
  char tmpBuf[12];
  

  tdata = (threadData *) threadarg;
  
  pthread_attr_getstacksize (&attr, &mystacksize);

  threadId = tdata->socknum;

  thread_data_array = (threadData *) tdata->ptr; 

  mess = (message_struct *) thread_data_array[STARFD].messageBack;
  theStar = (star *) mess->data;

  mess = (message_struct *) thread_data_array[UVFD].messageBack;
  uv = (suv *) mess->data;

  mess = (message_struct *) thread_data_array[GPSFD].messageBack;
  theUtSid = (ut_sid *) mess->data;

  mess = (message_struct *) thread_data_array[TIPTILTFD].messageBack;
  counts = (float *) mess->data;
 
  //pyconfig = (pythonconfig_struct *) header.pyconfig;
 
  while (!end) {

    //pthread_mutex_lock( &mutex1x1 );
    result = read(tdata->fd,& message.type, sizeof(message.type));
    //pthread_mutex_unlock( &mutex1 );

    if (result  == -1) {
       usleep(100); // JDM added to decrease burden on cpu.
    } else {
      
      switch (message.type) {
	
      case HEADER:

	firstpix[2]=1;

	start1 =(double) clock();
        //usleep(250000);
        //cstart2 = clock();
	//printf("%d:Time elapsed: %d %d %5.2f %5.2f \n", threadId,cstart1,cstart2, ((double)(cstart2 - cstart1)) , CLOCKS_PER_SEC);
	
	//if ( ( (start1 - start2) / CLOCKS_PER_SEC)  > 6.0) {
        //        printf("%d:Time elapsed: %f \n", threadId, (start1 - start2) / CLOCKS_PER_SEC);
        //  }
	// printf("%f: ",(start1 - start2) / CLOCKS_PER_SEC);
        // fflush(stdout);
	start2 = start1;



	//pthread_mutex_lock( &mutex1 );
	if (read(tdata->fd,&message.length, sizeof(message.length)) == -1)
	  printf("%d:Header length read error\n", threadId);
	
	if (read(tdata->fd, (unsigned char *) &header, message.length) == -1)
	  printf("%dHeader read error\n", threadId);

	//pthread_mutex_unlock( &mutex1 );

	pixelCount = header.rows * header.cols * header.nreads;
	 
	length = pixelCount * sizeof(unsigned short);
	if((message.data = (unsigned char *) malloc(length)) == NULL) {
	  printf("%d:Data malloc error\n", threadId);
	  exit -1;
	}

	if(!fileOpen && header.do_image_save){

	  /* Get UT time */
	  time ( &rawTime );
	  gmt = gmtime ( &rawTime );
    
	  sprintf (fitsdate, "%04d-%02d-%02dT%02d:%02d:%02d", gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday, gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

	  sprintf (gdate, "%02d/%02d/%04d", gmt->tm_mon + 1, gmt->tm_mday, gmt->tm_year + 1900);
	  sprintf(dir1,"%s%02d%s/", header.image_directory, gmt->tm_year + 1900, mon[gmt->tm_mon]);
	  sprintf(dir2,"%s%02d%s%02d", dir1, gmt->tm_year + 1900, mon[gmt->tm_mon], gmt->tm_mday);

	  if(chdir(dir1)){
	    fileNum = 0;
	    printf("%d:Create directory %s\n", threadId, dir1);	  
	    /* Open dir with directory name */
	    mkdir(dir1,0755);
	  }
	  if(chdir(dir2)){
	    fileNum = 0;
	    printf("%d:Create directory %s\n", threadId, dir2);	  
	    /* Open dir with directory name */
	    mkdir(dir2,0755);
	  }
	
	  /* -----------------------------------
	     Create the FITS file.
	     ----------------------------------- */

	  do{
	    /* Check existence of  fits file */
	    sprintf(fileName,"%s/mirc%04d.fits", dir2, fileNum);
	    tmpFile = fopen(fileName, "r");
	    if(tmpFile == NULL) {
	      fileNum ++;
	      break;
	    }
	    fclose(tmpFile);
	    printf("%d:Skipping %s\n", threadId,fileName);
	    fileNum ++;
	  } while(fileNum < 9999);
	  
	  status=0;

	  //printf("%d:Create fits file %s... ", threadId,fileName);
          printf("mirc%04d.fits: ",fileNum-1);
          fflush(stdout);
      	  fits_create_file(&fptr,fileName,&status);

	  naxes[1] = header.rows;
	  naxes[0] = header.cols * header.nreads;
	  naxes[2] = header.numberOfFrames;	  

	  if ( fits_create_img(fptr,  USHORT_IMG, naxis, naxes, &status) )
	    printerror( status );

	  /* Write keywords; must pass the ADDRESS of the value */
	  fits_update_key(fptr, TSTRING, "DATE", &fitsdate, "File creation date (YYYY-MM-DDThh:mm:ss UTC)", &status);
	  fits_update_key(fptr, TSTRING, "TELESCOP", "CHARA array 330m max baseline, 6dishes", "Telescope", &status);
	  fits_update_key(fptr, TSTRING, "INSTURME", "MIRC spectro/combiner", "The data acquisition instrument", &status);
	  fits_update_key(fptr, TSTRING, "ORIGIN", "Mount Wilson Institute", "Origin of the Observation", &status);
	  fits_update_key(fptr, TSTRING, "SITELAT","34.13.54", "Latitude (Geodetic, VLBI, to be verified)", &status);
	  fits_update_key(fptr, TSTRING, "SITELONG","118.03.33", "Longitude (Geodetic, VLBI, to be verified)", &status);
	  fits_update_key(fptr, TSTRING, "SITEELEV","5680.00", "Altitude above MSL, to be verified", &status);
	  fits_update_key(fptr, TSTRING, "HISTORY", "Multi-Dish FITS data", "File modification history", &status);

/* 	  if (!strcmp(complete,"COMMAND_COMPLETE:0")) fits_update_key(fptr, TSTRING, "OBJECT", target, "Target name", &status); */
	  fits_update_key(fptr, TSTRING, "OBSERVER", header.pyconfig.observer, "Observer name", &status);
	  fits_update_key(fptr, TSTRING, "DATE-OBS", gdate, "UT date (YYYY-MM-DD)", &status);

	  //pthread_mutex_lock( &mutex1 );
	  fits_update_key(fptr, TSTRING, "UTC-OBS", fmtString((char *) time_to_string(theUtSid->ut, tmpBuf)), "Universal Time hh:mm:ss.ddd", &status);
	  fits_update_key(fptr, TSTRING, "LST-OBS", fmtString((char *) time_to_string(theUtSid->sid, tmpBuf)), "Local Sidereal Time hh:mm:ss.ddd", &status);
	 	 
	  fits_update_key(fptr, TSTRING, "OBJECT", theStar->name, "Target name", &status);
	  printf("HA: %s   Star: %s " , fmtString((char *) time_to_string(uv->ha, tmpBuf)),theStar->name);
	  fits_update_key(fptr, TSTRING, "IRC", theStar->irc, "Target IRC no..", &status);
	  fits_update_key(fptr, TSTRING, "HR", theStar->hr, "Target HR no..", &status);
	  fits_update_key(fptr, TSTRING, "HD", theStar->hd, "Target HD no..", &status);
	  fits_update_key(fptr, TSTRING, "SAO", theStar->sao, "Target sao no..", &status);
	  fits_update_key(fptr, TSTRING, "RA", fmtString((char *) time_to_string(theStar->ra, tmpBuf)), "Right ascension hh:mm:ss.ddd", &status);
	  fits_update_key(fptr, TSTRING, "DEC", fmtString((char *) angle_to_string(theStar->ra, tmpBuf)), "Declination dd:mm:ss.dd", &status);
	  fits_update_key(fptr, TDOUBLE, "PM_RA", &theStar->pm_ra, "RA proper motion (rad/year)", &status);
	  fits_update_key(fptr, TDOUBLE, "PM_DEC", &theStar->pm_dec, "Dec proper motion (rad/yr)", &status);
	  fits_update_key(fptr, TDOUBLE, "PARALLAX", &theStar->parallax, "Parallax (arcseconds)", &status);
	  fits_update_key(fptr, TDOUBLE, "RADVEL", &theStar->radvel, "Radial velocity (km/s)", &status);
	  fits_update_key(fptr, TDOUBLE, "VMAG", &theStar->vmag, "V magnitude", &status);
	  fits_update_key(fptr, TDOUBLE, "KMAG", &theStar->kmag, "K magnitude", &status);
	  fits_update_key(fptr, TSTRING, "SPECTYPE", theStar->spectype, "Spectral type", &status);

	  fits_update_key(fptr, TFLOAT, "WAVELEN", &header.pyconfig.lambda, "Central wavelength", &status);
	  fits_update_key(fptr, TFLOAT, "BANDWID", &header.pyconfig.bandwidth, "Bandwidth of spectrum", &status);
	  
	  fits_update_key(fptr, TSTRING, "AZ", fmtString((char *) angle_to_string(uv->az, tmpBuf)), "Azimuth dd:mm:ss.dd", &status);
	  fits_update_key(fptr, TSTRING, "EL", fmtString((char *) angle_to_string(uv->el, tmpBuf)), "Elevation dd:mm:ss.dd", &status);
	  fits_update_key(fptr, TSTRING, "HA", fmtString((char *) time_to_string(uv->ha, tmpBuf)), "Hour Angle hh:mm:ss.ddd", &status);
  
	  for(loop=0;loop < (NUM_SCOPES*(NUM_SCOPES-1))/2; loop++){
	    sprintf(keyword,"U_%s", &uv->uv_names[loop][0]);
	    fits_update_key(fptr, TDOUBLE, keyword,  &uv->uv[loop][U], "u coord. (m)", &status);
	    sprintf(keyword,"V_%s", &uv->uv_names[loop][0]);
	    fits_update_key(fptr, TDOUBLE, keyword,  &uv->uv[loop][V], "v coord. (m)", &status);
	  }

	  //pthread_mutex_unlock( &mutex1 );

	  fits_update_key(fptr, TINT, "N_TELS", &header.pyconfig.NTels, "Number of used telescopes", &status);

	  for(loop=0;loop < header.pyconfig.NTels; loop++){
	    sprintf(keyword,"BEAMORD%i", loop);
	    fits_update_key(fptr, TINT, keyword, &header.pyconfig.BeamOrder[loop], "Beam order", &status);
	  }

	  fits_update_key(fptr, TINT, "REF_TEL", &header.pyconfig.RefTel, "Reference Telescope", &status);
	  fits_update_key(fptr, TSTRING, "TEL_KEY", header.pyconfig.TelKey, "Telescope/beam correspondence", &status);

	  for(loop=0;loop < header.pyconfig.NTels; loop++){
	    sprintf(keyword,"F_EXPLX%i", loop);
	    fits_update_key(fptr, TINT, keyword, header.pyconfig.FiberExplorerX, "Fiber explorer X coord.", &status);
	    sprintf(keyword,"F_EXPLY%i", loop);
	    fits_update_key(fptr, TINT, keyword, header.pyconfig.FiberExplorerY, "Fiber explorer Y coord.", &status);
	  }


	  for(loop=0;loop < (header.pyconfig.NTels*(header.pyconfig.NTels-1)/2); loop++){
	    sprintf(keyword,"GDT_TRS%d", loop);
	    fits_update_key(fptr, TFLOAT, keyword,  &header.pyconfig.gdtThreshold[loop], "GDT Threshold", &status);
	  }
	  fits_update_key(fptr, TFLOAT, "GDT_GAIN", &header.pyconfig.gdtGain, "GDT Gain", &status);

	  for(loop=0;loop < NUMDLS; loop++){
	    sprintf(keyword,"DLOFFST%d", loop);
	    fits_update_key(fptr, TDOUBLE, keyword,  &header.gdt.dlOffsets[loop], "Delay line offsets", &status);
	  }

	  for(loop=0;loop < NUMDLS; loop++){
	    sprintf(keyword,"MANOFFS%d", loop);
	    fits_update_key(fptr, TDOUBLE, keyword,  &header.gdt.manualOffset[loop], "Delay line manual offsets", &status);
	  }

	  for(loop=0;loop < BASELINES; loop++){
	    sprintf(keyword,"ACTBAS%d", loop);
	    fits_update_key(fptr, TBYTE, keyword,  (unsigned char *) &header.gdt.activeBaselines[loop], "Active baselines", &status);
	  }

	  fits_update_key(fptr, TBYTE, "GDT_LOCK",  (unsigned char *) &header.gdt.gdtLock, "Group delay track status", &status);

	  for(loop=0;loop < (header.pyconfig.NTels*(header.pyconfig.NTels-1)/2); loop++){
	    sprintf(keyword,"CPEAK_%d", loop);
	    fits_update_key(fptr, TBYTE, keyword,  &header.pyconfig.cpeak[loop], "Fourier peaks position for baselines", &status);
	    sprintf(keyword,"CPEAK0_%d", loop);
	    fits_update_key(fptr, TBYTE, keyword,  &header.pyconfig.cpeak0[loop], "Fourier peaks position for baselines", &status);
	    sprintf(keyword,"CPEAK1_%d", loop);
	    fits_update_key(fptr, TBYTE, keyword,  &header.pyconfig.cpeak1[loop], "Fourier peaks position for baselines", &status);
	  }
	  fits_update_key(fptr, TBYTE, "BKGNDNOI", &header.pyconfig.bkgndNoiseCol, "Background noise column position", &status);

	  for(loop=0;loop < NUM_SCOPES; loop++){
	    sprintf(keyword,"TTCOUNT%d", loop);
	    fits_update_key(fptr, TFLOAT, keyword,  &counts[loop], "Tiptilt counts", &status);
	  }

	  fits_update_key(fptr, TSTRING, "FILETYPE", header.pyconfig.fileDescription, "File description", &status);
	  fits_update_key(fptr, TSTRING, "FILTER1", header.pyconfig.MircFilter1, "Mirc filter 1", &status);
	  fits_update_key(fptr, TSTRING, "FILTER2", header.pyconfig.MircFilter2, "Mirc filter 2", &status);
	  fits_update_key(fptr, TSTRING, "CONF_NA", header.pyconfig.ConfigName, "Conf name", &status);
	  fits_update_key(fptr, TSTRING, "CONF_NA", header.pyconfig.ConfigName, "Conf name", &status);

	  fits_update_key(fptr, TFLOAT, "EXPOSURE", &header.tint, "Effective integration time in ms", &status);
	  fits_update_key(fptr, TSHORT, "ROWOFFS", &header.rowOffset, "Sub-image Y offset prom pixel 0", &status);
	  fits_update_key(fptr, TSHORT, "COLOFFS", &header.colOffset, "Sub-image X offset prom pixel 0", &status);
	  fits_update_key(fptr, TSHORT, "NREADS", &header.nreads, "Number of multiple reads for pixel", &status);
	  fits_update_key(fptr, TSHORT, "FRMPRST", &header.framesPerReset, "Number of frames per reset", &status);
	  fits_update_key(fptr, TFLOAT, "VOFFSET", &header.voffset, "PICNIC offset voltage", &status);
 	  fits_update_key(fptr, TFLOAT, "VD", &header.vd, "PICNIC drain bias", &status);
 	  fits_update_key(fptr, TFLOAT, "ICTL", &header.ictl, "PICNIC warm OA offset voltage", &status);
	  fits_update_key(fptr, TSTRING, "COMMENT1", header.pyconfig.comment1, "Any additional information", &status);	 
	  fits_update_key(fptr, TSTRING, "COMMENT2", header.pyconfig.comment2, "Any additional information", &status);	 
	  fileOpen = TRUE;
	  fflush(stdout);
	}

	break;

      case DATA:

	if (read(tdata->fd,&message.length, sizeof(message.length)) == -1) 
	  printf("%d:Data length read error\n", threadId);
	
	// allocate data buffer
	if (length == message.length) 
	  if((message.data = (unsigned char *) realloc(message.data, message.length)) == NULL)
	    printf("%d:Data realloc error\n", threadId);
	
	length = message.length;  // should not assign length here!! 
        if (length =! ( sizeof(float)*pixelCount) )  {
	  printf("Wrong DATA length: %d in buffer instead of %d\n",length,( sizeof(float)*pixelCount) );
	}
	if (read(tdata->fd, (unsigned char *) message.data, message.length) == -1) {
	  printf("%d:Data read error\n", threadId); 
	
	} else if(fileOpen) {
           
	  /* Write the image data. */
	  //pthread_mutex_lock( &mutex1 );
	  if (fits_write_pix(fptr, TUSHORT, firstpix, naxes[0]*naxes[1],
			     message.data, &status))
	    printerror( status ); 
	  //pthread_mutex_unlock( &mutex1 );
	}
	/* loop over all planes of the cube (2D images have 1 plane) */
	firstpix[2] ++;
	break;

      case END:
	free(message.data);

	if(fileOpen) {

	  fits_close_file(fptr,&status);
	  //printf(" close!\n");
          printf(" at UT %s %s\n",fmtString((char *) time_to_string(theUtSid->ut, tmpBuf)),header.pyconfig.fileDescription);
	  fileOpen = FALSE;
	}

	break;

      case KILL:
	printf("%d:Kill spooler process\n", threadId);
	end = TRUE;	break;
            
      default:
	  
	break;
      }
    }
  }

  pthread_exit((void *) 0);
} /* end spoolThread */

int main (int argc, char *argv[])
{
  unsigned long mask = 1;          /* Set affinity to processors 1 */
  int devNum=0;                    /* Open pipe number */
  char devName[50];                /* Pipe's device name */
  int i;                           /* used in for closing sockets fds */
  star theStar;
  shutter theShutter;
  suv uv;                          /* uv structure returnded by OPLE for 
                                       ASTROMOD_UV_DATA */
  int sockmanfd;
  ut_sid theUtSid;
  processSocket_struct processSocket;

/* Time structure to send OPLE together with ASTROMOD_UV_DATA */
  timenow timen = {
    0,
    0,
    0,
    0.0,
    M_PIl/(.18*3.6)
  };


 

/* Message to request OPLE uv plane */
  message_struct uvMessage = {
    ASTROMOD_UV_DATA,
    sizeof(timen),
    (char *) &timen
  };

  message_struct uvMessageBack = {
    ASTROMOD_UV_DATA,
    sizeof(uv),
    (char *) malloc(sizeof(uv))
  };
 
 
/* Message to request OPLE star info */
  message_struct starMessage = {
    ASTROMOD_STAR_DATA,
    0,
    NULL
  };

  message_struct starMessageBack = {
    ASTROMOD_STAR_DATA,
    sizeof(theStar),
    (char *) malloc(sizeof(theStar))
  };

/* Message to request OPLE targets */
  message_struct gpsMessage = {
    GPS_UT_SID,
    0,
    NULL
  };

  message_struct gpsMessageBack = {
    GPS_UT_SID,
    sizeof(theUtSid),
    (char *) malloc(sizeof(theUtSid))
  };
/* Message to request TIPTILT counts */
  message_struct tiptiltMessage = {
    TIPTILT_COUNTS,
    0,
    NULL
  };

  message_struct tiptiltMessageBack = {
    TIPTILT_COUNTS,
    sizeof(float) * NUM_SCOPES,
    (char *) malloc(sizeof(float) * NUM_SCOPES)
  };

  message_struct shutterMessage = {
    STATUS_SHUTTER,
    0,
    NULL
  };

  message_struct weatherMessage = {
    WEATHER_NOW,
    0,
    NULL
  };
 
  threadData testThread;
/* Init threads and socket params */
  threadData thread_data_array[] = {
    {
      STARFD,
      OPLE,
      PORT,
      -1,
      &socketThread,
      (void *) &starMessage,
      (void *) &starMessageBack,
      (void *) ' '
    },
    {
      UVFD,
      OPLE,
      PORT,
      -1,
      &socketThread,
      (void *) &uvMessage,
      (void *) &uvMessageBack,
      (void *) ' '
    },
    {
      GPSFD,
      GPS,
      PORT,
      -1,
      &socketThread,
      (void *) &gpsMessage,
      (void *) &gpsMessageBack,
      (void *) ' ',
    },
    {
      TIPTILTFD,
      TIPTILT,
      PORT,
      -1,
      &socketThread,
      (void *) &tiptiltMessage,
      (void *) &tiptiltMessageBack,     
      (void *) ' ',
    },
/*     { */
/*       0, */
/*       SHUTTER, */
/*       4000, */
/*       -1, */
/*       &socketThread, */
/*       (void *) &shutterMessage, */
/*       (void *) ' ', */
/*     }, */
    {
      SPOOLFD,
      SPOOLER,
      PORT,
      -1,
      &spoolThread,
      STATUSMSG,
      STATUSMSG,
      (void *) ' '
    }
  };


/* Message to request sockman message */
  message_struct sockMessage = {
    SOCKET_MANAGER_SET_PROCESS,
    sizeof(processSocket),
    (char *) &processSocket
  };


/* Define thread control vars */
  pthread_t threads[NUM_THREADS];
  int rc, t, status;
  size_t stacksize;

  init_jobs();
  usleep(1000000);
  threadNum = (sizeof(thread_data_array)/sizeof(testThread));

/* Set affinity to CPU 1 (0 for current process) */
  if (sched_setaffinity(0, sizeof(mask), (void *) &mask) < 0) { 
    perror("sched_setaffinity");
  }
  printf("Open sockman\n");
  sockmanfd = open_server_socket("ctrscrut",2200,"SOCKET SERVER");
	  
/* install signal handler */
  signal(SIGTERM, cleanExit);	
  signal(SIGINT, cleanExit);

  /* Initialize and set thread detached attribute */
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
  pthread_attr_getstacksize (&attr, &stacksize);

  printf("Default stack size = %d\n", stacksize);
  stacksize = sizeof(double)*N*N+MEGEXTRA;
  printf("Amount of stack needed per thread = %d\n",stacksize);
  pthread_attr_setstacksize (&attr, stacksize);
  printf("Creating threads with stack size = %d bytes\n",stacksize);

  for(t=0;t<threadNum;t++){
 
   
    printf("Contacting Sockman about Server: %s \n",thread_data_array[t].hostname);    


    if (thread_data_array[t].socknum != SPOOLFD) {
      message_struct tmpmsg = {
	SOCKET_MANAGER_GET_PROCESS,
	sizeof(thread_data_array[t].hostname),
	thread_data_array[t].hostname
      };

      if (0 == queryServer(sockmanfd, &tmpmsg)) {
       	readServer(sockmanfd, &sockMessage);
      }

      thread_data_array[t].port = processSocket.messages;
    }

    printf("Contacting Server: %s \n",thread_data_array[t].hostname);    

    openServer((void *) &thread_data_array[t]);
  
    thread_data_array[t].ptr = (void *) &thread_data_array;

    printf("Creating thread %d\n", t);

    rc = pthread_create(&threads[t], &attr, thread_data_array[t].func, 
			(void *) &thread_data_array[t]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }
  }
  printf("Close sockman\n");
  close(sockmanfd);

  /* Free attribute and wait for the other threads */
  pthread_attr_destroy(&attr);
  for(t=0; t<threadNum; t++) {
    rc = pthread_join(threads[t], (void **)&status);
    if (rc)
      {
	printf("ERROR; return code from pthread_join() is %d\n", rc);
	exit(-1);
      }
    printf("Completed join with thread %d status= %d\n",t, status);
  }

  for(i=0;i<threadNum;i++) close(thread_data_array[i].fd);
 
  pthread_exit(NULL);
}
