/*************************************************
*        Define ASCII Commands
*************************************************/
#define EN	0x4E45	/* End program		        */
#define IO	0x4F49	/* Pass	command to ioctl	*/
#define RD	0x4452	/* Send coadd frame		*/
#define CC      0x4343  /* Send coadd COUNT             */
#define AQ	0x5141	/* Config data acquisition	*/
#define RC      0x4352  /* Receive Config for IRCam     */
#define OK	0x4B4F	/* Message acknowledged 	*/
#define KO	0x4F4B	/* Message not acknowledged	*/
#define PS      0x5350  /* Send coadded PowSpec frame   */
#define SX      0x5853  /* Start Exposure               */
#define SB      0x4253  /* Set Picnic Bias              */
#define SC      0x4353  /* Set Controller               */
#define TB      0x4254  /* Take background frame        */
#define RB      0x4252  /* Reset Background frame       */
#define UB      0x4255  /* Use Background frame         */
#define OP      0x504F  /* send offsets to OPLE server  */
#define OF      0x464F  /* get offsets for OPLE server  */
#define FI      0x4946  /* send fringe info             */
#define GI      0x4947  /* receive gdt info from python */
#define AB      0x4241  /* abort exposure               */
#define XX      0x5858  /* stop exposure 		*/
#define CN      0x4E43  /* Change Name	                */
#define SD      0x4453  /* Toggle save data flag        */
#define FX      0x5846  /* Request FluxSeries           */
#define SP      0x5053  /* Send python config structure */
#define RP      0x5052  /* Receive py-config from socket*/
#define RO      0x4F52  /* Re-Open the OPLE/VME sockets */

#define OPLE "ople.chara-array.org"
#define MAX_SIZE 262144            /* 256x256x4 bytes */
#define STACK_SIZE 8192
#define STD_PRIORITY 1
#define HIGH_PRIORITY 99
#define MID_PRIORITY 50
#define TASK_MODE_0 T_FPU|T_CPU(0) /* Uses FPU, bound to CPU #0 */
#define TASK_MODE_1 T_FPU|T_CPU(1) /* Uses FPU, bound to CPU #1 */
#define PIPE_MINOR 0
#define BITS_PER_PIXEL 16
#define BUFFER_SIZE 134217728 * 8
#define MAXIMAGE 65536
#define MAXFRAMES 65536 // used to set size of structure for Flux Series.
#define MINOPDS 3
#define NUMDLS 6
#define NUMMEDIAN 3  //must be odd number
#define NUM_DELAY_LINES 6  // from /libs/charalib/chara.h (needed for vme control)
#define SOCKET_BUFFER 65536 // from /libs/charaui/src/charaui.h (needed for vme socket)

#define c_re(c) ((c)[0])
#define c_im(c) ((c)[1])

#define MAX_BOARD_ID 16
#define DAC_COUNT 6
#define DAC_OFFSET 2
#define DAC2 0 + DAC_OFFSET
#define DAC3 1 + DAC_OFFSET
#define DAC4 2 + DAC_OFFSET
#define DAC5 3 + DAC_OFFSET
#define DAC6 4 + DAC_OFFSET
#define DAC7 5 + DAC_OFFSET
#define LABEL_PREFIX "ARC42 DC Bias"
#define BOARD_ID "ARC42"
#define VOLTS2WORD 4095.0/5.0

#define TIMEOUT 10000000000 /* 10 second timeout used as idle execution for FSM */
                            /* 1 second gave us trouble in fiber explorer mode */
#define FSM_NUM_ROWS 4
#define FSM_NUM_COLS 4

#define GETCLOCK()  (long)   (  (double) (rt_timer_read()>>18) /3.814697265625 ) //returns time in ms.
#define MAX_ERROR_STRING	256

#define BASEPORT 0x378

char error_string[MAX_ERROR_STRING];


/** Possible states for the detector, as seen by the other tasks */
enum KEYSTATE
{
  S_IDLE,     /* Do Nothing            */
  S_INIT,     /* Init vars mem alloch  */
  S_RUN,      /* Run GDT               */
  S_BKGND     /* Take Background frame */
};
typedef enum KEYSTATE KEY_STATE;

enum COMMAND
{
  C_IDLE,     /* Switch to idle             */
  C_INIT,     /* switch to Init vars        */
  C_RUN,      /* Switch to GDT              */
  C_INITRUN   /* Do init, followed by run */
};
typedef enum COMMAND CMD;

KEY_STATE FSM[FSM_NUM_ROWS][FSM_NUM_COLS] = {
/*  C_IDLE     C_INIT     C_RUN     C_INITRUN             */
   {S_IDLE,    S_INIT,    S_RUN,    S_INIT } /* S_IDLE  */
  ,{S_IDLE,    S_IDLE,    S_RUN,    S_RUN} /* S_INIT  */
  ,{S_IDLE,    S_INIT,    S_RUN,    S_RUN } /* S_RUN   */
  ,{S_IDLE,    S_INIT,    S_RUN,    S_RUN} /* S_BKGND */
};

struct daqOut {
  float coadd[MAXIMAGE];  //256x256 max image
  int   coaddCount;       // Counter for use by external programs
  float bkgnd[MAXIMAGE];//256x256 background frame
  float pSpec2D[MAXIMAGE];//256x256 max pspec
  float opds[15];      //15  baselines   
  float offs[6];
  float fluxSeries[MAXFRAMES];
};

typedef struct daqOut daqOut_struct;

enum REFERENCE_TELESCOPE
{
  S1,
  S2,
  E1,
  E2,
  W1,
  W2
};
typedef enum REFERENCE_TELESCOPE REF_TEL;

/* This structure is used for inter-task communication */
struct exchange {
  unsigned int pixelCount;
  unsigned int byte_count_short; 
  float byte_count_float;
  boolean daqRunning;
  boolean endProg;
  boolean resetBkgnd;
  boolean takeBkgnd;
  boolean useBkgnd;
  CMD cmd;
  unsigned char bkgndNoiseCol;
  unsigned char cpeak[BASELINES];
  unsigned char cpeak0[BASELINES];
  unsigned char cpeak1[BASELINES];
  float gdtWeight[BASELINES];
  float gdtThreshold[BASELINES];
  float gdtOpd[BASELINES];
  float gdtOptOpd[MINOPDS];
  int OPDSigns[BASELINES];
  int BeamOrder[NUMTELS];
  double DLOffset[NUMDLS];
  boolean OffsetsReady;
  boolean manOffsetsReady;
/*   int RefTel; */
  REF_TEL RefTel;
  float lambda;
  float bandwidth;
  unsigned long framesPerBuffer;
  boolean abort;
  boolean continuousReadout;
  char image_filename[20];
  unsigned char rtpDev[32];
  boolean endTarget;
};
typedef struct exchange exchange_struct;

struct theDaq {
  unsigned short rows;
  unsigned short cols;
  unsigned short rowOffset;
  unsigned short colOffset;  
  unsigned short nreads;
  unsigned short numberOfCoadds;
  unsigned short framesPerReset;
  unsigned short exptime;
  unsigned long numberOfFrames;
  unsigned char byteSwapping;
  unsigned char do_reset;
  unsigned char do_deinterlace;
  unsigned char do_image_save;
  unsigned char daqRunning;
  unsigned char readToggle;
  float voffset;
  float vreset;  
  float vd;
  float ictl;
  char image_directory[30];
  char image_filename[20];
  unsigned char continuousReadout;
  unsigned short savenum;
  unsigned short numberCoherent;
};

/* This structure is used to communicate data to the python interface and with the spooler */
typedef struct theDaq daq_struct;

// typedef uint32_t OPLE_TIME; // also defined in dlTypes.h

void bubbleSort(float *a, unsigned int n);
void set_manual_target(int server, int dl, double pos);
void set_manual_offsets(int server, double *pos);
void sendTarget_task(void *cookie);
int nFramesPerBuffer(unsigned short *mem_fd, int bufferSize, int rows, int cols, int bits_per_pixel);
unsigned short *create_memory(int pci_fd, unsigned int bufferSize);
float add(unsigned short *src_fd, float *dest_fd, int n, int m);
void psAccum(float *src_fd, float *oldBuffer, float *im_fd, float *dest_fd, unsigned short r, unsigned short c, 
	     unsigned short nr, fftwf_plan plan, unsigned int pixelCount, fftwf_complex *fft, float *bkgndBuffer);
void linearFit(float *src_fd, float *dest_fd, 
	       unsigned short r,unsigned short c, unsigned short nr, unsigned short nca, unsigned int cac, int m);

void fluxQuad(float *src_fd, float flux, 
	      unsigned short r,unsigned short c, unsigned short nr, int quad);

void incrementFilename(char *filename);
int deinterlace(int cols, int rows, unsigned short *image_fd);
void swap_memory(unsigned short *mem_fd, int size);
void sem_task(void *cookie) ;
void clean_exit(int dummy);
void error(char *msg);

/* Routines borrowed from chara library */
void reverse_byte_order(void *p, int nbytes); // from rwfits.h
int open_connect_socket(char *machine, int port);
int char_waiting(int fd);
