/************************************************************************/
/* baytech.c								*/
/*                                                                      */
/* Test program for Baytech functions.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Jan 1999							*/
/************************************************************************/

#include "baytech.h"

/* baytech.c */

void close_function(void);
int main(int argc, char **argv);
int time_status(void);

/* A local global for the serial ports */

struct sserial rpc_port;
struct sserial m4_port;

void close_function(void) 
{
	call_close_baytech_rpc_port(0, NULL);
	call_close_baytech_m4_port(0, NULL);
}

int main(int argc, char **argv)
{
	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("BAYTECH INTERFACE TEST ROUTINE");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "BAYTECH VERSION 0.0";
	initialise_ui("menus.ini", "help.ini", "baytech");
	set_user_close_function(close_function);

	/* Setup background job(s) */

	background_add(time_status);

	rpc_port.fd = -1;
	m4_port.fd = -1;

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);
}

#define clean(y,x)	mvwaddstr(status_window,y,x,\
                                "                                       ")
/************************************************************************/
/* time_status()							*/
/* 									*/
/* Displays the time in the status window.				*/
/************************************************************************/

int time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	clean(0,0);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Local Time : ");
	wstandend(status_window);
	wprintw(status_window,"%2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* time_status() */
