/************************************************************************/
/* classicheader.c							*/
/*									*/
/* Program to read and print CLASSIC data file header.			*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : December 2006   						*/
/************************************************************************/

#include <stdio.h>
#include "chara.h"

int main(int argc, char **argv)
{
	struct s_classic_header	header;

	if (argc != 2)
	{
		fprintf(stderr,"Usage: %s filename.fit.\n", argv[0]);
		exit(-1);
	}

	if (read_classic_header(argv[1], &header) != 0)
	{
		fprintf(stderr,"Failed to open file %s\n", argv[1]);
		exit(-2);
	}

	printf("Star HD            : %s\n", header.hd);
	printf("Chara number       : %d\n", header.chara_number);
	printf("Object type        : %s\n", header.object);
	printf("UT date            : %s\n", header.ut_date);
	printf("Day of year        : %d\n", header.doy);
	printf("UT time            : %s\n", header.ut_time);
	printf("Scope beam 5       : %s\n", header.scope1);
	printf("Scope beam 6       : %s\n", header.scope2);
	printf("Azimuth            : %f\n", header.az);
	printf("Elevation          : %f\n", header.el);
	printf("Hour Angle         : %s\n", header.hour_angle);
	printf("UV Point           : %f %f\n", header.u, header.v);
	printf("Samples per scan   : %d\n", header.samples_per_scan);
	printf("Number of scans    : %d\n", header.number_of_scans);
	printf("Shutter A          : %d\n", header.shutter_a);
	printf("Shutter B          : %d\n", header.shutter_b);
	printf("Low SNR scans      : %d\n", header.lowsnr);
	printf("Skipped scans      : %d\n", header.nskip);
	printf("Data scans         : %d\n", header.number_data_scans);
	printf("Samples per second : %f\n", header.samples_per_second);
	printf("NIRO pixel size    : %d\n", header.pixel_size);
	printf("NIRO pixel row     : %d\n", header.pixel_row);
	printf("NIRO pixel cols    : %d %d\n",header.pixel_col1,
						header.pixel_col2);
	printf("NIRO reads         : %d\n", header.number_of_reads);
	printf("NIRO loops         : %d\n", header.number_of_loops);
	printf("NIRO reset boundary: %d\n", header.reset_boundary);
	printf("NIRO mode          : %s\n", header.readout_mode);
	printf("Dither State       : %s\n", header.dither_state);
	printf("Dither wavelength  : %f\n", header.lambda);
	printf("Dither range       : %f\n", header.dither_range);
	printf("Fringe frequency   : %f\n", header.fringe_frequency);

	exit(0);
}
