/************************************************************************/
/* uiend.c								*/
/*                                                                      */
/* Finish things off routine.                          	 		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"
#include "../../charalib/chara.h"

static int old_active_socket = -1;
extern char    sockman_name[256];

void (*user_close_function)(void) = NULL;

/************************************************************************/
/* ui_end()								*/
/* Process 14.10							*/
/*									*/
/* Stops the entire programme if the user wants it to.			*/
/* This is also the signal handler.  If the signal is SIGQUIT or 	*/
/* SIGTERM, stop without reference to the user!				*/
/* Returns NOERROR.							*/
/*									*/
/************************************************************************/

void	sig_end(int sig)
{
	if (active_socket != -1)
	{
		old_active_socket = active_socket;
		active_socket = -1;
	}
	ui_end(0, NULL);

} /* sig_end() */

int	ui_end(int argc, char **argv)
{
	char	filename[256];
	char	name[256];

	/* Reset the signal catcher for interupts */

	signal(SIGINT, sig_end);

	/* Make sure that the user is sure */

	if (argc > 1)
	{
		if (*argv[1]!='y' && *argv[1]!='Y')
		{
			if (old_active_socket != -1)
			{
				active_socket = old_active_socket;
				old_active_socket = -1;
			}
			return NOERROR;
		}
	}
	else if (ask_yes_no("Do you wish to exit the programme?","")
			==NO) 
	{
		if (old_active_socket != -1)
		{
			active_socket = old_active_socket;
			old_active_socket = -1;
		}
		return NOERROR;
	}

	/* Is there a user defined ending function call ? */

	if (argc > 2)
	{
		if (user_close_function != NULL && 
			(*argv[2] == 'y' || *argv[2] == 'Y'))
		{
			(*user_close_function)();
		}
	}
	else if (user_close_function != NULL)
	{
		(*user_close_function)();
	}

	/* Close down shop */

	sprintf(filename, "%s%s.port", ETC_DIRECTORY, get_server_name(name));
	unlink(filename);
	close_command_socket();
	close_data_socket();
	close_mirror_socket(0,NULL);
	if (*sockman_name != 0) socket_manager_remove_process(sockman_name);
	delete_scripts();
	clear();	/* Clear the screen */
	refresh();
	close_curses(); /* Shut down curses */
	send_server_stopping();

	exit(0);

} /* ui_end() */	

/************************************************************************/
/* set_user_close_function()						*/
/*									*/
/* Setup a function for the system to call when terminated. Good for	*/
/* proper cleanup of things.						*/
/************************************************************************/

void set_user_close_function(void (*function)(void))
{
	user_close_function = function;

} /* set_user_close_function() */
