/************************************************************************/
/* impulse_response.c 							*/
/*									*/
/* Calculates the impulse reposne of a filter. Should produce the filter*/
/* transfer function.							*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : December 1994						*/
/*									*/
/* Changes:								*/
/*									*/
/* 7/25/96 Removed refferences to fcvector as it is not really a part 	*/
/*         of NRC.							*/
/* 7/25/96 Added macro SCOPE_HEIGHT, SCOPE_X and SCOPE_Y for height 	*/
/*         and position of scope window.				*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "filter.h"
#include "../simpleX/simpleX.h"

#define SCOPE_HEIGHT 100 
#define SCOPE_X 50 
#define SCOPE_Y 50 

fcomplex *impulse_response(digital_filter filter, 
			   int *length, 
			   float **freqs,
			   char display)
{
	int 	i;
	float	*input;
	float	*output;
	fcomplex *FFT;
	float 	f6db;
	float	*values = NULL;
	scope	TheScope;

	/* Force length to be power of two... */

	printf("length = %d\n",*length);
	f6db = log((double)*length)/log(2.0);
	printf("Power float = %f\n",f6db);
        i = f6db;
        if ((float)i != f6db) i = i+1;
	printf("Power int = %d\n",i);
        *length = 1<<i;
	printf("length = %d\n",*length);

	/* Allocate memory */

	input  = vector(0,*length-1);
	FFT = (fcomplex *) vector(0,*length);

	*freqs = vector(0,*length/2);
	output = (float *)FFT;

	/* Create a `delta function' input array */

	input[0] = 1;
	for(i=1;i<*length;i++) input[i] = 0.0;

	/* Calculate the frequencies */

	(*freqs)[1] = 1.0 / (filter.dt * *length);
	for(i=0; i<= *length/2; i++) (*freqs)[i] = i * (*freqs)[1];

	/* Set up the Scope if required */

	if (display)
	{
		TheScope = open_scope("Impulse Response",SCOPE_X,SCOPE_Y,
					*length,SCOPE_HEIGHT);

		if (add_signal(&TheScope,65535,0,0,-0.2,1.2) != 1)
		{
			close_scope(&TheScope);
			quitX();
			exit(-2);
		}
                
		if (add_signal(&TheScope,0,0,65535,-0.2,1.2) != 2)
		{
			close_scope(&TheScope);
			quitX();
			exit(-2);
		}
		values = vector(1,2);
	}

	/* OK now loop around and send stuff through the filter */

	for(i=0;i<*length;i++)
	{
		output[i] = process_filter(filter, input[i]);

		if (display)
		{
			values[1] = input[i];
			values[2] = output[i];
			update_scope(TheScope,values);
		}
#ifdef DEBUG
		printf("%f %f\n",input[i],output[i]);
#endif
	}

	/* Calculate the FFT */

	realft(output-1,*length,1);

	/* NYQUIST AND DC TERM - SPECIAL CASES */

        FFT[*length/2].r = FFT[0].i;
        FFT[*length/2].i = 0.0;

        FFT[0].r = FFT[0].r;
        FFT[0].i = 0.0;

	/* Clean up */

	if (display)
	{
		close_scope(&TheScope);
		free_vector(values,1,2);
	}

	free_vector(input,0,*length-1);

	return FFT;

} /* impulse_response() */
