/************************************************************************/
/* impulsebp.c 								*/
/*									*/
/* Test driver for simple bp digital filter.				*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : December 1994						*/
/* Changes:                                                             */
/*                                                                      */
/* 7/25/96 Removed refferences to fcvector as it is not really a part   */
/*         of NRC.                                                      */
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "filter.h"

int main(int argc, char **argv)
{
	int 	j;
	float	f6db,dt;
	float	f0,e;
	digital_filter filter;
	fcomplex *response;
	fcomplex theory;
	int	length;
	float	*freqs;

	/* Check command line */

	if (argc < 5)
	{
		fprintf(stderr,"usage: impulsebp dt f0 e length\n");
		exit(-1);
	}

	sscanf(argv[1],"%f",&dt);
	sscanf(argv[2],"%f",&f0);
	sscanf(argv[3],"%f",&e);
	sscanf(argv[4],"%d",&length);

#ifdef DISPLAY
        if (!initX(NULL))
        {
                fprintf(stderr,"Failed to open display.\n");
                exit(-1);
        }
#endif
	/* Create the simple bp response */

	f6db = bandpass(&filter, dt, f0, e);

	/* Caluclate the response */
	
#ifdef DISPLAY
	response = impulse_response(filter,&length,&freqs,TRUE);
#else
	response = impulse_response(filter,&length,&freqs,FALSE);
#endif 

	/* Print out the results */

	printf("# FINAL RESULTS\n#\n");
	printf("# f0   = %f\n",f0);
	printf("# e    = %f\n",e);
	printf("#   f         Mag(T)     Phase(T)    Mag(Resp)  Phase(Resp)\n");

	for(j=0; j<=length/2; j++)
	{
		theory = bandpass_response(freqs[j],f0,e,dt);

		printf("%10.3e  %10.3e  %10.3e  %10.3e  %10.3e\n",
		freqs[j],
		mag_db(theory),phase(theory),
		mag_db(response[j]),-phase(response[j]));
	}
		
	/* Clean up */

#ifdef DISPLAY
	quitX();
#endif

        free_vector((float*)response,0,length);
	free_vector(freqs,0,length/2);
	clean_up_filter(filter);
	exit(0);

} /* main() */
