/************************************************************************/
/* calibration.c	 						*/
/*									*/
/* Routines for doing calibration of data.				*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* ZERNIKE CODE LIBRARY.						*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : July 2004    						*/
/************************************************************************/

#include <math.h>
#include <float.h>
#include <values.h>
#include <nrc.h>
#include "interferometry.h"

/************************************************************************/
/* calibrate_v()							*/
/*									*/
/* Does the simple minded calibration thing.				*/
/************************************************************************/

void   calibrate_v(float lambda, float dlambda,  /* Observing band (m)*/
	    float csize, /* Radian */
            float c1v, float c1err, 
		float c1t, float c1b, /* Cal1 V, time and baseline (m)*/
            float ov, float oerr, float ot, /* Obj V, and time */
            float c2v, float c2err, 
		float c2t, float c2b,
	float *v, float *err) /* Cal1 V, time and baseline (m)*/
{
	float	m, c;
	float	t;

	/* First we adjust for the size of the calibrator */

	c1v /= v_broadband(UNIFORM_DISK, c1b, lambda, dlambda, csize);
	c2v /= v_broadband(UNIFORM_DISK, c2b, lambda, dlambda, csize);

	/* Now do the interpolation */

	m = (c1v - c2v)/(c1t - c2t);
	c = c2v - m * c2t;
	t = m * ot + c;

	/* and the vis extimator is....*/

	*v = ov / t;
	*err = sqrt(c1err*c1err + c2err*c2err + oerr*oerr);

} /* calibrate_v() */

/************************************************************************/
/* calibrate_v2()							*/
/*									*/
/* Does the simple minded calibration thing for v2.			*/
/************************************************************************/

void   calibrate_v2(float lambda, float dlambda,  /* Observing band (m)*/
	    float csize, /* Radian */
            float c1v2, float c1err, 
		float c1t, float c1b, /* Cal1 V, time and baseline (m)*/
            float ov2, float oerr, float ot, /* Obj V, and time */
            float c2v2, float c2err, 
		float c2t, float c2b,
	float *v2, float *err) /* Cal1 V, time and baseline (m)*/
{
	float	m, c;
	float	t;

	/* First we adjust for the size of the calibrator */

	c1v2 /= (v_broadband(UNIFORM_DISK, c1b, lambda, dlambda, csize) *
		 v_broadband(UNIFORM_DISK, c1b, lambda, dlambda, csize));
	c2v2 /= (v_broadband(UNIFORM_DISK, c2b, lambda, dlambda, csize) *
		 v_broadband(UNIFORM_DISK, c2b, lambda, dlambda, csize));

	/* Now do the interpolation */

	m = (c1v2 - c2v2)/(c1t - c2t);
	c = c2v2 - m * c2t;
	t = m * ot + c;

	/* That should be all */

	*v2 = ov2 / t;
	*err = sqrt(c1err*c1err + c2err*c2err + oerr*oerr);

} /* calibrate_v2() */
