/*
 * Routine to test non linear least squares fit routine.
 */

#include <stdio.h>
#include <stdlib.h>
#include "nrc.h"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif 

void	funct_deriv(float *ftn, float *var, float *param);
void	funct_noderiv(float *ftn, float *var, float *param);

#define NDATA 3
#define SHOW  1

int main(int argc, char **argv)
{
	float	**param;
	float	*var;
	float	*data;
	float	rmsres,maxres;
	float	a,b,fa,fb;
	int 	maxit;
	int	i;

	if (argc < 6)
	{
		fprintf(stderr,"usage: testfit a b fa fb nit\n");
		exit(-1);
	}

	sscanf(argv[1],"%f",&a);
	sscanf(argv[2],"%f",&b);
	sscanf(argv[3],"%f",&fa);
	sscanf(argv[4],"%f",&fb);
	sscanf(argv[5],"%d",&maxit);

	/* Allocate memory */

	param = matrix(1,NDATA,1,1);
	var = vector(1,2);
	data = vector(1,NDATA);

	/* Create data */

	var[1] = a;
	var[2] = b;
	for(i=1;i<=NDATA;i++)
	{
		param[i][1] = i;
		data[i] = var[1]*i*i + var[2]*i;
		printf("%f\t%f\n",param[i][1],data[i]);
	}

	printf("Doing fit with calculated derivatives\n");

	/* Do the fit */

	var[1] = fa;
	var[2] = fb;
	i = non_linear_fit(2, var, NDATA, data, 1, param, TRUE,
			funct_deriv, &rmsres, &maxres, 1.0, maxit, SHOW);

	printf("RESULTS: nit = %d var[1] = %f var[2] = %f\n",i,var[1],var[2]);
	printf("Maxres = %f, RMSres = %f\n",maxres,rmsres);

        printf("Doing fit with approximated derivatives\n");


        /* Do the fit */

	var[1] = fa;
	var[2] = fb;
        i = non_linear_fit(2, var, NDATA, data, 1, param, FALSE, 
			funct_noderiv, &rmsres, &maxres, 1.0, maxit, SHOW);

        printf("RESULTS: nit = %d var[1] = %f var[2] = %f\n",i,var[1],var[2]);
	printf("Maxres = %f, RMSres = %f\n",maxres,rmsres);


	free_matrix(param,1,NDATA,1,2);
	free_vector(var,1,2);
	free_vector(data,1,NDATA);

	exit(0);
}

/************************************************************************/
/* funct_deriv()								*/
/*									*/
/* Calculates function value and derivatives.				*/
/************************************************************************/

void	funct_deriv(float *ftn, float *var, float *param)

{
	ftn[0] = var[1]*param[1]*param[1] + var[2]*param[1];
	ftn[1] = param[1]*param[1];
	ftn[2] = param[1];

#ifdef DEBUG
	printf("funct: %f %f %f %f %f %f %f\n",
		var[1],var[2],param[1],param[2],ftn[0],ftn[1],ftn[2]);
#endif
}

/************************************************************************/
/* funct_noderiv()							*/
/*									*/
/* Calculates function value and derivatives.				*/
/************************************************************************/

void	funct_noderiv(float *ftn, float *var, float *param)

{
	ftn[0] = var[1]*param[1]*param[1] + var[2]*param[1];

#ifdef DEBUG
	printf("funct: %f %f %f %f %f\n",
		var[1],var[2],param[1],param[2],ftn[0]);
#endif
}
