import serial
import common
import time
import struct
import numarray as na
import string

from os import O_WRONLY, O_TRUNC, O_CREAT
from stat import S_IRUSR, S_IWUSR, S_IRGRP, S_IWGRP

class luminosSerial:
    def __init__(self):
        self.configure()

    def configure(self):
        self.ser = None
        #Open named port at "common.luminosBaud,8,N,1", 1s timeout
        self.ser = serial.Serial(common.luminosPort, common.luminosBaud, timeout=1)
        if self.ser is None:
            print 'could not open serial port ', common.luminosPort
            return -1
        self.ser.flushInput()
        return 1

    def restart(self):
        if self.ser is None:
            print 'serial port not open', common.luminosPort
            return -1
        outbuf = chr(0)*6
        self.ser.flushInput()
        self.ser.write(outbuf)
        time.sleep(2)
        return 1
        
    def renumber(self):
        if self.ser is None:
            print 'serial port not open', common.luminosPort
            return -1
        list_outbuf = [chr(0)]*6        
        list_outbuf[1] = chr(2)
        
        inbuf = chr(1)*6
        outbuf = string.join(list_outbuf, '')
        
        self.ser.flushInput()
        self.ser.write(outbuf)
        time.sleep(2)
        
        list_outbuf[1] = chr(60)
        stages = 0
        
        while inbuf[1]!= chr(0):
            list_outbuf[0] = chr(3*stages+1)
            inbuf = chr(0)*6            
            self.ser.flushInput()

            outbuf = string.join(list_outbuf, '')
            self.ser.write(outbuf)
            
            self.ser.flushInput()
            
            inbuf = self.ser.read(6)
            
            if inbuf[1] != chr(0):
                stages+=1
        return stages
    
    def save_position(self, path, stages):
        if self.ser is None:
            print 'serial port not open', common.luminosPort
            return -1
        try: pos = os.open(path, O_WRONLY | O_TRUNC | O_CREAT)
        except:
            print 'Please choose another file name'
            return -1
        else:
            #set permitions
            os.chmod(path , S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP )
            for h in range(1, stages+1): #1, ..., stages
                for i in range(1, 4): #1, ..., 3
                    position = self.absolute_position(h, i)
                    line = str(position)
                    os.write(pos, line)
            os.close(pos)
            print "Luminos positions have been saved to %s.\n"%path
            return 1
            
    def recall_position(self, path, stages):
## Not sure why Phil does a configure here 
##         if self.configure() < 0:
##             print 'Could not open RS-232 interface.  Reset permissions on %s.\n' %common.luminosPort
##             return -1
        if self.ser is None:
            print 'serial port not open', common.luminosPort
            return -1
        try: pos = open(path, 'r')
        except:
            print 'File name invalid'
            return -1
        else:
            for h in range(1, stages+1): #1, ..., stages
                for i in range(1, 4): #1, ..., 3
                    line = pos.readline()
                    li = int(line)
                    self.move_absolute(h, i, li)
            pos.close()
            print 'Luminos stages have been moved to the positions specified by %s.\n' %path
            return 1

    def set_position(self, stage, axis, position):
        if self.ser is None:
            print 'Please configure serial port %s first with configure()' %common.luminosPort
            return -1
        device = 3*stage - 3 + axis
        command = 45
        list_outbuf = [chr(0)]*6
        inbuf = chr(0)*6
        list_outbuf[0] = chr(device)
        list_outbuf[1] = chr(command)
        output_data = struct.pack('l', position)
        list_outbuf[2:] = output_data
        outbuf = string.join(list_outbuf, '')
        self.ser.write(outbuf)
        self.ser.flushInput()
        if device == 0:
            print 'Please use save_position(filename) to save all axes postions to a file'
            return -1
        else:
            inbuf = self.ser.read(6)
        if (inbuf[0],inbuf[1]) == (chr(0),chr(0)):
            print 'Requested actuor did not reply'  
            return -1
        input_data = inbuf[2:]
        returned_data = struct.unpack('l', input_data)[0]
        if inbuf[1] == chr(255):
            print 'Requested position setting of stage %i and axis %i is out of range.'%(stage,axis)
            return -1
        return 1

    def command(self, device, cmnd, data):
        if self.ser is None:
            print 'Please configure serial port %s first with configure()' %common.luminosPort
            return -1
        list_outbuf = [chr(0)]*6
        inbuf = chr(0)*6
        list_outbuf[0] = chr(device)
        list_outbuf[1] = chr(cmnd)
        output_data = struct.pack('l', data)
        list_outbuf[2:] = output_data
        outbuf = string.join(list_outbuf, '')
        self.ser.write(outbuf)
        self.ser.flushInput()
        inbuf = self.ser.read(6)  
        if (inbuf[0],inbuf[1]) == (chr(0),chr(0)):
            print 'Requested actuor did not reply'  
            return -1
        input_data = inbuf[2:]
        returned_data = struct.unpack('l', input_data)[0]
        returned_device = struct.unpack('b', inbuf[0])[0]
        returned_command = struct.unpack('b', inbuf[1])[0]
        print 'Returned signal is: \n %i %i %li\n',%(returned_device,returned_command,returned_data)
        return 1
    
    def home(self, stage, axis):
        if self.ser is None:
            print 'Please configure serial port %s first with configure()' %common.luminosPort
            return -1
        list_outbuf = [chr(0)]*6
        list_outbuf[1] = chr(1)
        if stage != 0:
            if axis == 0:
                for i in range(1, 4):
                    list_outbuf[0] = chr(3*(stage - 1)+i)
                    outbuf = string.join(list_outbuf, '')
                    self.ser.write(outbuf)
                    inbuf = self.ser.read(6)
                return 1
            else:
                list_outbuf[0] = chr(3*(stage - 1)+axis)
        self.ser.write(outbuf)
        inbuf = self.ser.read(6)
        return 1

    def absolute_position(self, stage, axis):
        if self.ser is None:
            print 'Please configure serial port %s first with configure()' %common.luminosPort
            return -1
        inbuf = chr(0)*6
        list_outbuf = [chr(0)]*6
        list_outbuf[0] = chr(3*stage-3+axis)
        list_outbuf[1] = chr(60)
        outbuf = string.join(list_outbuf, '')
        self.ser.write(outbuf)
        self.ser.flushInput()
        inbuf = self.ser.read(6)
        if (inbuf[0],inbuf[1]) == (chr(0),chr(0)):
            print 'Requested actuor did not reply'  
            return -1
        input_data = inbuf[2:]
        returned_data = struct.unpack('l', input_data)[0]
        return returned_data

    def move_absolute(self, stage, axis, position):
        if self.ser is None:
            print 'Please configure serial port %s first with configure()' %common.luminosPort
            return -1
        device = 3*stage-3+axis
        command = 20
        inbuf = chr(0)*6
        list_outbuf = [chr(0)]*6
        list_outbuf[0] = chr(device)
        list_outbuf[1] = chr(command)
        output_data = struct.pack('l', position)
        list_outbuf[2:] = output_data
        outbuf = string.join(list_outbuf, '')
        self.ser.write(outbuf)
        self.ser.flushInput()
        inbuf = self.ser.read(6)
        if (inbuf[0],inbuf[1]) == (chr(0),chr(0)):
            print 'Requested actuor did not reply'  
            return -1
        input_data = inbuf[2:]
        returned_data = struct.unpack('l', input_data)[0]
        if inbuf[1] == chr(255):
            print 'Requested position setting of stage %i and axis %i is out of range.'%(stage,axis)
            return -1
        return 1
        
    def move_relative(self, stage, axis, position):
        if self.ser is None:
            print 'Please configure serial port %s first with configure()' %common.luminosPort
            return -1
        device = 3*stage-3+axis
        command = 21
        inbuf = chr(0)*6
        list_outbuf = [chr(0)]*6
        list_outbuf[0] = chr(device)
        list_outbuf[1] = chr(command)
        output_data = struct.pack('l', position)
        list_outbuf[2:] = output_data
        outbuf = string.join(list_outbuf, '')
        self.ser.write(outbuf)
        self.ser.flushInput()
        inbuf = self.ser.read(6)
        if (inbuf[0],inbuf[1]) == (chr(0),chr(0)):
            print 'Requested actuor did not reply'  
            return -1
        input_data = inbuf[2:]
        returned_data = struct.unpack('l', input_data)[0]
        if inbuf[1] == chr(255):
            print 'Requested position setting of stage %i and axis %i is out of range.'%(stage,axis)
            return -1
        return 1
